@@ -15,19 +15,28 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
+#include <math.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
 #include "support.h"
 
+/* common */
 #include "city.h"
 #include "game.h"
 #include "map.h"
+#include "specialist.h"
 #include "unitlist.h"
 
+/* agents */
 #include "cma_fec.h"
+
+/* client */
+#include "citydlg_common.h" /* city_production_cost_str() */
 #include "options.h"
 
 #include "cityrepdata.h"
@@ -48,11 +57,20 @@ static const char *cr_entry_cityname(const struct city *pcity,
   return city_name(pcity);
 }
 
+/************************************************************************
+  Translated name of nation who owns this city.
+*************************************************************************/
+static const char *cr_entry_nation(const struct city *pcity,
+                                   const void *data)
+{
+  return nation_adjective_for_player(city_owner(pcity));
+}
+
 static const char *cr_entry_size(const struct city *pcity,
 				 const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->size);
+  fc_snprintf(buf, sizeof(buf), "%2d", pcity->size);
   return buf;
 }
 
@@ -60,8 +78,9 @@ static const char *cr_entry_hstate_concise(const struct city *pcity,
 					   const void *data)
 {
   static char buf[4];
-  my_snprintf(buf, sizeof(buf), "%s", (city_celebrating(pcity) ? "*" :
-				       (city_unhappy(pcity) ? "X" : " ")));
+  fc_snprintf(buf, sizeof(buf), "%s",
+              (city_celebrating(pcity) ? "*"
+               : (city_unhappy(pcity) ? "X" : " ")));
   return buf;
 }
 
@@ -69,10 +88,10 @@ static const char *cr_entry_hstate_verbose(const struct city *pcity,
 					   const void *data)
 {
   static char buf[16];
-  my_snprintf(buf, sizeof(buf), "%s",
-	      (city_celebrating(pcity) ? Q_("?city_state:Rapture") :
-	       (city_unhappy(pcity) ? Q_("?city_state:Disorder") :
-		Q_("?city_state:Peace"))));
+  fc_snprintf(buf, sizeof(buf), "%s",
+              (city_celebrating(pcity) ? Q_("?city_state:Celebrating")
+               : (city_unhappy(pcity) ? Q_("?city_state:Disorder")
+                  : Q_("?city_state:Peace"))));
   return buf;
 }
 
@@ -80,11 +99,11 @@ static const char *cr_entry_workers(const struct city *pcity,
 				    const void *data)
 {
   static char buf[32];
-  my_snprintf(buf, sizeof(buf), "%d/%d/%d/%d",
-	      pcity->feel[CITIZEN_HAPPY][FEELING_FINAL],
-	      pcity->feel[CITIZEN_CONTENT][FEELING_FINAL],
-	      pcity->feel[CITIZEN_UNHAPPY][FEELING_FINAL],
-	      pcity->feel[CITIZEN_ANGRY][FEELING_FINAL]);
+  fc_snprintf(buf, sizeof(buf), "%d/%d/%d/%d",
+              pcity->feel[CITIZEN_HAPPY][FEELING_FINAL],
+              pcity->feel[CITIZEN_CONTENT][FEELING_FINAL],
+              pcity->feel[CITIZEN_UNHAPPY][FEELING_FINAL],
+              pcity->feel[CITIZEN_ANGRY][FEELING_FINAL]);
   return buf;
 }
 
@@ -92,7 +111,8 @@ static const char *cr_entry_happy(const struct city *pcity,
 				  const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->feel[CITIZEN_HAPPY][FEELING_FINAL]);
+  fc_snprintf(buf, sizeof(buf), "%2d",
+              pcity->feel[CITIZEN_HAPPY][FEELING_FINAL]);
   return buf;
 }
 
@@ -100,7 +120,8 @@ static const char *cr_entry_content(const struct city *pcity,
 				    const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->feel[CITIZEN_CONTENT][FEELING_FINAL]);
+  fc_snprintf(buf, sizeof(buf), "%2d",
+              pcity->feel[CITIZEN_CONTENT][FEELING_FINAL]);
   return buf;
 }
 
@@ -108,7 +129,8 @@ static const char *cr_entry_unhappy(const struct city *pcity,
 				    const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->feel[CITIZEN_UNHAPPY][FEELING_FINAL]);
+  fc_snprintf(buf, sizeof(buf), "%2d",
+              pcity->feel[CITIZEN_UNHAPPY][FEELING_FINAL]);
   return buf;
 }
 
@@ -116,7 +138,8 @@ static const char *cr_entry_angry(const struct city *pcity,
 				  const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->feel[CITIZEN_ANGRY][FEELING_FINAL]);
+  fc_snprintf(buf, sizeof(buf), "%2d",
+              pcity->feel[CITIZEN_ANGRY][FEELING_FINAL]);
   return buf;
 }
 
@@ -132,7 +155,8 @@ static const char *cr_entry_specialist(const struct city *pcity,
   static char buf[8];
   const struct specialist *sp = data;
 
-  my_snprintf(buf, sizeof(buf), "%2d", pcity->specialists[sp->index]);
+  fc_snprintf(buf, sizeof(buf), "%2d",
+              pcity->specialists[specialist_index(sp)]);
   return buf;
 }
 
@@ -210,7 +234,7 @@ static const char *cr_entry_supported(const struct city *pcity,
     num_supported++;
   } unit_list_iterate_end;
 
-  my_snprintf(buf, sizeof(buf), "%2d", num_supported);
+  fc_snprintf(buf, sizeof(buf), "%2d", num_supported);
 
   return buf;
 }
@@ -225,7 +249,7 @@ static const char *cr_entry_present(const struct city *pcity,
     num_present++;
   } unit_list_iterate_end;
 
-  my_snprintf(buf, sizeof(buf), "%2d", num_present);
+  fc_snprintf(buf, sizeof(buf), "%2d", num_present);
 
   return buf;
 }
@@ -234,10 +258,10 @@ static const char *cr_entry_resources(const struct city *pcity,
 				      const void *data)
 {
   static char buf[32];
-  my_snprintf(buf, sizeof(buf), "%d/%d/%d",
-	      pcity->surplus[O_FOOD], 
-	      pcity->surplus[O_SHIELD], 
-	      pcity->surplus[O_TRADE]);
+  fc_snprintf(buf, sizeof(buf), "%d/%d/%d",
+              pcity->surplus[O_FOOD],
+              pcity->surplus[O_SHIELD],
+              pcity->surplus[O_TRADE]);
   return buf;
 }
 
@@ -245,8 +269,7 @@ static const char *cr_entry_foodplus(const struct city *pcity,
 				     const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d",
-	      pcity->surplus[O_FOOD]);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->surplus[O_FOOD]);
   return buf;
 }
 
@@ -254,8 +277,7 @@ static const char *cr_entry_prodplus(const struct city *pcity,
 				     const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d",
-	      pcity->surplus[O_SHIELD]);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->surplus[O_SHIELD]);
   return buf;
 }
 
@@ -263,8 +285,7 @@ static const char *cr_entry_tradeplus(const struct city *pcity,
 				      const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d",
-	      pcity->surplus[O_TRADE]);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->surplus[O_TRADE]);
   return buf;
 }
 
@@ -274,10 +295,8 @@ static const char *cr_entry_output(const struct city *pcity,
   static char buf[32];
   int goldie = pcity->surplus[O_GOLD];
 
-  my_snprintf(buf, sizeof(buf), "%3d/%d/%d",
-	      goldie,
-	      pcity->prod[O_LUXURY],
-	      pcity->prod[O_SCIENCE]);
+  fc_snprintf(buf, sizeof(buf), "%3d/%d/%d",
+              goldie, pcity->prod[O_LUXURY], pcity->prod[O_SCIENCE]);
   return buf;
 }
 
@@ -287,9 +306,9 @@ static const char *cr_entry_gold(const struct city *pcity,
   static char buf[8];
 
   if (pcity->surplus[O_GOLD] > 0) {
-    my_snprintf(buf, sizeof(buf), "+%d", pcity->surplus[O_GOLD]);
+    fc_snprintf(buf, sizeof(buf), "+%d", pcity->surplus[O_GOLD]);
   } else {
-    my_snprintf(buf, sizeof(buf), "%3d", pcity->surplus[O_GOLD]);
+    fc_snprintf(buf, sizeof(buf), "%3d", pcity->surplus[O_GOLD]);
   }
   return buf;
 }
@@ -298,8 +317,7 @@ static const char *cr_entry_luxury(const struct city *pcity,
 				   const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d",
-	      pcity->prod[O_LUXURY]);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->prod[O_LUXURY]);
   return buf;
 }
 
@@ -307,8 +325,7 @@ static const char *cr_entry_science(const struct city *pcity,
 				    const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d",
-	      pcity->prod[O_SCIENCE]);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->prod[O_SCIENCE]);
   return buf;
 }
 
@@ -321,15 +338,13 @@ static const char *cr_entry_growturns(const struct city *pcity,
 
   if (turns == FC_INFINITY) {
     /* 'never' wouldn't be easily translatable here. */
-    my_snprintf(buffer, sizeof(buffer), "---");
+    fc_snprintf(buffer, sizeof(buffer), "---");
   } else {
     /* Shrinking cities get a negative value. */
-    my_snprintf(buffer, sizeof(buffer), "%4d", turns);
+    fc_snprintf(buffer, sizeof(buffer), "%4d", turns);
   }
-  my_snprintf(buf, sizeof(buf), "%s (%d/%d)",
-	      buffer,
-	      pcity->food_stock,
-	      city_granary_size(pcity->size) );
+  fc_snprintf(buf, sizeof(buf), "%s (%d/%d)",
+              buffer, pcity->food_stock, city_granary_size(pcity->size));
   return buf;
 }
 
@@ -337,15 +352,36 @@ static const char *cr_entry_pollution(const struct city *pcity,
 				      const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d", pcity->pollution);
+  fc_snprintf(buf, sizeof(buf), "%3d", pcity->pollution);
   return buf;
 }
 
-static const char *cr_entry_num_trade(const struct city *pcity,
-				      const void *data)
+static const char *cr_entry_trade_routes(const struct city *pcity,
+                                         const void *data)
+{
+  static char buf[16];
+  int num = 0, value = 0, i;
+
+  for (i = 0; i < NUM_TRADE_ROUTES; i++) {
+    if (0 != pcity->trade[i]) {
+      num++;
+      value += pcity->trade_value[i];
+    }
+  }
+
+  if (0 == num) {
+    sz_strlcpy(buf, "0");
+  } else {
+    fc_snprintf(buf, sizeof(buf), "%d (+%d)", num, value);
+  }
+  return buf;
+}
+
+static const char *cr_entry_build_slots(const struct city *pcity,
+                                        const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%d", city_num_trade_routes(pcity));
+  fc_snprintf(buf, sizeof(buf), "%3d", city_build_slots(pcity));
   return buf;
 }
 
@@ -356,27 +392,17 @@ static const char *cr_entry_building(const struct city *pcity,
   const char *from_worklist =
     worklist_is_empty(&pcity->worklist) ? "" :
     concise_city_production ? "+" : _("(worklist)");
-	
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
-    my_snprintf(buf, sizeof(buf), "%s (%d)%s",
-		get_impr_name_ex(pcity, pcity->production.value),
-		MAX(0, pcity->surplus[O_SHIELD]), from_worklist);
+
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    fc_snprintf(buf, sizeof(buf), "%s (%d)%s",
+                city_production_name_translation(pcity),
+                MAX(0, pcity->surplus[O_SHIELD]), from_worklist);
   } else {
-    const char *name;
-    int cost;
-
-    if(pcity->production.is_unit) {
-      name = utype_name_translation(utype_by_number(pcity->production.value));
-      cost = unit_build_shield_cost(utype_by_number(pcity->production.value));
-    } else {
-      name = get_impr_name_ex(pcity, pcity->production.value);
-      cost = impr_build_shield_cost(pcity->production.value);
-    }
-
-    my_snprintf(buf, sizeof(buf), "%s (%d/%d)%s", name,
-		pcity->shield_stock, cost,
-		from_worklist);
+    fc_snprintf(buf, sizeof(buf), "%s (%d/%s)%s",
+                city_production_name_translation(pcity),
+                pcity->shield_stock,
+                city_production_cost_str(pcity),
+                from_worklist);
   }
 
   return buf;
@@ -385,29 +411,30 @@ static const char *cr_entry_building(const struct city *pcity,
 static const char *cr_entry_build_cost(const struct city *pcity,
 				  const void *data)
 {
-  int price = city_buy_cost(pcity);
-  int turns = city_turns_to_build(pcity, pcity->production, TRUE);
   char bufone[8];
   char buftwo[8];
   static char buf[32];
+  int price;
+  int turns;
 
-  if (!pcity->production.is_unit
-      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
-    my_snprintf(buf, sizeof(buf), "*");
+  if (city_production_has_flag(pcity, IF_GOLD)) {
+    fc_snprintf(buf, sizeof(buf), "*");
     return buf;
   }
+  price = city_production_buy_gold_cost(pcity);
+  turns = city_production_turns_to_build(pcity, TRUE);
 
   if (price > 99999) {
-    my_snprintf(bufone, sizeof(bufone), "---");
+    fc_snprintf(bufone, sizeof(bufone), "---");
   } else {
-    my_snprintf(bufone, sizeof(bufone), "%d", price);
+    fc_snprintf(bufone, sizeof(bufone), "%d", price);
   }
   if (turns > 999) {
-    my_snprintf(buftwo, sizeof(buftwo), "--");
+    fc_snprintf(buftwo, sizeof(buftwo), "--");
   } else {
-    my_snprintf(buftwo, sizeof(buftwo), "%3d", turns);
+    fc_snprintf(buftwo, sizeof(buftwo), "%3d", turns);
   }
-  my_snprintf(buf, sizeof(buf), "%s/%s", buftwo, bufone);
+  fc_snprintf(buf, sizeof(buf), "%s/%s", buftwo, bufone);
   return buf;
 }
 
@@ -415,7 +442,7 @@ static const char *cr_entry_corruption(const struct city *pcity,
 				       const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d", -(pcity->waste[O_TRADE]));
+  fc_snprintf(buf, sizeof(buf), "%3d", -(pcity->waste[O_TRADE]));
   return buf;
 }
 
@@ -423,7 +450,20 @@ static const char *cr_entry_waste(const struct city *pcity,
 				  const void *data)
 {
   static char buf[8];
-  my_snprintf(buf, sizeof(buf), "%3d", -(pcity->waste[O_SHIELD]));
+  fc_snprintf(buf, sizeof(buf), "%3d", -(pcity->waste[O_SHIELD]));
+  return buf;
+}
+
+static const char *cr_entry_plague_risk(const struct city *pcity,
+                                        const void *data)
+{
+  static char buf[8];
+  if (!game.info.illness_on) {
+    fc_snprintf(buf, sizeof(buf), " -.-");
+  } else {
+    fc_snprintf(buf, sizeof(buf), "%4.1f",
+                (float)city_illness_calc(pcity, NULL, NULL, NULL, NULL)/10.0);
+  }
   return buf;
 }
 
@@ -445,19 +485,15 @@ static const char *cr_entry_cma(const struct city *pcity,
 static const struct city_report_spec base_city_report_specs[] = {
   { TRUE, -15, 0, NULL,  N_("?city:Name"), N_("City Name"),
     NULL, FUNC_TAG(cityname) },
+  { FALSE, -15, 0, NULL, N_("Nation"),  N_("Nation"),
+    NULL, FUNC_TAG(nation) },
   { TRUE,   2, 1, NULL,  N_("?size [short]:Sz"), N_("Size"),
     NULL, FUNC_TAG(size) },
-  { TRUE,  -8, 1, NULL,  N_("State"),   N_("Rapture/Peace/Disorder"),
+  { TRUE,  -8, 1, NULL,  N_("State"),   N_("Celebrating/Peace/Disorder"),
     NULL, FUNC_TAG(hstate_verbose) },
-  { FALSE,  1, 1, NULL,  NULL,          N_("Concise *=Rapture, X=Disorder"),
+  { FALSE,  1, 1, NULL,  NULL,          N_("Concise *=Celebrating, X=Disorder"),
     NULL, FUNC_TAG(hstate_concise) },
 
-  /* Specialists grouped with init_city_report_game_data specialists */ 
-  { FALSE,  7, 1, N_("Special"),
-    N_("?entertainers/scientists/taxmen:E/S/T"),
-    N_("Entertainers, Scientists, Taxmen"),
-    NULL, FUNC_TAG(specialists) },
-
   { FALSE, 2, 1, NULL, N_("?Happy workers:H"), N_("Workers: Happy"),
     NULL, FUNC_TAG(happy) },
   { FALSE, 2, 1, NULL, N_("?Content workers:C"), N_("Workers: Content"),
@@ -515,13 +551,19 @@ static const struct city_report_spec base_city_report_specs[] = {
     NULL, FUNC_TAG(science) },
   { FALSE,  1, 1, N_("?number_trade_routes:n"), N_("?number_trade_routes:R"),
                                          N_("Number of Trade Routes"),
-    NULL, FUNC_TAG(num_trade) },
+    NULL, FUNC_TAG(trade_routes) },
   { FALSE,  3, 1, NULL, N_("?pollution [short]:Pol"), N_("Pollution"),
     NULL, FUNC_TAG(pollution) },
+  { FALSE,  4, 1, N_("?plague risk [short]:Pla"), N_("(%)"), N_("Plague risk"),
+    NULL, FUNC_TAG(plague_risk) },
   { FALSE, 15, 1, NULL, N_("?cma:Governor"), N_("Citizen Governor"),
     NULL, FUNC_TAG(cma) },
 
+  /* TRANS: "BS" = "build slots" */
+  { FALSE,  3, 1, NULL, N_("BS"), N_("Maximum units buildable per turn"),
+    NULL, FUNC_TAG(build_slots) },
   { TRUE,   9, 1, N_("Production"), N_("Turns/Buy"),
+  /*N_("Turns or gold to complete production"), future menu needs translation */
     N_("Production"),
     NULL, FUNC_TAG(build_cost) },
   { TRUE,   0, 1, N_("Currently Building"),
@@ -550,36 +592,61 @@ const char *city_report_spec_tagname(int i)
 }
 
 /******************************************************************
-  Initialize city report data.  Currently all this does is
-  pre-translate the fields (to make things easier on the GUI
-  writers).  Should be called before the GUI starts up.
+  Initialize city report data.  This deals with ruleset-depedent
+  columns and pre-translates the fields (to make things easier on
+  the GUI writers).  Should be called before the GUI starts up.
 ******************************************************************/
 void init_city_report_game_data(void)
 {
-  static char explanation[SP_MAX][128];
+  static char sp_explanation[SP_MAX][128];
+  static char sp_explanations[SP_MAX*128];
   struct city_report_spec *p;
   int i;
 
-  num_creport_cols = ARRAY_SIZE(base_city_report_specs) + SP_COUNT;
+  num_creport_cols = ARRAY_SIZE(base_city_report_specs)
+                   + specialist_count() + 1;
   city_report_specs
     = fc_realloc(city_report_specs,
 		 num_creport_cols * sizeof(*city_report_specs));
   p = &city_report_specs[0];
 
-  specialist_type_iterate(sp) {
+  fc_snprintf(sp_explanations, sizeof(sp_explanations),
+              "%s", _("Specialists: "));
+  specialist_type_iterate(i) {
+    struct specialist *s = specialist_by_number(i);
     p->show = FALSE;
     p->width = 2;
     p->space = 1;
     p->title1 = Q_("?specialist:S");
-    p->title2 = Q_(get_specialist(sp)->short_name);
-    my_snprintf(explanation[sp], sizeof(explanation[sp]), "%s: %s",
-		_("Specialists"), _(get_specialist(sp)->name));
-    p->explanation = explanation[sp];
-    p->data = get_specialist(sp);
+    p->title2 = specialist_abbreviation_translation(s);
+    fc_snprintf(sp_explanation[i], sizeof(sp_explanation[i]),
+                _("Specialists: %s"), specialist_plural_translation(s));
+    cat_snprintf(sp_explanations, sizeof(sp_explanations),
+                 "%s%s", (i == 0) ? "" : ", ",
+                 specialist_plural_translation(s));
+    p->explanation = sp_explanation[i];
+    p->data = s;
     p->func = cr_entry_specialist;
-    p->tagname = get_specialist(sp)->name;
+    p->tagname = specialist_rule_name(s);
     p++;
   } specialist_type_iterate_end;
+  
+  /* Summary column for all specialists. */
+  {
+    static char sp_summary[128];
+    p->show = FALSE;
+    p->width = MAX(7, specialist_count()*2-1);
+    p->space = 1;
+    p->title1 = _("Special");
+    fc_snprintf(sp_summary, sizeof(sp_summary),
+                "%s", specialists_abbreviation_string());
+    p->title2 = sp_summary;
+    p->explanation = sp_explanations;
+    p->data = NULL;
+    p->func = cr_entry_specialists;
+    p->tagname = "specialists";
+    p++;
+  }
 
   memcpy(p, base_city_report_specs,
 	 sizeof(base_city_report_specs));
@@ -595,7 +662,8 @@ void init_city_report_game_data(void)
     p++;
   }
 
-  assert(NUM_CREPORT_COLS == ARRAY_SIZE(base_city_report_specs) + SP_COUNT);
+  fc_assert(NUM_CREPORT_COLS == ARRAY_SIZE(base_city_report_specs)
+            + specialist_count() + 1);
 }
 
 /**********************************************************************
@@ -604,8 +672,8 @@ void init_city_report_game_data(void)
   it's better than sorting alphabetically.
 
   The GUI gives us two values to compare (as strings).  We try to split
-  them into an array of integer and string fields, then we compare
-  lexicographically.  Two integer fields are compared in the obvious
+  them into an array of numeric and string fields, then we compare
+  lexicographically.  Two numeric fields are compared in the obvious
   way, two character fields are compared alphabetically.  Arbitrarily, a
   numeric field is sorted before a character field (for "justification"
   note that numbers are before letters in the ASCII table).
@@ -615,7 +683,7 @@ void init_city_report_game_data(void)
    A datum_vector represents a long string of alternating strings and
    numbers. */
 struct datum {
-  long numeric_value;
+  float numeric_value;
   char *string_value;
   bool is_numeric;
 };
@@ -638,10 +706,10 @@ static void init_datum_string(struct datum *dat, const char *left,
 }
 
 /**********************************************************************
-  Init a datum from a number (a long because we happen to use
-  strtol).
+  Init a datum from a number (a float because we happen to use
+  strtof).
 **********************************************************************/
-static void init_datum_number(struct datum *dat, long val)
+static void init_datum_number(struct datum *dat, float val)
 {
   dat->is_numeric = TRUE;
   dat->numeric_value = val;
@@ -668,7 +736,15 @@ static int datum_compare(const struct datum *a, const struct datum *b)
 {
   if(a->is_numeric == b->is_numeric) {
     if(a->is_numeric) {
-      return a->numeric_value - b->numeric_value;
+      if (a->numeric_value == b->numeric_value) {
+        return 0;
+      } else if (a->numeric_value < b->numeric_value) {
+        return -1;
+      } else if (a->numeric_value > b->numeric_value) {
+        return +1;
+      } else {
+        return 0; /* shrug */
+      }
     } else {
       return strcmp(a->string_value, b->string_value);
     }
@@ -716,24 +792,24 @@ static void split_string(struct datum_vector *data, const char *str)
   string_start = str;
   while(*str) {
     char *endptr;
-    long value;
+    float value;
 
-    value = strtol(str, &endptr, 0);
-    if(endptr == str) {
-      /* that wasn't an integer; go on */
+    value = strtof(str, &endptr);
+    if(endptr == str || !isfinite(value)) {
+      /* that wasn't a sensible number; go on */
       str++;
     } else {
-      /* that was an integer, so stop the string we were parsing, add 
-         it (unless it's empty), then add the integer we just parsed */
+      /* that was a number, so stop the string we were parsing, add 
+         it (unless it's empty), then add the number we just parsed */
       struct datum d;
 
       if(str != string_start) {
         init_datum_string(&d, string_start, str);
-        datum_vector_append(data, &d);
+        datum_vector_append(data, d);
       }
 
       init_datum_number(&d, value);
-      datum_vector_append(data, &d);
+      datum_vector_append(data, d);
 
       /* finally, update the string position pointers */
       string_start = str;
@@ -746,7 +822,7 @@ static void split_string(struct datum_vector *data, const char *str)
     struct datum d;
 
     init_datum_string(&d, string_start, str);
-    datum_vector_append(data, &d);
+    datum_vector_append(data, d);
   }
 }
 
@@ -772,6 +848,14 @@ int cityrepfield_compare(const char *str1, const char *str2)
   struct datum_vector data1, data2;
   int retval;
 
+  if (str1 == str2) {
+    return 0;
+  } else if (NULL == str1) {
+    return 1;
+  } else if (NULL == str2) {
+    return -1;
+  }
+
   split_string(&data1, str1);
   split_string(&data2, str2);
 
