@@ -13,9 +13,9 @@
 #ifndef FC__CLIMISC_H
 #define FC__CLIMISC_H
 
-#include "shared.h"		/* MAX_LEN_NAME */
-
+/* common */
 #include "fc_types.h"
+#include "featured_text.h"      /* struct ft_color */
 #include "events.h"
 #include "unittype.h"
 #include "unitlist.h"
@@ -28,8 +28,8 @@ void client_remove_player(int plrno);
 void client_remove_city(struct city *pcity);
 void client_remove_unit(struct unit *punit);
 
-void client_change_all(struct city_production from,
-		       struct city_production to);
+void client_change_all(struct universal from,
+		       struct universal to);
 
 const char *get_embassy_status(const struct player *me,
 				const struct player *them);
@@ -38,6 +38,9 @@ const char *get_vision_status(const struct player *me,
 void client_diplomacy_clause_string(char *buf, int bufsiz,
 				    struct Clause *pclause);
 
+void global_warming_scaled(int *chance, int *rate, int max);
+void nuclear_winter_scaled(int *chance, int *rate, int max);
+
 struct sprite *client_research_sprite(void);
 struct sprite *client_warming_sprite(void);
 struct sprite *client_cooling_sprite(void);
@@ -53,62 +56,54 @@ void center_on_something(void);
  * unit_type_id of (cid - B_LAST).
  */
 
-cid cid_encode(struct city_production target);
-cid cid_encode_unit(const struct unit_type *punittype);
-cid cid_encode_building(Impr_type_id building);
+cid cid_encode(struct universal target);
+cid cid_encode_unit(struct unit_type *punittype);
+cid cid_encode_building(struct impr_type *pimprove);
 cid cid_encode_from_city(const struct city *pcity);
 
-struct city_production cid_decode(cid cid);
+struct universal cid_decode(cid cid);
 #define cid_production cid_decode
 
-bool city_can_build_impr_or_unit(const struct city *pcity,
-				 struct city_production target);
 bool city_unit_supported(const struct city *pcity,
-			 struct city_production target);
+			 struct universal target);
 bool city_unit_present(const struct city *pcity,
-		       struct city_production target);
+		       struct universal target);
 bool city_building_present(const struct city *pcity,
-			   struct city_production target);
+			   struct universal target);
 
 struct item {
-  struct city_production item;
+  struct universal item;
   char descr[MAX_LEN_NAME + 40];
 };
 
-typedef bool (*TestCityFunc)(const struct city *, struct city_production);
+typedef bool (*TestCityFunc)(const struct city *, struct universal);
 
 #define MAX_NUM_PRODUCTION_TARGETS (U_LAST + B_LAST)
-void name_and_sort_items(struct city_production *targets, int num_items,
+void name_and_sort_items(struct universal *targets, int num_items,
 			 struct item *items,
 			 bool show_cost, struct city *pcity);
-int collect_production_targets(struct city_production *targets,
+int collect_production_targets(struct universal *targets,
 			       struct city **selected_cities,
 			       int num_selected_cities, bool append_units,
 			       bool append_wonders, bool change_prod,
 			       TestCityFunc test_func);
-int collect_currently_building_targets(struct city_production *targets);
-int collect_buildable_targets(struct city_production *targets);
-int collect_eventually_buildable_targets(struct city_production *targets,
+int collect_currently_building_targets(struct universal *targets);
+int collect_buildable_targets(struct universal *targets);
+int collect_eventually_buildable_targets(struct universal *targets,
 					 struct city *pcity,
 					 bool advanced_tech);
-int collect_already_built_targets(struct city_production *targets,
+int collect_already_built_targets(struct universal *targets,
 				  struct city *pcity);
 
 /* the number of units in city */
 int num_present_units_in_city(struct city* pcity);
 int num_supported_units_in_city(struct city* pcity);	
 
-void handle_event(char *message, struct tile *ptile,
-		  enum event_type event, int conn_id);
+void handle_event(const char *featured_text, struct tile *ptile,
+                  enum event_type event, int conn_id);
 void create_event(struct tile *ptile, enum event_type event,
-		  const char *format, ...)
-     fc__attribute((__format__ (__printf__, 3, 4)));
-void write_chatline_content(const char *txt);
-
-void reports_freeze(void);
-void reports_freeze_till(int request_id);
-void reports_thaw(void);
-void reports_force_thaw(void);
+                  const struct ft_color color, const char *format, ...)
+                  fc__attribute((__format__ (__printf__, 4, 5)));
 
 struct city *get_nearest_city(const struct unit *punit, int *sq_dist);
 
@@ -118,6 +113,21 @@ void common_taxrates_callback(int i);
 bool can_units_do_connect(struct unit_list *punits,
 			  enum unit_activity activity);
 
+enum unit_bg_color_type { UNIT_BG_HP_LOSS,
+                          UNIT_BG_LAND,
+                          UNIT_BG_SEA,
+                          UNIT_BG_AMPHIBIOUS,
+                          UNIT_BG_FLYING };
+
+enum unit_bg_color_type unit_color_type(const struct unit_type *punittype);
+
 void buy_production_in_selected_cities(void);
 
+void set_unit_focus_status(struct player *pplayer);
+
+void client_player_init(struct player *pplayer);
+void client_player_destroy(struct player *pplayer);
+
+void client_player_maps_reset(void);
+
 #endif  /* FC__CLIMISC_H */
