@@ -21,15 +21,22 @@
 
 #include <gtk/gtk.h>
 
+/* utility */
 #include "fcintl.h"
+#include "log.h"
+
+/* common */
 #include "game.h"
 #include "player.h"
 
+/* client */
+#include "options.h"
+
+/* gui-gtk-2.0 */
 #include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
-#include "options.h"
 
 #include "finddlg.h"
 
@@ -128,13 +135,12 @@ void popup_find_dialog(void)
 **************************************************************************/
 static void update_find_dialog(GtkListStore *store)
 {
-  int i;
   GtkTreeIter it;
 
   gtk_list_store_clear(store);
 
-  for(i = 0; i < game.info.nplayers; i++) {
-    city_list_iterate(game.players[i].cities, pcity) {
+  players_iterate(pplayer) {
+    city_list_iterate(pplayer->cities, pcity) {
 	GValue value = { 0, };
 
 	gtk_list_store_append(store, &it);
@@ -146,7 +152,7 @@ static void update_find_dialog(GtkListStore *store)
 
 	gtk_list_store_set(store, &it, 1, pcity, -1);
     } city_list_iterate_end;
-  }
+  } players_iterate_end;
 }
 
 /**************************************************************************
