@@ -39,7 +39,7 @@
 #include "options.h"
 #include "tilespec.h"
 
-/* gui-gtk-2.0 */
+/* client/gui-gtk-2.0 */
 #include "chatline.h"
 #include "cityrep.h"
 #include "dialogs.h"
@@ -109,7 +109,7 @@ static void save_chat_logs_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void local_options_callback(GtkAction *action, gpointer data)
 {
-  popup_option_dialog();
+  option_dialog_popup(_("Set local options"), client_optset);
 }
 
 /****************************************************************
@@ -125,7 +125,7 @@ static void message_options_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void server_options_callback(GtkAction *action, gpointer data)
 {
-  popup_settable_options_dialog();
+  option_dialog_popup(_("Game Settings"), server_optset);
 }
 
 /****************************************************************
@@ -257,7 +257,7 @@ static void report_top_cities_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void report_messages_callback(GtkAction *action, gpointer data)
 {
-  popup_meswin_dialog(TRUE);
+  meswin_dialog_popup(TRUE);
 }
 
 /****************************************************************
@@ -436,6 +436,14 @@ static void help_space_rate_callback(GtkAction *action, gpointer data)
   popup_help_dialog_string(HELP_SPACE_RACE_ITEM);
 }
 
+/****************************************************************
+  Action "HELP_NATIONS" callback.
+*****************************************************************/
+static void help_nations_callback(GtkAction *action, gpointer data)
+{
+  popup_help_dialog_string(HELP_NATIONS_ITEM);
+}
+
 /****************************************************************
   Action "HELP_COPYING" callback.
 *****************************************************************/
@@ -468,7 +476,8 @@ static void edit_mode_callback(GtkToggleAction *action, gpointer data)
 {
   if (game.info.is_edit_mode ^ gtk_toggle_action_get_active(action)) {
     key_editor_toggle();
-    popdown_science_dialog(); /* Unreachbale techs in reqtree on/off */
+    /* Unreachbale techs in reqtree on/off */
+    science_report_dialog_popdown();
   }
 }
 
@@ -952,6 +961,14 @@ static void unit_upgrade_callback(GtkAction *action, gpointer data)
   popup_upgrade_dialog(get_units_in_focus());
 }
 
+/****************************************************************
+  Action "UNIT_CONVERT" callback.
+*****************************************************************/
+static void unit_convert_callback(GtkAction *action, gpointer data)
+{
+  key_unit_convert();
+}
+
 /****************************************************************
   Action "UNIT_DISBAND" callback.
 *****************************************************************/
@@ -970,7 +987,7 @@ static void build_city_callback(GtkAction *action, gpointer data)
      * not good! */
     /* Enable the button for adding to a city in all cases, so we
        get an eventual error message from the server if we try. */
-    if (can_unit_add_or_build_city(punit)) {
+    if (unit_can_add_or_build_city(punit)) {
       request_unit_build_city(punit);
     } else if (unit_has_type_flag(punit, F_HELP_WONDER)) {
       request_unit_caravan_action(punit, PACKET_UNIT_HELP_BUILD_WONDER);
@@ -1178,7 +1195,7 @@ static void center_view_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void report_units_callback(GtkAction *action, gpointer data)
 {
-  popup_activeunits_report_dialog(TRUE);
+  units_report_dialog_popup(TRUE);
 }
 
 /****************************************************************
@@ -1186,7 +1203,7 @@ static void report_units_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void report_cities_callback(GtkAction *action, gpointer data)
 {
-  popup_city_report_dialog(TRUE);
+  city_report_dialog_popup(TRUE);
 }
 
 /****************************************************************
@@ -1194,7 +1211,7 @@ static void report_cities_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void report_economy_callback(GtkAction *action, gpointer data)
 {
-  popup_economy_report_dialog(TRUE);
+  economy_report_dialog_popup(TRUE);
 }
 
 /****************************************************************
@@ -1202,7 +1219,7 @@ static void report_economy_callback(GtkAction *action, gpointer data)
 *****************************************************************/
 static void report_research_callback(GtkAction *action, gpointer data)
 {
-  popup_science_dialog(TRUE);
+  science_report_dialog_popup(TRUE);
 }
 
 /****************************************************************
@@ -1273,8 +1290,12 @@ static GtkActionGroup *get_safe_group(void)
       /* Civilization menu. */
       {"MAP_VIEW", NULL, Q_("?noun:_View"),
        "F1", NULL, G_CALLBACK(map_view_callback)},
+      {"REPORT_UNITS", NULL, _("_Units"),
+       "F2", NULL, G_CALLBACK(report_units_callback)},
       {"REPORT_NATIONS", NULL, _("_Nations"),
        "F3", NULL, G_CALLBACK(report_nations_callback)},
+      {"REPORT_CITIES", NULL, _("_Cities"),
+       "F4", NULL, G_CALLBACK(report_cities_callback)},
 
       {"REPORT_WOW", NULL, _("_Wonders of the World"),
        "F7", NULL, G_CALLBACK(report_wow_callback)},
@@ -1317,6 +1338,8 @@ static GtkActionGroup *get_safe_group(void)
        NULL, NULL, G_CALLBACK(help_space_rate_callback)},
       {"HELP_RULESET", NULL, _("About Ruleset"),
        NULL, NULL, G_CALLBACK(help_ruleset_callback)},
+      {"HELP_NATIONS", NULL, _("About Nations"),
+       NULL, NULL, G_CALLBACK(help_nations_callback)},
 
       {"HELP_CONNECTING", NULL, _("Connecting"),
        NULL, NULL, G_CALLBACK(help_connecting_callback)},
@@ -1404,7 +1427,7 @@ static GtkActionGroup *get_safe_group(void)
       {"SHOW_BETTER_FOG_OF_WAR", NULL, _("Better Fog of War"),
        NULL, NULL, G_CALLBACK(show_better_fog_of_war_callback), FALSE},
 
-      {"FULL_SCREEN", NULL, _("_Full Screen"),
+      {"FULL_SCREEN", NULL, _("_Fullscreen"),
        "<Alt>Return", NULL, G_CALLBACK(full_screen_callback), FALSE}
     };
 
@@ -1513,6 +1536,8 @@ static GtkActionGroup *get_unit_group(void)
        "h", NULL, G_CALLBACK(unit_homecity_callback)},
       {"UNIT_UPGRADE", NULL, _("Upgr_ade"),
        "<Shift>u", NULL, G_CALLBACK(unit_upgrade_callback)},
+      {"UNIT_CONVERT", NULL, _("C_onvert"),
+       "<Shift>o", NULL, G_CALLBACK(unit_convert_callback)},
       {"UNIT_DISBAND", NULL, _("_Disband"),
        "<Shift>d", NULL, G_CALLBACK(unit_disband_callback)},
 
@@ -1546,9 +1571,9 @@ static GtkActionGroup *get_unit_group(void)
        "n", NULL, G_CALLBACK(clean_fallout_callback)},
 
       /* Combat menu. */
-      {"BUILD_FORTRESS", NULL, _("Build _Fortress"),
+      {"BUILD_FORTRESS", NULL, _("Build Type A Base"),
        "f", NULL, G_CALLBACK(build_fortress_callback)},
-      {"BUILD_AIRBASE", NULL, _("Build Airbas_e"),
+      {"BUILD_AIRBASE", NULL, _("Build Type B Base"),
        "e", NULL, G_CALLBACK(build_airbase_callback)},
 
       {"DO_PILLAGE", NULL, _("_Pillage"),
@@ -1619,10 +1644,6 @@ static GtkActionGroup *get_player_group(void)
        "c", NULL, G_CALLBACK(center_view_callback)},
 
       /* Civilization menu. */
-      {"REPORT_UNITS", NULL, _("_Units"),
-       "F2", NULL, G_CALLBACK(report_units_callback)},
-      {"REPORT_CITIES", NULL, _("_Cities"),
-       "F4", NULL, G_CALLBACK(report_cities_callback)},
       {"REPORT_ECONOMY", NULL, _("_Economy"),
        "F5", NULL, G_CALLBACK(report_economy_callback)},
       {"REPORT_RESEARCH", NULL, _("_Research"),
@@ -1646,17 +1667,17 @@ static GtkActionGroup *get_player_group(void)
 static const gchar *get_ui_filename(void)
 {
   static char filename[256];
-  char *name;
+  const char *name;
 
   if ((name = getenv("FREECIV_MENUS"))
-      || (name = datafilename("gtk_menus.xml"))) {
+      || (name = fileinfoname(get_data_dirs(), "gtk_menus.xml"))) {
     sz_strlcpy(filename, name);
   } else {
-    freelog(LOG_ERROR, "Gtk menus: file definition not found");
+    log_error("Gtk menus: file definition not found");
     filename[0] = '\0';
   }
 
-  freelog(LOG_VERBOSE, "ui menu file is \"%s\".", filename);
+  log_verbose("ui menu file is \"%s\".", filename);
   return filename;
 }
 
@@ -1698,7 +1719,7 @@ GtkWidget *setup_menus(GtkWidget *window)
   /* Load the menus. */
   if (0 == gtk_ui_manager_add_ui_from_file(ui_manager,
                                            get_ui_filename(), &error)) {
-    freelog(LOG_ERROR, "Gtk menus: %s", error->message);
+    log_error("Gtk menus: %s", error->message);
     g_error_free(error);
   }
 
@@ -1719,16 +1740,16 @@ static void menus_set_active(GtkActionGroup *group,
   GtkAction *action = gtk_action_group_get_action(group, action_name);
 
   if (!action) {
-    freelog(LOG_ERROR, "Can't set active for non-existent "
-            "action \"%s\" in group \"%s\".",
-            action_name, gtk_action_group_get_name(group));
+    log_error("Can't set active for non-existent "
+              "action \"%s\" in group \"%s\".",
+              action_name, gtk_action_group_get_name(group));
     return;
   }
 
   if (!GTK_IS_TOGGLE_ACTION(action)) {
-    freelog(LOG_ERROR, "Can't set active for non-togglable "
-            "action \"%s\" in group \"%s\".",
-            action_name, gtk_action_group_get_name(group));
+    log_error("Can't set active for non-togglable "
+              "action \"%s\" in group \"%s\".",
+              action_name, gtk_action_group_get_name(group));
     return;
   }
 
@@ -1745,9 +1766,9 @@ static void menus_set_sensitive(GtkActionGroup *group,
   GtkAction *action = gtk_action_group_get_action(group, action_name);
 
   if (!action) {
-    freelog(LOG_ERROR, "Can't set active for non-existent "
-            "action \"%s\" in group \"%s\".",
-            action_name, gtk_action_group_get_name(group));
+    log_error("Can't set active for non-existent "
+              "action \"%s\" in group \"%s\".",
+              action_name, gtk_action_group_get_name(group));
     return;
   }
 
@@ -1765,9 +1786,9 @@ static void menus_set_visible(GtkActionGroup *group,
   GtkAction *action = gtk_action_group_get_action(group, action_name);
 
   if (!action) {
-    freelog(LOG_ERROR, "Can't set visible for non-existent "
-            "action \"%s\" in group \"%s\".",
-            action_name, gtk_action_group_get_name(group));
+    log_error("Can't set visible for non-existent "
+              "action \"%s\" in group \"%s\".",
+              action_name, gtk_action_group_get_name(group));
     return;
   }
 
@@ -1785,9 +1806,9 @@ static void menus_rename(GtkActionGroup *group,
   GtkAction *action = gtk_action_group_get_action(group, action_name);
 
   if (!action) {
-    freelog(LOG_ERROR, "Can't rename non-existent "
-            "action \"%s\" in group \"%s\".",
-            action_name, gtk_action_group_get_name(group));
+    log_error("Can't rename non-existent "
+              "action \"%s\" in group \"%s\".",
+              action_name, gtk_action_group_get_name(group));
     return;
   }
 
@@ -1857,16 +1878,19 @@ static void view_menu_update_sensitivity(void)
 static const char *get_tile_change_menu_text(struct tile *ptile,
                                              enum unit_activity activity)
 {
-  struct tile newtile = *ptile;
+  struct tile *newtile = tile_virtual_new(ptile);
+  const char *text;
 
-  tile_apply_activity(&newtile, activity);
-  return tile_get_info_text(&newtile, 0);
+  tile_apply_activity(newtile, activity);
+  text = tile_get_info_text(newtile, 0);
+  tile_virtual_destroy(newtile);
+  return text;
 }
 
 /****************************************************************
   Updates the menus.
 *****************************************************************/
-static gboolean update_menus_callback(gpointer data)
+void real_menus_update(void)
 {
   GtkActionGroup *safe_group;
   GtkActionGroup *edit_group;
@@ -1874,15 +1898,13 @@ static gboolean update_menus_callback(gpointer data)
   GtkActionGroup *playing_group;
   GtkActionGroup *player_group;
   struct unit_list *punits = NULL;
-  bool units_all_same_tile = TRUE;
+  bool units_all_same_tile = TRUE, units_all_same_type = TRUE;
   GtkMenu *menu;
-  char irrtext[128], mintext[128], transtext[128];
+  char acttext[128], irrtext[128], mintext[128], transtext[128];
   struct terrain *pterrain;
 
-  /* Remove GSource id. */
-  *((guint *) data) = 0;
-  if (!ui_manager) {
-    return FALSE;
+  if (NULL == ui_manager && !can_client_change_view()) {
+    return;
   }
 
   safe_group = get_safe_group();
@@ -1891,189 +1913,86 @@ static gboolean update_menus_callback(gpointer data)
   playing_group = get_playing_group();
   player_group = get_player_group();
 
-  menus_set_active(safe_group, "SAVE_OPTIONS_ON_EXIT", save_options_on_exit);
-  menus_set_sensitive(safe_group, "SERVER_OPTIONS", client.conn.established);
-
-  menus_set_sensitive(safe_group, "GAME_SAVE_AS",
-                      can_client_access_hack()
-                      && C_S_RUNNING <= client_state());
-  menus_set_sensitive(safe_group, "GAME_SAVE",
-                      can_client_access_hack()
-                      && C_S_RUNNING <= client_state());
-  menus_set_sensitive(safe_group, "LEAVE",
-                      client.conn.established);
-
-  if (!can_client_change_view()) {
-    gtk_action_group_set_sensitive(safe_group, FALSE);
-    gtk_action_group_set_sensitive(edit_group, FALSE);
-    gtk_action_group_set_sensitive(unit_group, FALSE);
-    gtk_action_group_set_sensitive(player_group, FALSE);
-    gtk_action_group_set_sensitive(playing_group, FALSE);
-    return FALSE;
-  }
-
   if (get_num_units_in_focus() > 0) {
     const struct tile *ptile = NULL;
+    const struct unit_type *ptype = NULL;
     punits = get_units_in_focus();
     unit_list_iterate(punits, punit) {
-      if (ptile) {
+      fc_assert((ptile==NULL) == (ptype==NULL));
+      if (ptile || ptype) {
         if (punit->tile != ptile) {
           units_all_same_tile = FALSE;
-          break;
+        }
+        if (unit_type(punit) != ptype) {
+          units_all_same_type = FALSE;
         }
       } else {
         ptile = punit->tile;
+        ptype = unit_type(punit);
       }
     } unit_list_iterate_end;
   }
 
-  if ((menu = find_action_menu(playing_group, "MENU_GOVERNMENT"))) {
-    GList *list, *iter;
-    GtkWidget *item, *image;
-    struct sprite *gsprite;
-    char buf[256];
-
-    /* Remove previous government entries. */
-    list = gtk_container_get_children(GTK_CONTAINER(menu));
-    for (iter = list; iter; iter = g_list_next(iter)) {
-      if (g_object_get_data(G_OBJECT(iter->data), "government") != NULL
-          || GTK_IS_SEPARATOR_MENU_ITEM(iter->data)) {
-        gtk_widget_destroy(GTK_WIDGET(iter->data));
-      }
-    }
-    g_list_free(list);
-
-    /* Add new government entries. */
-    item = gtk_separator_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-    gtk_widget_show(item);
-
-    government_iterate(g) {
-      if (g != game.government_during_revolution) {
-        /* TRANS: %s is a government name */
-        my_snprintf(buf, sizeof(buf), _("%s..."),
-                    government_name_translation(g));
-        item = gtk_image_menu_item_new_with_label(buf);
-        g_object_set_data(G_OBJECT(item), "government", g);
-
-        if ((gsprite = get_government_sprite(tileset, g))) {
-          image = gtk_image_new_from_pixbuf(sprite_get_pixbuf(gsprite));
-          gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
-          gtk_widget_show(image);
-        }
-
-        g_signal_connect(item, "activate",
-                         G_CALLBACK(government_callback), g);
-
-        if (!can_change_to_government(client.conn.playing, g)) {
-          gtk_widget_set_sensitive(item, FALSE);
-        }
-
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        gtk_widget_show(item);
-      }
-    } government_iterate_end;
-  }
-
-  if ((menu = find_action_menu(unit_group, "MENU_BUILD_BASE"))) {
-    GList *list, *iter;
-    GtkWidget *item;
-
-    /* Remove previous base entries. */
-    list = gtk_container_get_children(GTK_CONTAINER(menu));
-    for (iter = list; iter; iter = g_list_next(iter)) {
-      gtk_widget_destroy(GTK_WIDGET(iter->data));
-    }
-    g_list_free(list);
-
-    /* Add new base entries. */
-    base_type_iterate(p) {
-      if (p->buildable) {
-        item = gtk_menu_item_new_with_label(base_name_translation(p));
-
-        g_signal_connect(item, "activate", G_CALLBACK(base_callback), p);
-
-         if (punits) {
-           gtk_widget_set_sensitive(item, can_units_do_base(punits,
-                                                            base_number(p)));
-         }
-
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        gtk_widget_show(item);
-      }
-    } base_type_iterate_end;
-  }
-
-  gtk_action_group_set_sensitive(safe_group, TRUE);
   gtk_action_group_set_sensitive(edit_group,
                                  editor_is_active());
-  gtk_action_group_set_sensitive(player_group, client_has_player());
   gtk_action_group_set_sensitive(playing_group, can_client_issue_orders()
                                  && !editor_is_active());
   gtk_action_group_set_sensitive(unit_group, can_client_issue_orders()
                                  && !editor_is_active() && punits != NULL);
 
-  menus_set_sensitive(playing_group, "TAX_RATE",
-                      game.info.changable_tax
-                      && can_client_issue_orders());
-
   menus_set_active(safe_group, "EDIT_MODE", game.info.is_edit_mode);
   menus_set_sensitive(safe_group, "EDIT_MODE",
                       can_conn_enable_editing(&client.conn));
   editgui_refresh();
 
-  menus_set_active(safe_group, "SHOW_CITY_OUTLINES", draw_city_outlines);
-  menus_set_active(safe_group, "SHOW_CITY_OUTPUT", draw_city_output);
-  menus_set_active(safe_group, "SHOW_MAP_GRID", draw_map_grid);
-  menus_set_active(safe_group, "SHOW_NATIONAL_BORDERS", draw_borders);
-  menus_set_sensitive(safe_group, "SHOW_NATIONAL_BORDERS",
-                      game.info.borders > 0);
-  menus_set_active(safe_group, "SHOW_CITY_FULL_BAR", draw_full_citybar);
-  menus_set_active(safe_group, "SHOW_CITY_NAMES", draw_city_names);
-  menus_set_active(safe_group, "SHOW_CITY_GROWTH", draw_city_growth);
-  menus_set_active(safe_group, "SHOW_CITY_PRODUCTIONS",
-                   draw_city_productions);
-  menus_set_active(safe_group, "SHOW_CITY_BUY_COST", draw_city_buycost);
-  menus_set_active(safe_group, "SHOW_CITY_TRADE_ROUTES",
-                   draw_city_trade_routes);
-  menus_set_active(safe_group, "SHOW_TERRAIN", draw_terrain);
-  menus_set_active(safe_group, "SHOW_COASTLINE", draw_coastline);
-  menus_set_active(safe_group, "SHOW_ROADS_RAILS", draw_roads_rails);
-  menus_set_active(safe_group, "SHOW_IRRIGATION", draw_irrigation);
-  menus_set_active(safe_group, "SHOW_MINES", draw_mines);
-  menus_set_active(safe_group, "SHOW_FORTRESS_AIRBASE",
-                   draw_fortress_airbase);
-  menus_set_active(safe_group, "SHOW_SPECIALS", draw_specials);
-  menus_set_active(safe_group, "SHOW_POLLUTION", draw_pollution);
-  menus_set_active(safe_group, "SHOW_CITIES", draw_cities);
-  menus_set_active(safe_group, "SHOW_UNITS", draw_units);
-  menus_set_active(safe_group, "SHOW_UNIT_SOLID_BG",
-                   solid_color_behind_units);
-  menus_set_active(safe_group, "SHOW_UNIT_SHIELDS", draw_unit_shields);
-  menus_set_active(safe_group, "SHOW_FOCUS_UNIT", draw_focus_unit);
-  menus_set_active(safe_group, "SHOW_FOG_OF_WAR", draw_fog_of_war);
-  if (tileset_use_hard_coded_fog(tileset)) {
-    menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", TRUE, TRUE);
-    menus_set_active(safe_group, "SHOW_BETTER_FOG_OF_WAR",
-                     gui_gtk2_better_fog);
-  } else {
-    menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", FALSE, FALSE);
+  if (!can_client_issue_orders()) {
+    return;
   }
 
-  view_menu_update_sensitivity();
+  /* Set government sensitivity. */
+  if ((menu = find_action_menu(playing_group, "MENU_GOVERNMENT"))) {
+    GList *list, *iter;
+    struct government *pgov;
 
-  menus_set_active(safe_group, "FULL_SCREEN", fullscreen_mode);
+    list = gtk_container_get_children(GTK_CONTAINER(menu));
+    for (iter = list; NULL != iter; iter = g_list_next(iter)) {
+      pgov = g_object_get_data(G_OBJECT(iter->data), "government");
+      if (NULL != pgov) {
+        gtk_widget_set_sensitive(GTK_WIDGET(iter->data),
+                                 can_change_to_government(client_player(),
+                                                          pgov));
+      }
+    }
+    g_list_free(list);
+  }
+
+  if (!punits) {
+    return;
+  }
 
   /* Remaining part of this function: Update Unit, Work, and Combat menus */
 
-  if (!can_client_issue_orders() || !punits) {
-    return FALSE;
+  /* Set base sensitivity. */
+  if ((menu = find_action_menu(unit_group, "MENU_BUILD_BASE"))) {
+    GList *list, *iter;
+    struct base_type *pbase;
+
+    list = gtk_container_get_children(GTK_CONTAINER(menu));
+    for (iter = list; NULL != iter; iter = g_list_next(iter)) {
+      pbase = g_object_get_data(G_OBJECT(iter->data), "base");
+      if (NULL != pbase) {
+        gtk_widget_set_sensitive(GTK_WIDGET(iter->data),
+                                 can_units_do_base(punits,
+                                                   base_number(pbase)));
+      }
+    }
+    g_list_free(list);
   }
 
   /* Enable the button for adding to a city in all cases, so we
    * get an eventual error message from the server if we try. */
   menus_set_sensitive(unit_group, "BUILD_CITY",
-            (can_units_do(punits, can_unit_add_or_build_city)
+            (can_units_do(punits, unit_can_add_or_build_city)
              || can_units_do(punits, unit_can_help_build_wonder_here)));
   menus_set_sensitive(unit_group, "GO_BUILD_CITY",
                       units_have_flag(punits, F_CITIES, TRUE));
@@ -2108,7 +2027,8 @@ static gboolean update_menus_callback(gpointer data)
   menus_set_sensitive(unit_group, "UNIT_DISBAND",
                       units_have_flag(punits, F_UNDISBANDABLE, FALSE));
   menus_set_sensitive(unit_group, "UNIT_UPGRADE",
-                      TRUE /* FIXME: what check should we do? */);
+                      units_can_upgrade(punits));
+  /* "UNIT_CONVERT" dealt with below */
   menus_set_sensitive(unit_group, "UNIT_HOMECITY",
                       can_units_do(punits, can_unit_change_homecity));
   menus_set_sensitive(unit_group, "UNIT_UNLOAD_TRANSPORTER",
@@ -2151,7 +2071,7 @@ static gboolean update_menus_callback(gpointer data)
     } unit_list_iterate_end;
     
     if (city_on_tile && units_have_flag(punits, F_ADD_TO_CITY, TRUE)) {
-      menus_rename(unit_group, "BUILD_CITY", _("Add to City (_B)"));
+      menus_rename(unit_group, "BUILD_CITY", _("Add to City"));
     } else {
       /* refresh default order */
       menus_rename(unit_group, "BUILD_CITY", _("_Build City"));
@@ -2181,13 +2101,54 @@ static gboolean update_menus_callback(gpointer data)
     menus_rename(unit_group, "BUILD_ROAD", _("Build _Road"));
   }
 
+  if (units_all_same_type) {
+    struct unit *punit = unit_list_get(punits, 0);
+    struct unit_type *to_unittype =
+      can_upgrade_unittype(client_player(), unit_type(punit));
+    if (to_unittype) {
+      /* TRANS: %s is a unit type. */
+      fc_snprintf(acttext, sizeof(acttext), _("Upgr_ade to %s"),
+                  utype_name_translation(
+                    can_upgrade_unittype(client_player(), unit_type(punit))));
+    } else {
+      acttext[0] = '\0';
+    }
+  } else {
+    acttext[0] = '\0';
+  }
+  if ('\0' != acttext[0]) {
+    menus_rename(unit_group, "UNIT_UPGRADE", acttext);
+  } else {
+    menus_rename(unit_group, "UNIT_UPGRADE", _("Upgr_ade"));
+  }
+
+  if (units_can_convert(punits)) {
+    menus_set_sensitive(unit_group, "UNIT_CONVERT", TRUE);
+    if (units_all_same_type) {
+      struct unit *punit = unit_list_get(punits, 0);
+      /* TRANS: %s is a unit type. */
+      fc_snprintf(acttext, sizeof(acttext), _("C_onvert to %s"),
+                  utype_name_translation(unit_type(punit)->converted_to));
+    } else {
+      acttext[0] = '\0';
+    }
+  } else {
+    menus_set_sensitive(unit_group, "UNIT_CONVERT", FALSE);
+    acttext[0] = '\0';
+  }
+  if ('\0' != acttext[0]) {
+    menus_rename(unit_group, "UNIT_CONVERT", acttext);
+  } else {
+    menus_rename(unit_group, "UNIT_CONVERT", _("C_onvert"));
+  }
+
   if (units_all_same_tile) {
     struct unit *punit = unit_list_get(punits, 0);
 
     pterrain = tile_terrain(punit->tile);
     if (pterrain->irrigation_result != T_NONE
         && pterrain->irrigation_result != pterrain) {
-      my_snprintf(irrtext, sizeof(irrtext), _("Change to %s (_I)"),
+      fc_snprintf(irrtext, sizeof(irrtext), _("Change to %s"),
                   get_tile_change_menu_text(punit->tile,
                                             ACTIVITY_IRRIGATE));
     } else if (tile_has_special(punit->tile, S_IRRIGATION)
@@ -2200,7 +2161,7 @@ static gboolean update_menus_callback(gpointer data)
 
     if (pterrain->mining_result != T_NONE
         && pterrain->mining_result != pterrain) {
-      my_snprintf(mintext, sizeof(mintext), _("Change to %s (_M)"),
+      fc_snprintf(mintext, sizeof(mintext), _("Change to %s"),
                   get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
     } else {
       sz_strlcpy(mintext, _("Build _Mine"));
@@ -2208,7 +2169,7 @@ static gboolean update_menus_callback(gpointer data)
 
     if (pterrain->transform_result != T_NONE
         && pterrain->transform_result != pterrain) {
-      my_snprintf(transtext, sizeof(transtext), _("Transf_orm to %s"),
+      fc_snprintf(transtext, sizeof(transtext), _("Transf_orm to %s"),
                   get_tile_change_menu_text(punit->tile,
                                             ACTIVITY_TRANSFORM));
     } else {
@@ -2227,7 +2188,7 @@ static gboolean update_menus_callback(gpointer data)
   if (can_units_do_activity(punits, ACTIVITY_FORTIFYING)) {
     menus_rename(unit_group, "BUILD_FORTRESS", _("_Fortify Unit"));
   } else {
-    menus_rename(unit_group, "BUILD_FORTRESS", _("Build _Fortress"));
+    menus_rename(unit_group, "BUILD_FORTRESS", _("Build Type A Base"));
   }
 
   if (units_have_flag(punits, F_PARATROOPERS, TRUE)) {
@@ -2235,18 +2196,165 @@ static gboolean update_menus_callback(gpointer data)
   } else {
     menus_rename(unit_group, "CLEAN_POLLUTION", _("Clean _Pollution"));
   }
-
-  return FALSE;
 }
 
-/****************************************************************
-  Updates the menus.
-*****************************************************************/
-void update_menus(void)
+/**************************************************************************
+  Initialize menus (sensitivity, name, etc.) based on the
+  current state and current ruleset, etc.  Call menus_update().
+**************************************************************************/
+void real_menus_init(void)
 {
-  static guint callback_id = 0;
+  GtkActionGroup *safe_group;
+  GtkActionGroup *edit_group;
+  GtkActionGroup *unit_group;
+  GtkActionGroup *playing_group;
+  GtkActionGroup *player_group;
+  GtkMenu *menu;
 
-  if (NULL != ui_manager && 0 == callback_id) {
-    callback_id = g_idle_add(update_menus_callback, &callback_id);
+  if (NULL == ui_manager) {
+    return;
   }
+
+  safe_group = get_safe_group();
+  edit_group = get_edit_group();
+  unit_group = get_unit_group();
+  playing_group = get_playing_group();
+  player_group = get_player_group();
+
+  menus_set_sensitive(safe_group, "GAME_SAVE_AS",
+                      can_client_access_hack()
+                      && C_S_RUNNING <= client_state());
+  menus_set_sensitive(safe_group, "GAME_SAVE",
+                      can_client_access_hack()
+                      && C_S_RUNNING <= client_state());
+
+  menus_set_active(safe_group, "SAVE_OPTIONS_ON_EXIT", save_options_on_exit);
+  menus_set_sensitive(safe_group, "SERVER_OPTIONS", client.conn.established);
+
+  menus_set_sensitive(safe_group, "LEAVE",
+                      client.conn.established);
+
+  if (!can_client_change_view()) {
+    gtk_action_group_set_sensitive(safe_group, FALSE);
+    gtk_action_group_set_sensitive(edit_group, FALSE);
+    gtk_action_group_set_sensitive(unit_group, FALSE);
+    gtk_action_group_set_sensitive(player_group, FALSE);
+    gtk_action_group_set_sensitive(playing_group, FALSE);
+    return;
+  }
+
+  if ((menu = find_action_menu(playing_group, "MENU_GOVERNMENT"))) {
+    GList *list, *iter;
+    GtkWidget *item, *image;
+    struct sprite *gsprite;
+    char buf[256];
+
+    /* Remove previous government entries. */
+    list = gtk_container_get_children(GTK_CONTAINER(menu));
+    for (iter = list; NULL != iter; iter = g_list_next(iter)) {
+      if (g_object_get_data(G_OBJECT(iter->data), "government") != NULL
+          || GTK_IS_SEPARATOR_MENU_ITEM(iter->data)) {
+        gtk_widget_destroy(GTK_WIDGET(iter->data));
+      }
+    }
+    g_list_free(list);
+
+    /* Add new government entries. */
+    item = gtk_separator_menu_item_new();
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    gtk_widget_show(item);
+
+    governments_iterate(g) {
+      if (g != game.government_during_revolution) {
+        /* TRANS: %s is a government name */
+        fc_snprintf(buf, sizeof(buf), _("%s..."),
+                    government_name_translation(g));
+        item = gtk_image_menu_item_new_with_label(buf);
+        g_object_set_data(G_OBJECT(item), "government", g);
+
+        if ((gsprite = get_government_sprite(tileset, g))) {
+          image = gtk_image_new_from_pixbuf(sprite_get_pixbuf(gsprite));
+          gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
+          gtk_widget_show(image);
+        }
+
+        g_signal_connect(item, "activate",
+                         G_CALLBACK(government_callback), g);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        gtk_widget_show(item);
+      }
+    } governments_iterate_end;
+  }
+
+  if ((menu = find_action_menu(unit_group, "MENU_BUILD_BASE"))) {
+    GList *list, *iter;
+    GtkWidget *item;
+
+    /* Remove previous base entries. */
+    list = gtk_container_get_children(GTK_CONTAINER(menu));
+    for (iter = list; NULL != iter; iter = g_list_next(iter)) {
+      gtk_widget_destroy(GTK_WIDGET(iter->data));
+    }
+    g_list_free(list);
+
+    /* Add new base entries. */
+    base_type_iterate(p) {
+      if (p->buildable) {
+        item = gtk_menu_item_new_with_label(base_name_translation(p));
+        g_object_set_data(G_OBJECT(item), "base", p);
+        g_signal_connect(item, "activate", G_CALLBACK(base_callback), p);
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        gtk_widget_show(item);
+      }
+    } base_type_iterate_end;
+  }
+
+  gtk_action_group_set_sensitive(safe_group, TRUE);
+  gtk_action_group_set_sensitive(player_group, client_has_player());
+
+  menus_set_sensitive(playing_group, "TAX_RATE",
+                      game.info.changable_tax
+                      && can_client_issue_orders());
+
+  menus_set_active(safe_group, "SHOW_CITY_OUTLINES", draw_city_outlines);
+  menus_set_active(safe_group, "SHOW_CITY_OUTPUT", draw_city_output);
+  menus_set_active(safe_group, "SHOW_MAP_GRID", draw_map_grid);
+  menus_set_active(safe_group, "SHOW_NATIONAL_BORDERS", draw_borders);
+  menus_set_sensitive(safe_group, "SHOW_NATIONAL_BORDERS",
+                      BORDERS_DISABLED != game.info.borders);
+  menus_set_active(safe_group, "SHOW_CITY_FULL_BAR", draw_full_citybar);
+  menus_set_active(safe_group, "SHOW_CITY_NAMES", draw_city_names);
+  menus_set_active(safe_group, "SHOW_CITY_GROWTH", draw_city_growth);
+  menus_set_active(safe_group, "SHOW_CITY_PRODUCTIONS",
+                   draw_city_productions);
+  menus_set_active(safe_group, "SHOW_CITY_BUY_COST", draw_city_buycost);
+  menus_set_active(safe_group, "SHOW_CITY_TRADE_ROUTES",
+                   draw_city_trade_routes);
+  menus_set_active(safe_group, "SHOW_TERRAIN", draw_terrain);
+  menus_set_active(safe_group, "SHOW_COASTLINE", draw_coastline);
+  menus_set_active(safe_group, "SHOW_ROADS_RAILS", draw_roads_rails);
+  menus_set_active(safe_group, "SHOW_IRRIGATION", draw_irrigation);
+  menus_set_active(safe_group, "SHOW_MINES", draw_mines);
+  menus_set_active(safe_group, "SHOW_FORTRESS_AIRBASE",
+                   draw_fortress_airbase);
+  menus_set_active(safe_group, "SHOW_SPECIALS", draw_specials);
+  menus_set_active(safe_group, "SHOW_POLLUTION", draw_pollution);
+  menus_set_active(safe_group, "SHOW_CITIES", draw_cities);
+  menus_set_active(safe_group, "SHOW_UNITS", draw_units);
+  menus_set_active(safe_group, "SHOW_UNIT_SOLID_BG",
+                   solid_color_behind_units);
+  menus_set_active(safe_group, "SHOW_UNIT_SHIELDS", draw_unit_shields);
+  menus_set_active(safe_group, "SHOW_FOCUS_UNIT", draw_focus_unit);
+  menus_set_active(safe_group, "SHOW_FOG_OF_WAR", draw_fog_of_war);
+  if (tileset_use_hard_coded_fog(tileset)) {
+    menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", TRUE, TRUE);
+    menus_set_active(safe_group, "SHOW_BETTER_FOG_OF_WAR",
+                     gui_gtk2_better_fog);
+  } else {
+    menus_set_visible(safe_group, "SHOW_BETTER_FOG_OF_WAR", FALSE, FALSE);
+  }
+
+  view_menu_update_sensitivity();
+
+  menus_set_active(safe_group, "FULL_SCREEN", fullscreen_mode);
 }
