@@ -24,10 +24,11 @@
 #include "unitlist.h"
 
 /* client */
-#include "clinet.h"
+#include "client_main.h"
 #include "control.h"
 
-/* gui-sdl */
+/* client/gui-sdl */
+#include "citydlg.h"
 #include "colors.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -80,13 +81,14 @@ static int diplomat_dlg_window_callback(struct widget *pWindow)
 static int diplomat_embassy_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
-      request_diplomat_action(DIPLOMAT_EMBASSY, pDiplomat_Dlg->diplomat_id,
-                                         pDiplomat_Dlg->diplomat_target_id, 0);
+    if (NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)
+        && NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)) {
+      request_diplomat_action(DIPLOMAT_INVESTIGATE,
+                              pDiplomat_Dlg->diplomat_id,
+                              pDiplomat_Dlg->diplomat_target_id, 0);
     }
-  
-    popdown_diplomat_dialog();  
+
+    popdown_diplomat_dialog();
   }
   return -1;
 }
@@ -97,10 +99,11 @@ static int diplomat_embassy_callback(struct widget *pWidget)
 static int diplomat_investigate_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
-      request_diplomat_action(DIPLOMAT_INVESTIGATE, pDiplomat_Dlg->diplomat_id,
-                                         pDiplomat_Dlg->diplomat_target_id, 0);
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
+      request_diplomat_action(DIPLOMAT_INVESTIGATE,
+                              pDiplomat_Dlg->diplomat_id,
+                              pDiplomat_Dlg->diplomat_target_id, 0);
     }
   
     popdown_diplomat_dialog();
@@ -114,10 +117,10 @@ static int diplomat_investigate_callback(struct widget *pWidget)
 static int spy_poison_callback( struct widget *pWidget )
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
       request_diplomat_action(SPY_POISON, pDiplomat_Dlg->diplomat_id,
-                                         pDiplomat_Dlg->diplomat_target_id, 0);
+                              pDiplomat_Dlg->diplomat_target_id, 0);
     }
   
     popdown_diplomat_dialog();
@@ -131,10 +134,10 @@ static int spy_poison_callback( struct widget *pWidget )
 *****************************************************************/
 static int spy_sabotage_request(struct widget *pWidget)
 {
-  if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-     && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
+  if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+      && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
     request_diplomat_answer(DIPLOMAT_SABOTAGE, pDiplomat_Dlg->diplomat_id,
-                                       pDiplomat_Dlg->diplomat_target_id, 0);
+                            pDiplomat_Dlg->diplomat_target_id, 0);
   }
 
   popdown_diplomat_dialog();
@@ -148,10 +151,10 @@ static int spy_sabotage_request(struct widget *pWidget)
 static int diplomat_sabotage_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
       request_diplomat_action(DIPLOMAT_SABOTAGE, pDiplomat_Dlg->diplomat_id,
-                                         pDiplomat_Dlg->diplomat_target_id, 0);
+                              pDiplomat_Dlg->diplomat_target_id, B_LAST + 1);
     }
     
     popdown_diplomat_dialog();
@@ -181,10 +184,11 @@ static int spy_steal_callback(struct widget *pWidget)
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
     int steal_advance = MAX_ID - pWidget->ID;
   
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
       request_diplomat_action(DIPLOMAT_STEAL, pDiplomat_Dlg->diplomat_id,
-                              pDiplomat_Dlg->diplomat_target_id, steal_advance);
+                              pDiplomat_Dlg->diplomat_target_id,
+                              steal_advance);
     }
     
     popdown_diplomat_dialog();
@@ -220,21 +224,22 @@ static int spy_steal_popup(struct widget *pWidget)
   }
   
   count = 0;
-  for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-    if (tech_is_available(game.player_ptr, i)
-      && get_invention(pVictim, i)==TECH_KNOWN
-      && (get_invention(game.player_ptr, i)==TECH_UNKNOWN
-      || get_invention(game.player_ptr, i)==TECH_REACHABLE)) {
-	count++;
-      }
-  }
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_reachable(client.conn.playing, i, FALSE)
+     && TECH_KNOWN == player_invention_state(pVictim, i)
+     && (TECH_UNKNOWN == player_invention_state(client.conn.playing, i)
+         || TECH_PREREQS_KNOWN ==
+              player_invention_state(client.conn.playing, i))) {
+      count++;
+    }
+  } advance_index_iterate_end;
   
   if(!count) {    
     /* if there is no known tech to steal then 
        send steal order at Spy's Discretion */
     int target_id = pVcity->id;
 
-    request_diplomat_action(DIPLOMAT_STEAL, id, target_id, game.control.num_tech_types);
+    request_diplomat_action(DIPLOMAT_STEAL, id, target_id, advance_count());
     return -1;
   }
     
@@ -264,8 +269,9 @@ static int spy_steal_popup(struct widget *pWidget)
   /* ------------------ */
   /* exit button */
   pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			  WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+                          WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+  pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
+                                            adj_font(12));
   area.w += pBuf->size.w + adj_size(10);
   pBuf->action = exit_spy_steal_dlg_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
@@ -303,14 +309,15 @@ static int spy_steal_popup(struct widget *pWidget)
   pStr->style |= (TTF_STYLE_BOLD | SF_CENTER);
   
   count = 0;
-  for(i=A_FIRST; i<game.control.num_tech_types; i++) {
-    if (tech_is_available(game.player_ptr, i)
-      && get_invention(pVictim, i)==TECH_KNOWN
-      && (get_invention(game.player_ptr, i)==TECH_UNKNOWN
-      || get_invention(game.player_ptr, i)==TECH_REACHABLE)) {
-    
-      count++;  
-      copy_chars_to_string16(pStr, advance_name_translation(i));
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_reachable(client.conn.playing, i, FALSE)
+     && TECH_KNOWN == player_invention_state(pVictim, i)
+     && (TECH_UNKNOWN == player_invention_state(client.conn.playing, i)
+         || TECH_PREREQS_KNOWN ==
+              player_invention_state(client.conn.playing, i))) {
+      count++;
+
+      copy_chars_to_string16(pStr, advance_name_translation(advance_by_number(i)));
       pSurf = create_sellect_tech_icon(pStr, i, FULL_MODE);
       pBuf = create_icon2(pSurf, pWindow->dst,
       		WF_FREE_THEME | WF_RESTORE_BACKGROUND);
@@ -325,10 +332,10 @@ static int spy_steal_popup(struct widget *pWidget)
         set_wflag(pBuf, WF_HIDDEN);
       }
     }
-  }
+  } advance_index_iterate_end;
   
   /* get spy tech */
-  i = unit_type(game_find_unit_by_number(id))->tech_requirement;
+  i = advance_number(unit_type(game_unit_by_number(id))->require_advance);
   copy_chars_to_string16(pStr, _("At Spy's Discretion"));
   pSurf = create_sellect_tech_icon(pStr, i, FULL_MODE);
 	
@@ -338,7 +345,7 @@ static int spy_steal_popup(struct widget *pWidget)
   pBuf->action = spy_steal_callback;
   pBuf->data.cont = pCont;
     
-  add_to_gui_list(MAX_ID - game.control.num_tech_types, pBuf);
+  add_to_gui_list(MAX_ID - advance_count(), pBuf);
   count++;
   
   /* --------------------------------------------------------- */
@@ -407,10 +414,10 @@ static int spy_steal_popup(struct widget *pWidget)
 static int diplomat_steal_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
       request_diplomat_action(DIPLOMAT_STEAL, pDiplomat_Dlg->diplomat_id,
-                                     pDiplomat_Dlg->diplomat_target_id, A_UNSET);
+                              pDiplomat_Dlg->diplomat_target_id, A_UNSET);
     }
     
     popdown_diplomat_dialog();  
@@ -424,11 +431,10 @@ static int diplomat_steal_callback(struct widget *pWidget)
 static int diplomat_incite_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
-      request_diplomat_answer(DIPLOMAT_INCITE,
-			      pDiplomat_Dlg->diplomat_id,
-			      pDiplomat_Dlg->diplomat_target_id, 0);
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pDiplomat_Dlg->diplomat_target_id)) {
+      request_diplomat_answer(DIPLOMAT_INCITE, pDiplomat_Dlg->diplomat_id,
+                              pDiplomat_Dlg->diplomat_target_id, 0);
     }
     
     popdown_diplomat_dialog();
@@ -445,10 +451,10 @@ static int diplomat_keep_moving_callback(struct widget *pWidget)
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
     struct unit *punit;
     struct city *pcity;
-    
-    if( (punit=game_find_unit_by_number(pDiplomat_Dlg->diplomat_id))
-        && (pcity=game_find_city_by_number(pDiplomat_Dlg->diplomat_target_id))
-        && !same_pos(punit->tile, pcity->tile)) {
+
+    if ((punit = game_unit_by_number(pDiplomat_Dlg->diplomat_id))
+        && (pcity = game_city_by_number(pDiplomat_Dlg->diplomat_target_id))
+        && !same_pos(unit_tile(punit), city_tile(pcity))) {
       request_diplomat_action(DIPLOMAT_MOVE, pDiplomat_Dlg->diplomat_id,
                               pDiplomat_Dlg->diplomat_target_id, 0);
     }
@@ -464,12 +470,10 @@ static int diplomat_keep_moving_callback(struct widget *pWidget)
 static int diplomat_bribe_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-  
-    if (game_find_unit_by_number(pDiplomat_Dlg->diplomat_id)
-       && game_find_unit_by_number(pDiplomat_Dlg->diplomat_target_id)) {  
-      request_diplomat_answer(DIPLOMAT_BRIBE,
-			      pDiplomat_Dlg->diplomat_id,
-			      pDiplomat_Dlg->diplomat_target_id, 0);
+    if (NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_id)
+        && NULL != game_unit_by_number(pDiplomat_Dlg->diplomat_target_id)) {
+      request_diplomat_answer(DIPLOMAT_BRIBE, pDiplomat_Dlg->diplomat_id,
+                              pDiplomat_Dlg->diplomat_target_id, 0);
     }
     
     popdown_diplomat_dialog();
@@ -538,7 +542,7 @@ void popup_diplomat_dialog(struct unit *pUnit, struct tile *ptile)
   }
   
   is_unit_move_blocked = TRUE;
-  pCity = tile_get_city(ptile);
+  pCity = tile_city(ptile);
   spy = unit_has_type_flag(pUnit, F_SPY);
   
   pDiplomat_Dlg = fc_calloc(1, sizeof(struct diplomat_dialog));
@@ -805,9 +809,9 @@ static int sabotage_impr_callback(struct widget *pWidget)
     {
       sabotage_improvement = -1;
     }
-    
-    if(game_find_unit_by_number(diplomat_id)
-      && game_find_city_by_number(diplomat_target_id)) { 
+
+    if (NULL != game_unit_by_number(diplomat_id)
+        && NULL != game_city_by_number(diplomat_target_id)) {
       request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
                               diplomat_target_id, sabotage_improvement + 1);
     }
@@ -860,8 +864,9 @@ void popup_sabotage_dialog(struct city *pCity)
   /* ---------- */
   /* exit button */
   pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			  WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+                          WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+  pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
+                                            adj_font(12));
   area.w += pBuf->size.w + adj_size(10);
   pBuf->action = diplomat_close_callback;
   set_wstate(pBuf, FC_WS_NORMAL);
@@ -887,16 +892,16 @@ void popup_sabotage_dialog(struct city *pCity)
   area.h += pBuf->next->size.h;
   /* ------------------ */
   n = 0;
-  built_impr_iterate(pCity, imp) {
-    if (improvement_by_number(imp)->sabotage > 0) {
+  city_built_iterate(pCity, pImprove) {
+    if (pImprove->sabotage > 0) {
       
       create_active_iconlabel(pBuf, pWindow->dst, pStr,
-	      (char *) get_impr_name_ex(pCity, imp),
+	      (char *) city_improvement_name_translation(pCity, pImprove),
 				      sabotage_impr_callback);
       pBuf->data.cont = pCont;
       set_wstate(pBuf , FC_WS_NORMAL);
   
-      add_to_gui_list(MAX_ID - imp, pBuf);
+      add_to_gui_list(MAX_ID - improvement_number(pImprove), pBuf);
     
       area.w = MAX(area.w , pBuf->size.w);
       imp_h += pBuf->size.h;
@@ -906,7 +911,7 @@ void popup_sabotage_dialog(struct city *pCity)
 	pDiplomat_Dlg->pdialog->pEndActiveWidgetList = pBuf;
       }
     
-      if (imp > 9)
+      if (improvement_number(pImprove) > 9)
       {
         set_wflag(pBuf, WF_HIDDEN);
       }
@@ -914,7 +919,7 @@ void popup_sabotage_dialog(struct city *pCity)
       n++;    
       /* ----------- */
     }  
-  } built_impr_iterate_end;
+  } city_built_iterate_end;
 
   pDiplomat_Dlg->pdialog->pBeginActiveWidgetList = pBuf;
   
@@ -1064,10 +1069,10 @@ static int incite_dlg_window_callback(struct widget *pWindow)
 static int diplomat_incite_yes_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pIncite_Dlg->diplomat_id)
-       && game_find_city_by_number(pIncite_Dlg->diplomat_target_id)) {  
+    if (NULL != game_unit_by_number(pIncite_Dlg->diplomat_id)
+        && NULL != game_city_by_number(pIncite_Dlg->diplomat_target_id)) {
       request_diplomat_action(DIPLOMAT_INCITE, pIncite_Dlg->diplomat_id,
-                                         pIncite_Dlg->diplomat_target_id, 0);       
+                              pIncite_Dlg->diplomat_target_id, 0);
     }
     popdown_incite_dialog();
   }  
@@ -1110,7 +1115,7 @@ void popup_incite_dialog(struct city *pCity, int cost)
   struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_String16 *pStr;
   struct unit *pUnit;
-  char cBuf[255]; 
+  char tBuf[255], cBuf[255];
   bool exit = FALSE;
   SDL_Rect area;
   
@@ -1131,7 +1136,12 @@ void popup_incite_dialog(struct city *pCity, int cost)
   pIncite_Dlg->diplomat_id = pUnit->id;
   pIncite_Dlg->diplomat_target_id = pCity->id;
   pIncite_Dlg->pdialog = fc_calloc(1, sizeof(struct SMALL_DLG));  
-  
+
+  fc_snprintf(tBuf, ARRAY_SIZE(tBuf), PL_("Treasury contains %d gold.",
+                                          "Treasury contains %d gold.",
+                                          client_player()->economic.gold),
+              client_player()->economic.gold);
+
   /* window */
   pStr = create_str16_from_char(_("Incite a Revolt!"), adj_font(12));
     
@@ -1153,8 +1163,10 @@ void popup_incite_dialog(struct city *pCity, int cost)
     
     /* exit button */
     pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			    WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);  
-    pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+                            WF_WIDGET_HAS_INFO_LABEL
+                            | WF_RESTORE_BACKGROUND);
+    pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
+                                              adj_font(12));
     area.w += pBuf->size.w + adj_size(10);
     pBuf->action = exit_incite_dlg_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
@@ -1164,7 +1176,7 @@ void popup_incite_dialog(struct city *pCity, int cost)
     exit = TRUE;
     /* --------------- */
     
-    my_snprintf(cBuf, sizeof(cBuf), _("You can't incite a revolt in %s."),
+    fc_snprintf(cBuf, sizeof(cBuf), _("You can't incite a revolt in %s."),
 		city_name(pCity));
 
     create_active_iconlabel(pBuf, pWindow->dst, pStr, cBuf, NULL);
@@ -1182,11 +1194,12 @@ void popup_incite_dialog(struct city *pCity, int cost)
     area.w = MAX(area.w , pBuf->size.w);
     area.h += pBuf->size.h;
     
-  } else if (game.player_ptr->economic.gold >= cost) {
-    my_snprintf(cBuf, sizeof(cBuf),
-		_("Incite a revolt for %d gold?\nTreasury contains %d gold."), 
-		cost, game.player_ptr->economic.gold);
-    
+  } else if (cost <= client_player()->economic.gold) {
+    fc_snprintf(cBuf, sizeof(cBuf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Incite a revolt for %d gold?\n%s",
+                    "Incite a revolt for %d gold?\n%s", cost), cost, tBuf);
+
     create_active_iconlabel(pBuf, pWindow->dst, pStr, cBuf, NULL);
         
   
@@ -1221,8 +1234,10 @@ void popup_incite_dialog(struct city *pCity, int cost)
   } else {
     /* exit button */
     pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			    WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+                            WF_WIDGET_HAS_INFO_LABEL
+                            | WF_RESTORE_BACKGROUND);
+    pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
+                                              adj_font(12));
     area.w += pBuf->size.w + adj_size(10);
     pBuf->action = exit_incite_dlg_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
@@ -1232,11 +1247,11 @@ void popup_incite_dialog(struct city *pCity, int cost)
     exit = TRUE;
     /* --------------- */
 
-    my_snprintf(cBuf, sizeof(cBuf),
-		_("Inciting a revolt costs %d gold.\n"
-		  "Treasury contains %d gold."), 
-		cost, game.player_ptr->economic.gold);
-    
+    fc_snprintf(cBuf, sizeof(cBuf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Inciting a revolt costs %d gold.\n%s",
+                    "Inciting a revolt costs %d gold.\n%s", cost), cost, tBuf);
+
     create_active_iconlabel(pBuf, pWindow->dst, pStr, cBuf, NULL);
         
   
@@ -1308,10 +1323,10 @@ static int bribe_dlg_window_callback(struct widget *pWindow)
 static int diplomat_bribe_yes_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    if (game_find_unit_by_number(pIncite_Dlg->diplomat_id)
-       && game_find_unit_by_number(pIncite_Dlg->diplomat_target_id)) {  
-       request_diplomat_action(DIPLOMAT_BRIBE, pBribe_Dlg->diplomat_id,
-                                          pBribe_Dlg->diplomat_target_id, 0);       
+    if (NULL != game_unit_by_number(pBribe_Dlg->diplomat_id)
+        && NULL != game_unit_by_number(pBribe_Dlg->diplomat_target_id)) {
+      request_diplomat_action(DIPLOMAT_BRIBE, pBribe_Dlg->diplomat_id,
+                              pBribe_Dlg->diplomat_target_id, 0);
     }
     popdown_bribe_dialog();
   }  
@@ -1351,7 +1366,7 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
   struct widget *pWindow = NULL, *pBuf = NULL;
   SDL_String16 *pStr;
   struct unit *pDiplomatUnit;
-  char cBuf[255]; 
+  char tBuf[255], cBuf[255];
   bool exit = FALSE;
   SDL_Rect area;
   
@@ -1372,7 +1387,12 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
   pBribe_Dlg->diplomat_id = pDiplomatUnit->id;
   pBribe_Dlg->diplomat_target_id = pUnit->id;
   pBribe_Dlg->pdialog = fc_calloc(1, sizeof(struct SMALL_DLG));
-  
+
+  fc_snprintf(tBuf, ARRAY_SIZE(tBuf), PL_("Treasury contains %d gold.",
+                                          "Treasury contains %d gold.",
+                                          client_player()->economic.gold),
+              client_player()->economic.gold);
+
   /* window */
   pStr = create_str16_from_char(_("Bribe Enemy Unit"), adj_font(12));
     
@@ -1390,11 +1410,12 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
   area.w = MAX(area.w, adj_size(8));
   area.h = MAX(area.h, adj_size(2));
   
-  if (game.player_ptr->economic.gold >= cost) {
-    my_snprintf(cBuf, sizeof(cBuf),
-		_("Bribe unit for %d gold?\nTreasury contains %d gold."), 
-		cost, game.player_ptr->economic.gold);
-    
+  if (cost <= client_player()->economic.gold) {
+    fc_snprintf(cBuf, sizeof(cBuf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Bribe unit for %d gold?\n%s",
+                    "Bribe unit for %d gold?\n%s", cost), cost, tBuf);
+
     create_active_iconlabel(pBuf, pWindow->dst, pStr, cBuf, NULL);
   
     add_to_gui_list(ID_LABEL, pBuf);
@@ -1427,8 +1448,10 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
   } else {
     /* exit button */
     pBuf = create_themeicon(pTheme->Small_CANCEL_Icon, pWindow->dst,
-  			    WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-    pBuf->string16 = create_str16_from_char(_("Close Dialog (Esc)"), adj_font(12));
+                            WF_WIDGET_HAS_INFO_LABEL
+                            | WF_RESTORE_BACKGROUND);
+    pBuf->info_label = create_str16_from_char(_("Close Dialog (Esc)"),
+                                              adj_font(12));
     area.w += pBuf->size.w + adj_size(10);
     pBuf->action = exit_bribe_dlg_callback;
     set_wstate(pBuf, FC_WS_NORMAL);
@@ -1438,11 +1461,11 @@ void popup_bribe_dialog(struct unit *pUnit, int cost)
     exit = TRUE;
     /* --------------- */
 
-    my_snprintf(cBuf, sizeof(cBuf),
-		_("Bribing the unit costs %d gold.\n"
-		  "Treasury contains %d gold."), 
-		cost, game.player_ptr->economic.gold);
-    
+    fc_snprintf(cBuf, sizeof(cBuf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Bribing the unit costs %d gold.\n%s",
+                    "Bribing the unit costs %d gold.\n%s", cost), cost, tBuf);
+
     create_active_iconlabel(pBuf, pWindow->dst, pStr, cBuf, NULL);
   
     add_to_gui_list(ID_LABEL, pBuf);
