@@ -37,7 +37,7 @@
 #include "climisc.h"
 #include "overview_common.h"
 
-/* gui-sdl */
+/* client/gui-sdl */
 #include "citydlg.h"
 #include "cityrep.h"
 #include "colors.h"
@@ -53,6 +53,7 @@
 #include "menu.h"
 #include "messagewin.h"
 #include "optiondlg.h"
+#include "pages.h"
 #include "plrdlg.h"
 #include "repodlgs.h"
 #include "sprite.h"
@@ -143,7 +144,7 @@ static int units_action_callback(struct widget *pWidget)
     set_wstate(pWidget, FC_WS_NORMAL);
     widget_redraw(pWidget);
     widget_mark_dirty(pWidget);
-    popup_activeunits_report_dialog(FALSE);
+    units_report_dialog_popup(FALSE);
   }
   return -1;
 }
@@ -170,7 +171,7 @@ static int cities_action_callback(struct widget *pButton)
         popup_find_dialog();
       break;
       default:
-        popup_city_report_dialog(FALSE);
+        city_report_dialog_popup(FALSE);
       break;
     }
   } else {
@@ -214,7 +215,7 @@ static int revolution_callback(struct widget *pButton)
 static int research_callback(struct widget *pButton)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    popup_science_dialog(TRUE);
+    science_report_dialog_popup(TRUE);
   }
   return -1;
 }
@@ -225,7 +226,7 @@ static int research_callback(struct widget *pButton)
 static int economy_callback(struct widget *pButton)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) {
-    popup_economy_report_dialog(FALSE);
+    economy_report_dialog_popup(FALSE);
   }
   return -1;
 }
@@ -270,7 +271,8 @@ static int toggle_unit_info_window_callback(struct widget *pIcon_Widget)
       /* new button direction */
       alphablit(pTheme->L_ARROW_Icon, NULL, pIcon_Widget->theme, NULL);
   
-      copy_chars_to_string16(pIcon_Widget->string16, _("Show Unit Info Window"));
+      copy_chars_to_string16(pIcon_Widget->info_label,
+                             _("Show Unit Info Window"));
           
       SDL_Client_Flags &= ~CF_UNITINFO_SHOWN;
   
@@ -326,7 +328,8 @@ static int toggle_unit_info_window_callback(struct widget *pIcon_Widget)
         pSellected_Widget = NULL;
                     
         /* SHOW */
-        copy_chars_to_string16(pIcon_Widget->string16, _("Hide Unit Info Window"));
+        copy_chars_to_string16(pIcon_Widget->info_label,
+                               _("Hide Unit Info Window"));
          
         alphablit(pTheme->R_ARROW_Icon, NULL, pIcon_Widget->theme, NULL);
   
@@ -379,9 +382,9 @@ static int toggle_map_window_callback(struct widget *pMap_Button)
   
       set_wstate(pMap_Button, FC_WS_NORMAL);
       pSellected_Widget = NULL;
-      
-      copy_chars_to_string16(pMap_Button->string16, _("Show Mini Map"));
-          
+
+      copy_chars_to_string16(pMap_Button->info_label, _("Show Mini Map"));
+
       /* make new map icon */
       alphablit(pTheme->R_ARROW_Icon, NULL, pMap_Button->theme, NULL);
   
@@ -458,8 +461,8 @@ static int toggle_map_window_callback(struct widget *pMap_Button)
         pSellected_Widget = NULL;
         
         /* show MiniMap */
-        copy_chars_to_string16(pMap_Button->string16, _("Hide Mini Map"));
-          
+        copy_chars_to_string16(pMap_Button->info_label, _("Hide Mini Map"));
+
         alphablit(pTheme->L_ARROW_Icon, NULL, pMap_Button->theme, NULL);
         SDL_Client_Flags |= CF_OVERVIEW_SHOWN;
               
@@ -508,12 +511,12 @@ static int toggle_minimap_mode_callback(struct widget *pWidget)
 static int toggle_msg_window_callback(struct widget *pWidget)
 {
   if (Main.event.button.button == SDL_BUTTON_LEFT) { 
-    if (is_meswin_open()) {
-      popdown_meswin_dialog();
-      copy_chars_to_string16(pWidget->string16, _("Show Messages (F9)"));
+    if (meswin_dialog_is_open()) {
+      meswin_dialog_popdown();
+      copy_chars_to_string16(pWidget->info_label, _("Show Messages (F9)"));
     } else {
-      popup_meswin_dialog(true);
-      copy_chars_to_string16(pWidget->string16, _("Hide Messages (F9)"));
+      meswin_dialog_popup(TRUE);
+      copy_chars_to_string16(pWidget->info_label, _("Hide Messages (F9)"));
     }
   
     pSellected_Widget = pWidget;
@@ -538,7 +541,7 @@ int resize_minimap(void)
     popdown_minimap_window();
     popup_minimap_window();
     refresh_overview();
-    update_menus();
+    menus_update();
   }
   
   return 0;
@@ -578,7 +581,7 @@ static int up_width_callback(struct widget *pWidget)
          (pMiniMap_Window->size.w - pMiniMap_Window->area.w) + BLOCKM_W) <= 
          pUnits_Info_Window->dst->dest_rect.x) {
       char cBuf[4];
-      my_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_WIDTH);
+      fc_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_WIDTH);
       copy_chars_to_string16(pWidget->next->string16, cBuf);
       widget_redraw(pWidget->next);
       widget_mark_dirty(pWidget->next);
@@ -598,7 +601,7 @@ static int down_width_callback(struct widget *pWidget)
     widget_mark_dirty(pWidget);
     if (OVERVIEW_TILE_WIDTH > 1) {
       char cBuf[4];
-      my_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_WIDTH);
+      fc_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_WIDTH);
       copy_chars_to_string16(pWidget->prev->string16, cBuf);
       widget_redraw(pWidget->prev);
       widget_mark_dirty(pWidget->prev);
@@ -620,7 +623,7 @@ static int up_height_callback(struct widget *pWidget)
       char cBuf[4];
         
       OVERVIEW_TILE_HEIGHT++;
-      my_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_HEIGHT);
+      fc_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_HEIGHT);
       copy_chars_to_string16(pWidget->next->string16, cBuf);
       widget_redraw(pWidget->next);
       widget_mark_dirty(pWidget->next);
@@ -640,7 +643,7 @@ static int down_height_callback(struct widget *pWidget)
       char cBuf[4];
       
       OVERVIEW_TILE_HEIGHT--;
-      my_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_HEIGHT);
+      fc_snprintf(cBuf, sizeof(cBuf), "%d", OVERVIEW_TILE_HEIGHT);
       copy_chars_to_string16(pWidget->prev->string16, cBuf);
       widget_redraw(pWidget->prev);
       widget_mark_dirty(pWidget->prev);
@@ -694,7 +697,7 @@ static void popup_minimap_scale_dialog(void)
   set_wstate(pBuf, FC_WS_NORMAL);
   add_to_gui_list(ID_BUTTON, pBuf);
   
-  my_snprintf(cBuf, sizeof(cBuf), "%d" , OVERVIEW_TILE_WIDTH);
+  fc_snprintf(cBuf, sizeof(cBuf), "%d" , OVERVIEW_TILE_WIDTH);
   pStr = create_str16_from_char(cBuf, adj_font(24));
   pStr->style |= (TTF_STYLE_BOLD|SF_CENTER);
   pBuf = create_iconlabel(NULL, pWindow->dst, pStr, WF_RESTORE_BACKGROUND);
@@ -714,7 +717,7 @@ static void popup_minimap_scale_dialog(void)
   set_wstate(pBuf, FC_WS_NORMAL);
   add_to_gui_list(ID_BUTTON, pBuf);
   
-  my_snprintf(cBuf, sizeof(cBuf), "%d" , OVERVIEW_TILE_HEIGHT);
+  fc_snprintf(cBuf, sizeof(cBuf), "%d" , OVERVIEW_TILE_HEIGHT);
   pStr = create_str16_from_char(cBuf, adj_font(24));
   pStr->style |= (TTF_STYLE_BOLD|SF_CENTER);
   pBuf = create_iconlabel(NULL, pWindow->dst, pStr, WF_RESTORE_BACKGROUND);
@@ -940,7 +943,7 @@ int resize_unit_info(void)
   }
   
   if (C_S_RUNNING == client_state()) {
-    update_menus();
+    menus_update();
   }
   update_unit_info_label(get_units_in_focus());
       
@@ -1393,11 +1396,12 @@ void popup_unitinfo_window() {
   pUnits_Info_Window->private_data.adv_dlg = pUnitInfo_Dlg;
 
   /* economy button */
-  pWidget = create_icon2(get_tax_surface(O_GOLD), pUnits_Info_Window->dst, WF_FREE_GFX
-                      | WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND | WF_FREE_THEME);
+  pWidget = create_icon2(get_tax_surface(O_GOLD), pUnits_Info_Window->dst,
+                         WF_FREE_GFX | WF_WIDGET_HAS_INFO_LABEL
+                         | WF_RESTORE_BACKGROUND | WF_FREE_THEME);
 
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Economy"), "F5");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Economy"), "F5");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = economy_callback;
   pWidget->key = SDLK_F5;
 
@@ -1406,11 +1410,13 @@ void popup_unitinfo_window() {
   pTax_Button = pWidget; 
 
   /* research button */
-  pWidget = create_icon2(adj_surf(GET_SURF(client_research_sprite())), pUnits_Info_Window->dst, WF_FREE_GFX
-		       | WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND | WF_FREE_THEME);
+  pWidget = create_icon2(adj_surf(GET_SURF(client_research_sprite())),
+                         pUnits_Info_Window->dst,
+                         WF_FREE_GFX | WF_WIDGET_HAS_INFO_LABEL
+                         | WF_RESTORE_BACKGROUND | WF_FREE_THEME);
   /* TRANS: Research report action */
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Research"), "F6");                       
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Research"), "F6");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = research_callback;
   pWidget->key = SDLK_F6;
 
@@ -1419,10 +1425,12 @@ void popup_unitinfo_window() {
   pResearch_Button = pWidget;
 
   /* revolution button */
-  pWidget = create_icon2(adj_surf(GET_SURF(client_government_sprite())), pUnits_Info_Window->dst, (WF_FREE_GFX
-			| WF_WIDGET_HAS_INFO_LABEL| WF_RESTORE_BACKGROUND | WF_FREE_THEME));
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Revolution"), "Ctrl+Shift+R");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+  pWidget = create_icon2(adj_surf(GET_SURF(client_government_sprite())),
+                         pUnits_Info_Window->dst,
+                         WF_FREE_GFX | WF_WIDGET_HAS_INFO_LABEL
+                         | WF_RESTORE_BACKGROUND | WF_FREE_THEME);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Revolution"), "Ctrl+Shift+R");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = revolution_callback;
   pWidget->key = SDLK_r;
   pWidget->mod = KMOD_CTRL | KMOD_SHIFT;
@@ -1440,11 +1448,12 @@ void popup_unitinfo_window() {
   alphablit(pTheme->R_ARROW_Icon, NULL, pIcon_theme, NULL);
 
   pWidget = create_themeicon(pIcon_theme, pUnits_Info_Window->dst,
-			  WF_FREE_GFX | WF_FREE_THEME |
-		WF_RESTORE_BACKGROUND | WF_WIDGET_HAS_INFO_LABEL);
+                             WF_FREE_GFX | WF_FREE_THEME |
+                             WF_RESTORE_BACKGROUND
+                             | WF_WIDGET_HAS_INFO_LABEL);
+  pWidget->info_label = create_str16_from_char(_("Hide Unit Info Window"),
+                                               adj_font(12));
 
-  pWidget->string16 = create_str16_from_char(_("Hide Unit Info Window"), adj_font(12));
-  
   pWidget->action = toggle_unit_info_window_callback;
 
   add_to_gui_list(ID_TOGGLE_UNITS_WINDOW_BUTTON, pWidget);
@@ -1561,9 +1570,10 @@ void popup_minimap_window() {
 
   /* new turn button */
   pWidget = create_themeicon(pTheme->NEW_TURN_Icon, pMiniMap_Window->dst,
-			  WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Turn Done"), _("Shift+Return"));
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Turn Done"), _("Shift+Return"));
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = end_turn_callback;
   pWidget->key = SDLK_RETURN;
   pWidget->mod = KMOD_SHIFT;
@@ -1574,10 +1584,11 @@ void popup_minimap_window() {
 
   /* players button */
   pWidget = create_themeicon(pTheme->PLAYERS_Icon, pMiniMap_Window->dst,
-			     WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
   /* TRANS: Nations report action */
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Nations"), "F3");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Nations"), "F3");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = players_action_callback;
   pWidget->key = SDLK_F3;
 
@@ -1585,11 +1596,12 @@ void popup_minimap_window() {
 
   /* find city button */
   pWidget = create_themeicon(pTheme->FindCity_Icon, pMiniMap_Window->dst,
-   			     WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)\n%s\n%s (%s)", _("Cities Report"),
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)\n%s\n%s (%s)", _("Cities Report"),
                                 "F4", _("or"), _("Find City"), "Ctrl+F");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
-  pWidget->string16->style |= SF_CENTER;
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
+  pWidget->info_label->style |= SF_CENTER;
   pWidget->action = cities_action_callback;
   pWidget->key = SDLK_f;
   pWidget->mod = KMOD_CTRL;
@@ -1600,9 +1612,10 @@ void popup_minimap_window() {
 
   /* units button */
   pWidget = create_themeicon(pTheme->UNITS2_Icon, pMiniMap_Window->dst,
-		             WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Units"), "F2");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Units"), "F2");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = units_action_callback;
   pWidget->key = SDLK_F2;
 
@@ -1610,9 +1623,10 @@ void popup_minimap_window() {
 
   /* show/hide log window button */
   pWidget = create_themeicon(pTheme->LOG_Icon, pMiniMap_Window->dst,
- 			     WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Hide Messages"), "F9");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Hide Messages"), "F9");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = toggle_msg_window_callback;
   pWidget->key = SDLK_F9;
 
@@ -1620,9 +1634,10 @@ void popup_minimap_window() {
 
   /* toggle minimap mode button */
   pWidget = create_themeicon(pTheme->BORDERS_Icon, pMiniMap_Window->dst,
- 			     WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Toggle Mini Map Mode"), "Shift+\\");
-  pWidget->string16 = create_str16_from_char(buf, adj_font(12));
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Toggle Mini Map Mode"), "Shift+\\");
+  pWidget->info_label = create_str16_from_char(buf, adj_font(12));
   pWidget->action = toggle_minimap_mode_callback;
   pWidget->key = SDLK_BACKSLASH;
   pWidget->mod = KMOD_SHIFT;
@@ -1631,10 +1646,12 @@ void popup_minimap_window() {
 
   #ifdef SMALL_SCREEN
   /* options button */
-  pOptions_Button = create_themeicon(pTheme->Options_Icon, pMiniMap_Window->dst,
- 			             WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
-  my_snprintf(buf, sizeof(buf), "%s (%s)", _("Options"), "Esc");
-  pOptions_Button->string16 = create_str16_from_char(buf, adj_font(12));
+  pOptions_Button = create_themeicon(pTheme->Options_Icon,
+                                     pMiniMap_Window->dst,
+                                     WF_WIDGET_HAS_INFO_LABEL
+                                     | WF_RESTORE_BACKGROUND);
+  fc_snprintf(buf, sizeof(buf), "%s (%s)", _("Options"), "Esc");
+  pOptions_Button->info_label = create_str16_from_char(buf, adj_font(12));
   
   pOptions_Button->action = optiondlg_callback;  
   pOptions_Button->key = SDLK_ESCAPE;
@@ -1651,10 +1668,12 @@ void popup_minimap_window() {
   alphablit(pTheme->L_ARROW_Icon, NULL, pIcon_theme, NULL);
 
   pWidget = create_themeicon(pIcon_theme, pMiniMap_Window->dst,
-			  WF_FREE_GFX | WF_FREE_THEME |
-		          WF_WIDGET_HAS_INFO_LABEL | WF_RESTORE_BACKGROUND);
+                             WF_FREE_GFX | WF_FREE_THEME |
+                             WF_WIDGET_HAS_INFO_LABEL
+                             | WF_RESTORE_BACKGROUND);
 
-  pWidget->string16 = create_str16_from_char(_("Hide Mini Map"), adj_font(12));
+  pWidget->info_label = create_str16_from_char(_("Hide Mini Map"),
+                                               adj_font(12));
   pWidget->action = toggle_map_window_callback;
 
   add_to_gui_list(ID_TOGGLE_MAP_WINDOW_BUTTON, pWidget);
@@ -1843,7 +1862,7 @@ void show_game_page()
   
   /* cooling icon */
   pIcon_theme = adj_surf(GET_SURF(client_cooling_sprite()));
-  assert(pIcon_theme != NULL);
+  fc_assert(pIcon_theme != NULL);
   pWidget = create_iconlabel(pIcon_theme, Main.gui, NULL, WF_FREE_THEME);
 
 #ifdef SMALL_SCREEN
@@ -1860,7 +1879,7 @@ void show_game_page()
 
   /* warming icon */
   pIcon_theme = adj_surf(GET_SURF(client_warming_sprite()));
-  assert(pIcon_theme != NULL);
+  fc_assert(pIcon_theme != NULL);
 
   pWidget = create_iconlabel(pIcon_theme, Main.gui, NULL, WF_FREE_THEME);
 
@@ -2555,7 +2574,7 @@ static int move_new_city_dlg_callback(struct widget *pWindow)
   Popup a dialog to ask for the name of a new city.  The given string
   should be used as a suggestion.
 **************************************************************************/
-void popup_newcity_dialog(struct unit *pUnit, char *pSuggestname)
+void popup_newcity_dialog(struct unit *pUnit, const char *pSuggestname)
 {
   SDL_Surface *pBackground;
   struct SDL_String16 *pStr = NULL;
@@ -2571,7 +2590,7 @@ void popup_newcity_dialog(struct unit *pUnit, char *pSuggestname)
   }
 
   pSuggestedCityName = fc_calloc(1, strlen(pSuggestname) + 1);
-  mystrlcpy(pSuggestedCityName, pSuggestname, strlen(pSuggestname) + 1);
+  fc_strlcpy(pSuggestedCityName, pSuggestname, strlen(pSuggestname) + 1);
   
   pNewCity_Dlg = fc_calloc(1, sizeof(struct SMALL_DLG));
 
@@ -2697,7 +2716,7 @@ void popdown_newcity_dialog(void)
 **************************************************************************/
 void set_turn_done_button_state(bool state)
 {
-  if (C_S_RUNNING == client_state()) {
+  if (PAGE_GAME == get_current_client_page()) {
     if (state) {
       set_wstate(pNew_Turn_Button, FC_WS_NORMAL);
     } else {
