@@ -193,6 +193,9 @@ static int unit_order_callback(struct widget *pOrder_Widget)
     case ID_UNIT_ORDER_UPGRADE:
       popup_unit_upgrade_dlg(pUnit, FALSE);
       break;
+    case ID_UNIT_ORDER_CONVERT:
+      key_unit_convert();
+      break;
     case ID_UNIT_ORDER_DISBAND:
       key_unit_disband();
       break;
@@ -370,15 +373,15 @@ void create_units_order_widgets(void)
   size_t len;
   
   /* No orders */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("No Orders"),
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("No Orders"),
               /* TRANS: "Space" refers to the space bar on a keyboard. */
               _("Space"));
   pBuf = create_themeicon(pTheme->ODone_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_SPACE;
   add_to_gui_list(ID_UNIT_ORDER_DONE, pBuf);
   /* --------- */  
@@ -386,196 +389,208 @@ void create_units_order_widgets(void)
   pEndOrderWidgetList = pBuf;
 
   /* Wait */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Wait"), "W");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Wait"), "W");
   pBuf = create_themeicon(pTheme->OWait_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_w;
   add_to_gui_list(ID_UNIT_ORDER_WAIT, pBuf);
   /* --------- */  
 
   /* Explode Nuclear */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Explode Nuclear"), "Shift+N");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Explode Nuclear"), "Shift+N");
   pBuf = create_themeicon(pTheme->ONuke_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_n;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_NUKE, pBuf);
   /* --------- */
 
   /* Diplomat|Spy Actions */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Diplomat/Spy Actions"), "D");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Diplomat/Spy Actions"), "D");
   pBuf = create_themeicon(pTheme->OSpy_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_d;
   add_to_gui_list(ID_UNIT_ORDER_DIPLOMAT_DLG, pBuf);
   /* --------- */
 
   /* Disband */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Disband Unit"), "Shift+D");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Disband Unit"), "Shift+D");
   pBuf = create_themeicon(pTheme->ODisband_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_d;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_DISBAND, pBuf);
   /* --------- */  
 
   /* Upgrade */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Upgrade Unit"), "Shift+U");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Upgrade Unit"), "Shift+U");
   pBuf = create_themeicon(pTheme->Order_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_u;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_UPGRADE, pBuf);
   /* --------- */
 
-  /* Return to nearest city */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Return to Nearest City"), "Shift+G");
-  pBuf = create_themeicon(pTheme->OReturn_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+  /* Convert */
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Convert Unit"), "Shift+O");
+  pBuf = create_themeicon(pTheme->Order_Icon, Main.gui,
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->key = SDLK_o;
+  pBuf->mod = KMOD_SHIFT;
+  add_to_gui_list(ID_UNIT_ORDER_CONVERT, pBuf);
+  /* --------- */
+
+  /* Return to nearest city */
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Return to Nearest City"), "Shift+G");
+  pBuf = create_themeicon(pTheme->OReturn_Icon, Main.gui,
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
+  set_wstate(pBuf, FC_WS_NORMAL);
+  pBuf->action = unit_order_callback;
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_g;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_RETURN, pBuf);
   /* --------- */
   
   /* Goto City */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Go to City"), "T");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Go to City"), "T");
   pBuf = create_themeicon(pTheme->OGotoCity_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_t;
   add_to_gui_list(ID_UNIT_ORDER_GOTO_CITY, pBuf);
   /* --------- */
 
   /* Airlift */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Airlift to City"), "T");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Airlift to City"), "T");
   pBuf = create_themeicon(pTheme->Order_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_t;
   add_to_gui_list(ID_UNIT_ORDER_AIRLIFT, pBuf);
   /* --------- */
   
   /* Goto location */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Go to Tile"), "G");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Go to Tile"), "G");
   pBuf = create_themeicon(pTheme->OGoto_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_g;
   add_to_gui_list(ID_UNIT_ORDER_GOTO, pBuf);
   /* --------- */
 
   /* Patrol */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Patrol"), "Q");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Patrol"), "Q");
   pBuf = create_themeicon(pTheme->OPatrol_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_q;
   add_to_gui_list(ID_UNIT_ORDER_PATROL, pBuf);
   /* --------- */
 
   /* Connect irrigation */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Irrigation"), "Shift+I");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Irrigation"), "Shift+I");
   pBuf = create_themeicon(pTheme->OAutoConnect_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_i;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_CONNECT_IRRIGATE, pBuf);
   /* --------- */
 
   /* Connect road */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Road"), "Shift+R");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Road"), "Shift+R");
   pBuf = create_themeicon(pTheme->OAutoConnect_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_r;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_CONNECT_ROAD, pBuf);
   /* --------- */
 
   /* Connect railroad */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Rail"), "Shift+L");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Connect With Rail"), "Shift+L");
   pBuf = create_themeicon(pTheme->OAutoConnect_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_l;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_CONNECT_RAILROAD, pBuf);
   /* --------- */
 
   /* Auto-Explore */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Explore"), "X");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Explore"), "X");
   pBuf = create_themeicon(pTheme->OAutoExp_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 =
-      create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_x;
   add_to_gui_list(ID_UNIT_ORDER_AUTO_EXPLORE, pBuf);
   /* --------- */
 
   /* Auto-Attack / Auto-Settler */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Attack"), "A");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Attack"), "A");
   len = strlen(cBuf);
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Settler"), "A");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Settler"), "A");
   len = MAX(len, strlen(cBuf));
   
   pBuf = create_themeicon(pTheme->OAutoSett_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
   len = (len + 1) * sizeof(Uint16);
   unibuf = fc_calloc(1, len);
   convertcopy_to_utf16(unibuf, len, cBuf);
-  pBuf->string16 = create_string16(unibuf, len, adj_font(10));
+  pBuf->info_label = create_string16(unibuf, len, adj_font(10));
   pBuf->key = SDLK_a;
   add_to_gui_list(ID_UNIT_ORDER_AUTO_SETTLER, pBuf);
   
@@ -583,185 +598,185 @@ void create_units_order_widgets(void)
   /* --------- */    
   
   /* Wake Up Others */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unsentry All On Tile"), "Shift+S");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unsentry All On Tile"), "Shift+S");
   pBuf = create_themeicon(pTheme->OWakeUp_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_s;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_WAKEUP_OTHERS, pBuf);
   /* --------- */
 
   /* Unload */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unload All From Transporter"), "Shift+T");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unload All From Transporter"), "Shift+T");
   pBuf = create_themeicon(pTheme->OUnload_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->mod = KMOD_SHIFT;
   pBuf->key = SDLK_t;
   add_to_gui_list(ID_UNIT_ORDER_UNLOAD_TRANSPORTER, pBuf);
   /* --------- */
 
   /* Load */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Load Unit"), "L");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Load Unit"), "L");
   pBuf = create_themeicon(pTheme->OLoad_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_l;
   add_to_gui_list(ID_UNIT_ORDER_LOAD, pBuf);
   /* --------- */
 
   /* Unload */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unload Unit"), "U");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Unload Unit"), "U");
   pBuf = create_themeicon(pTheme->OUnload_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_u;
   add_to_gui_list(ID_UNIT_ORDER_UNLOAD, pBuf);
   /* --------- */
 
   /* Find Homecity */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Find Home City"), "H");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Find Home City"), "H");
   pBuf = create_themeicon(pTheme->OHomeCity_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_h;
   add_to_gui_list(ID_UNIT_ORDER_HOMECITY, pBuf);
   /* --------- */
 
   /* Pillage */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Pillage"), "Shift+P");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Pillage"), "Shift+P");
   pBuf = create_themeicon(pTheme->OPillage_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_p;
   pBuf->mod = KMOD_SHIFT;
   add_to_gui_list(ID_UNIT_ORDER_PILLAGE, pBuf);
   /* --------- */
 
   /* Sentry */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Sentry Unit"), "S");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Sentry Unit"), "S");
   pBuf = create_themeicon(pTheme->OSentry_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_s;
   add_to_gui_list(ID_UNIT_ORDER_SENTRY, pBuf);
   /* --------- */
 
   /* Clean Nuclear Fallout */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Clean Nuclear Fallout"), "N");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Clean Nuclear Fallout"), "N");
   pBuf = create_themeicon(pTheme->OFallout_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_n;
   add_to_gui_list(ID_UNIT_ORDER_FALLOUT, pBuf);
   /* --------- */
 
   /* Paradrop */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Drop Paratrooper"), "P");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Drop Paratrooper"), "P");
   pBuf = create_themeicon(pTheme->OParaDrop_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 =  create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label =  create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_p;
   add_to_gui_list(ID_UNIT_ORDER_PARADROP, pBuf);
   /* --------- */
 
   /* Clean Pollution */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Clean Pollution"), "P");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Clean Pollution"), "P");
   pBuf = create_themeicon(pTheme->OPollution_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_p;
   add_to_gui_list(ID_UNIT_ORDER_POLLUTION, pBuf);
   /* --------- */
 
   /* Build Airbase */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Airbase"), "E");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Airbase"), "E");
   pBuf = create_themeicon(pTheme->OAirBase_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_e;
   add_to_gui_list(ID_UNIT_ORDER_AIRBASE, pBuf);
   /* --------- */
 
   /* Fortify */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Fortify Unit"), "F");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Fortify Unit"), "F");
   pBuf = create_themeicon(pTheme->OFortify_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_f;
   add_to_gui_list(ID_UNIT_ORDER_FORTIFY, pBuf);
   /* --------- */
 
   /* Build Fortress */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Fortress"), "F");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Fortress"), "F");
   pBuf = create_themeicon(pTheme->OFortress_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_f;
   add_to_gui_list(ID_UNIT_ORDER_FORTRESS, pBuf);
   /* --------- */
 
   /* Transform Tile */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Transform Tile"), "O");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Transform Tile"), "O");
   pBuf = create_themeicon(pTheme->OTransform_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_o;
   pOrder_Transform_Button = pBuf;
   add_to_gui_list(ID_UNIT_ORDER_TRANSFORM, pBuf);
   /* --------- */
 
   /* Build Mine */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Mine"), "M");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Mine"), "M");
   pBuf = create_themeicon(pTheme->OMine_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_m;
   add_to_gui_list(ID_UNIT_ORDER_MINE, pBuf);
   
@@ -769,27 +784,27 @@ void create_units_order_widgets(void)
   /* --------- */    
 
   /* Build Irrigation */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Irrigation"), "I");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build Irrigation"), "I");
   pBuf = create_themeicon(pTheme->OIrrigation_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
   pBuf->key = SDLK_i;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   add_to_gui_list(ID_UNIT_ORDER_IRRIGATE, pBuf);
   
   pOrder_Irrigation_Button = pBuf;
   /* --------- */    
 
   /* Form Trade route */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Establish Trade Route"), "R");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Establish Trade Route"), "R");
   pBuf = create_themeicon(pTheme->OTrade_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_r;
   add_to_gui_list(ID_UNIT_ORDER_TRADE_ROUTE, pBuf);
 
@@ -797,24 +812,24 @@ void create_units_order_widgets(void)
   /* --------- */    
 
   /* Build (Rail-)Road */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Railroad"), "R", 999, 
 			PL_("turn", "turns", 999));
   len = strlen(cBuf);
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Road"), "R", 999, 
 			PL_("turn", "turns", 999));
   len = MAX(len, strlen(cBuf));
   
   pBuf = create_themeicon(pTheme->ORoad_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
   len = (len + 1) * sizeof(Uint16);
   unibuf = fc_calloc(1, len);
   convertcopy_to_utf16(unibuf, len, cBuf);
-  pBuf->string16 = create_string16(unibuf, len, adj_font(10));
+  pBuf->info_label = create_string16(unibuf, len, adj_font(10));
   pBuf->key = SDLK_r;
   add_to_gui_list(ID_UNIT_ORDER_ROAD, pBuf);
 
@@ -822,32 +837,32 @@ void create_units_order_widgets(void)
   /* --------- */  
   
   /* Help Build Wonder */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Help Build Wonder"), "B");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Help Build Wonder"), "B");
   pBuf = create_themeicon(pTheme->OWonder_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
-  pBuf->string16 = create_str16_from_char(cBuf, adj_font(10));
+  pBuf->info_label = create_str16_from_char(cBuf, adj_font(10));
   pBuf->key = SDLK_b;
   add_to_gui_list(ID_UNIT_ORDER_BUILD_WONDER, pBuf);
   /* --------- */  
 
   /* Add to City / Build New City */
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Add to City"), "B");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Add to City"), "B");
   len = strlen(cBuf);
-  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build City"), "B");
+  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build City"), "B");
   len = MAX(len, strlen(cBuf));
       
   pBuf = create_themeicon(pTheme->OBuildCity_Icon, Main.gui,
-			  (WF_HIDDEN | WF_RESTORE_BACKGROUND |
-			   WF_WIDGET_HAS_INFO_LABEL));
+                          WF_HIDDEN | WF_RESTORE_BACKGROUND
+                          | WF_WIDGET_HAS_INFO_LABEL);
   set_wstate(pBuf, FC_WS_NORMAL);
   pBuf->action = unit_order_callback;
   len = (len + 1) * sizeof(Uint16);
   unibuf = fc_calloc(1, len);
   convertcopy_to_utf16(unibuf, len, cBuf);
-  pBuf->string16 = create_string16(unibuf, len, adj_font(10));
+  pBuf->info_label = create_string16(unibuf, len, adj_font(10));
   pBuf->key = SDLK_b;
   add_to_gui_list(ID_UNIT_ORDER_BUILD_CITY, pBuf);
   
@@ -924,10 +939,19 @@ void free_bcgd_order_widgets(void)
 
 /* ============================== Native =============================== */
 
+/**************************************************************************
+  Initialize menus (sensitivity, name, etc.) based on the
+  current state and current ruleset, etc.  Call menus_update().
+**************************************************************************/
+void real_menus_init(void)
+{
+  /* PORTME */
+}
+
 /**************************************************************************
   Update all of the menus (sensitivity, etc.) based on the current state.
 **************************************************************************/
-void update_menus(void)
+void real_menus_update(void)
 {
   static Uint16 counter = 0;
   struct unit_list *punits = NULL;
@@ -998,7 +1022,7 @@ void update_menus(void)
     punits = get_units_in_focus();
     pUnit = unit_list_get(punits, 0);
 
-    if (pUnit && !pUnit->ai.control) {
+    if (pUnit && !pUnit->ai_controlled) {
       struct city *pHomecity;
       int time;
       struct tile *pTile = pUnit->tile;
@@ -1017,13 +1041,14 @@ void update_menus(void)
       /* Enable the button for adding to a city in all cases, so we
        * get an eventual error message from the server if we try. */
 
-      if (can_unit_add_or_build_city(pUnit)) {
+      if (unit_can_add_or_build_city(pUnit)) {
 	if(pCity) {
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Add to City"), "B");
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Add to City"), "B");
 	} else {
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build City"), "B");
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Build City"), "B");
 	}
-	copy_chars_to_string16(pOrder_Build_AddTo_City_Button->string16, cBuf);
+        copy_chars_to_string16(pOrder_Build_AddTo_City_Button->info_label,
+                               cBuf);
 	clear_wflag(pOrder_Build_AddTo_City_Button, WF_HIDDEN);
       } else {
 	set_wflag(pOrder_Build_AddTo_City_Button, WF_HIDDEN);
@@ -1039,18 +1064,18 @@ void update_menus(void)
       if (can_unit_do_activity(pUnit, ACTIVITY_ROAD) || time) {
 	if(time) {
 	  time = tile_activity_time(ACTIVITY_RAILROAD, pUnit->tile);
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Railroad"), "R", time , 
 			PL_("turn", "turns", time));
 	  pOrder_Road_Button->theme = pTheme->ORailRoad_Icon;
 	} else {
 	  time = tile_activity_time(ACTIVITY_ROAD, pUnit->tile);
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Road"), "R", time , 
 			PL_("turn", "turns", time));
 	  pOrder_Road_Button->theme = pTheme->ORoad_Icon;
 	}
-	copy_chars_to_string16(pOrder_Road_Button->string16, cBuf);
+        copy_chars_to_string16(pOrder_Road_Button->info_label, cBuf);
 	clear_wflag(pOrder_Road_Button, WF_HIDDEN);
       } else {
 	set_wflag(pOrder_Road_Button, WF_HIDDEN);
@@ -1058,27 +1083,27 @@ void update_menus(void)
       
 	/* unit_can_est_trade_route_here(pUnit) */
       if (pCity && unit_has_type_flag(pUnit, F_TRADE_ROUTE)
-        && (pHomecity = game_find_city_by_number(pUnit->homecity))
-	&& can_cities_trade(pHomecity, pCity)) {
+          && (pHomecity = game_city_by_number(pUnit->homecity))
+          && can_cities_trade(pHomecity, pCity)) {
 	int revenue = get_caravan_enter_city_trade_bonus(pHomecity, pCity);
 	
         if (can_establish_trade_route(pHomecity, pCity)) {
-          my_snprintf(cBuf, sizeof(cBuf),
+          fc_snprintf(cBuf, sizeof(cBuf),
                       _("Establish Trade Route With %s ( %d R&G + %d trade ) (R)"),
                       city_name(pHomecity),
                       revenue,
                       trade_between_cities(pHomecity, pCity));
         } else {
           revenue = (revenue + 2) / 3;
-          my_snprintf(cBuf, sizeof(cBuf),
+          fc_snprintf(cBuf, sizeof(cBuf),
                       _("Trade With %s ( %d R&G bonus ) (R)"),
                       city_name(pHomecity),
                       revenue);
         }
-	copy_chars_to_string16(pOrder_Trade_Button->string16, cBuf);
-	clear_wflag(pOrder_Trade_Button, WF_HIDDEN);
+        copy_chars_to_string16(pOrder_Trade_Button->info_label, cBuf);
+        clear_wflag(pOrder_Trade_Button, WF_HIDDEN);
       } else {
-	set_wflag(pOrder_Trade_Button, WF_HIDDEN);
+        set_wflag(pOrder_Trade_Button, WF_HIDDEN);
       }
 
       if (can_unit_do_activity(pUnit, ACTIVITY_IRRIGATE)) {
@@ -1087,26 +1112,26 @@ void update_menus(void)
         if (!strcmp(terrain_rule_name(pTerrain), "Forest") ||
           !strcmp(terrain_rule_name(pTerrain), "Jungle")) {
 	  /* set Crop Forest Icon */
-	  my_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
 			_("Cut Down to"),
                 terrain_name_translation(pTerrain->irrigation_result)
 			,"I", time , PL_("turn", "turns", time));
 	  pOrder_Irrigation_Button->theme = pTheme->OCutDownForest_Icon;
         }	else if (!strcmp(terrain_rule_name(pTerrain), "Swamp")) {
-	  my_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
 			_("Irrigate to"),
                 terrain_name_translation(pTerrain->irrigation_result)
 			,"I", time , PL_("turn", "turns", time));
 	  pOrder_Irrigation_Button->theme = pTheme->OIrrigation_Icon;
         } else {
 	  /* set Irrigation Icon */
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Irrigation"), "I", time , 
 			PL_("turn", "turns", time));
 	  pOrder_Irrigation_Button->theme = pTheme->OIrrigation_Icon;
 	}
 
-	copy_chars_to_string16(pOrder_Irrigation_Button->string16, cBuf);
+        copy_chars_to_string16(pOrder_Irrigation_Button->info_label, cBuf);
 	clear_wflag(pOrder_Irrigation_Button, WF_HIDDEN);
       } else {
 	set_wflag(pOrder_Irrigation_Button, WF_HIDDEN);
@@ -1118,7 +1143,7 @@ void update_menus(void)
 	/* FIXME: THIS CODE IS WRONG */
    if (!strcmp(terrain_rule_name(pTerrain), "Forest")) {  
 	  /* set Irrigate Icon -> make swamp */
-	  my_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
 			_("Irrigate to"),
 			terrain_name_translation(pTerrain->mining_result)
 			,"M", time , PL_("turn", "turns", time));
@@ -1128,20 +1153,20 @@ void update_menus(void)
               !strcmp(terrain_rule_name(pTerrain), "Grassland") ||
               !strcmp(terrain_rule_name(pTerrain), "Swamp")) {
 	  /* set Forest Icon -> plant Forrest*/
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Plant Forest"), "M", time , 
 			PL_("turn", "turns", time));
 	  pOrder_Mine_Button->theme = pTheme->OPlantForest_Icon;
    
    } else {
 	  /* set Mining Icon */
-	  my_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
+	  fc_snprintf(cBuf, sizeof(cBuf),"%s (%s) %d %s",
 			_("Build Mine"), "M", time , 
 			PL_("turn", "turns", time));
 	  pOrder_Mine_Button->theme = pTheme->OMine_Icon;
 	}
 
-        copy_chars_to_string16(pOrder_Mine_Button->string16, cBuf);
+        copy_chars_to_string16(pOrder_Mine_Button->info_label, cBuf);
 	clear_wflag(pOrder_Mine_Button, WF_HIDDEN);
       } else {
 	set_wflag(pOrder_Mine_Button, WF_HIDDEN);
@@ -1149,12 +1174,12 @@ void update_menus(void)
 
       if (can_unit_do_activity(pUnit, ACTIVITY_TRANSFORM)) {
 	time = tile_activity_time(ACTIVITY_TRANSFORM, pUnit->tile);
-	my_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
+	fc_snprintf(cBuf, sizeof(cBuf),"%s %s (%s) %d %s",
 	  _("Transform to"),
 	  terrain_name_translation(pTerrain->transform_result),
 			"M", time , 
 			PL_("turn", "turns", time));
-	copy_chars_to_string16(pOrder_Transform_Button->string16, cBuf);
+        copy_chars_to_string16(pOrder_Transform_Button->info_label, cBuf);
 	clear_wflag(pOrder_Transform_Button, WF_HIDDEN);
       } else {
 	set_wflag(pOrder_Transform_Button, WF_HIDDEN);
@@ -1244,15 +1269,17 @@ void update_menus(void)
       if (can_unit_do_autosettlers(pUnit)) {
 	if (unit_has_type_flag(pUnit, F_SETTLERS)) {
 	  if(pOrder_Automate_Unit_Button->theme != pTheme->OAutoSett_Icon) {
-	    my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Settler"), "A");
+	    fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Settler"), "A");
 	    pOrder_Automate_Unit_Button->theme = pTheme->OAutoSett_Icon;
-	    copy_chars_to_string16(pOrder_Automate_Unit_Button->string16, cBuf);
+            copy_chars_to_string16(pOrder_Automate_Unit_Button->info_label,
+                                   cBuf);
 	  }
 	} else {
 	  if(pOrder_Automate_Unit_Button->theme != pTheme->OAutoAtt_Icon) {
-	    my_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Attack"), "A");
+	    fc_snprintf(cBuf, sizeof(cBuf),"%s (%s)", _("Auto Attack"), "A");
 	    pOrder_Automate_Unit_Button->theme = pTheme->OAutoAtt_Icon;
-	    copy_chars_to_string16(pOrder_Automate_Unit_Button->string16, cBuf);
+            copy_chars_to_string16(pOrder_Automate_Unit_Button->info_label,
+                                   cBuf);
 	  }
 	}
 	clear_wflag(pOrder_Automate_Unit_Button, WF_HIDDEN);
@@ -1312,6 +1339,12 @@ void update_menus(void)
 	local_hide(ID_UNIT_ORDER_UPGRADE);
       }
 
+      if (unit_can_convert(pUnit)) {
+        local_show(ID_UNIT_ORDER_CONVERT);
+      } else {
+        local_hide(ID_UNIT_ORDER_CONVERT);
+      }
+
       set_new_order_widget_start_pos();
       counter = redraw_order_widgets();
 
