@@ -15,12 +15,20 @@
 #include <config.h>
 #endif
 
+#ifdef AUDIO_SDL
+#include "SDL.h"
+#endif
+
 #include <stdio.h>
 
+/* utility */
 #include "fciconv.h"
 #include "log.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
+#include "editgui_g.h"
+#include "ggz_g.h"
 #include "options.h"
 
 #include "gui_main.h"
@@ -30,12 +38,6 @@ const char *client_string = "gui-stub";
 const char * const gui_character_encoding = "UTF-8";
 const bool gui_use_transliteration = FALSE;
 
-client_option gui_options[] = {
-  /* None. */
-};
-const int num_gui_options = ARRAY_SIZE(gui_options);
-
-
 /****************************************************************************
   Called by the tileset code to set the font size that should be used to
   draw the city names and productions.
@@ -43,7 +45,7 @@ const int num_gui_options = ARRAY_SIZE(gui_options);
 void set_city_names_font_sizes(int my_city_names_font_size,
 			       int my_city_productions_font_size)
 {
-  freelog(LOG_ERROR, "Unimplemented set_city_names_font_sizes.");
+  log_error("Unimplemented set_city_names_font_sizes.");
   /* PORTME */
 }
 
@@ -55,6 +57,14 @@ void ui_init(void)
   /* PORTME */
 }
 
+/**************************************************************************
+  Entry point for whole freeciv client program.
+**************************************************************************/
+int main(int argc, char **argv)
+{
+  return client_main(argc, argv);
+}
+
 /**************************************************************************
   The main loop for the UI.  This is called from main(), and when it
   exits the client will exit.
@@ -65,6 +75,14 @@ void ui_main(int argc, char *argv[])
   fc_fprintf(stderr, "Freeciv rules!\n");
 }
 
+/****************************************************************************
+  Extra initializers for client options.
+****************************************************************************/
+void gui_options_extra_init(void)
+{
+  /* Nothing to do. */
+}
+
 /**************************************************************************
   Do any necessary UI-specific cleanup
 **************************************************************************/
@@ -73,10 +91,18 @@ void ui_exit()
   /* PORTME */
 }
 
+/**************************************************************************
+  Return our GUI type
+**************************************************************************/
+enum gui_type get_gui_type(void)
+{
+  return GUI_STUB;
+}
+
 /**************************************************************************
  Update the connected users list at pregame state.
 **************************************************************************/
-void update_conn_list_dialog(void)
+void real_conn_list_dialog_update(void)
 {
   /* PORTME */
 }
@@ -157,6 +183,16 @@ void set_unit_icons_more_arrow(bool onoff)
   /* PORTME */
 }
 
+/****************************************************************************
+  Called when the set of units in focus (get_units_in_focus()) changes.
+  Standard updates like update_unit_info_label() are handled in the platform-
+  independent code, so some clients will not need to do anything here.
+****************************************************************************/
+void real_focus_units_changed(void)
+{
+  /* PORTME */
+}
+
 /****************************************************************************
   Enqueue a callback to be called during an idle moment.  The 'callback'
   function should be called sometimes soon, and passed the 'data' pointer
@@ -167,6 +203,63 @@ void add_idle_callback(void (callback)(void *), void *data)
   /* PORTME */
 
   /* This is a reasonable fallback if it's not ported. */
-  freelog(LOG_ERROR, "Unimplemented add_idle_callback.");
+  log_error("Unimplemented add_idle_callback.");
   (callback)(data);
 }
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_tileset_changed(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_refresh(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popup_properties(const struct tile_list *tiles, int objtype)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popdown_all(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_changed(int objtype, int object_id, bool remove)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_created(int tag, int id)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_leave_table(void)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_ensure_server(void)
+{}
+
+
+/**************************************************************************
+  Updates a gui font style.
+**************************************************************************/
+void gui_update_font(const char *font_name, const char *font_value)
+{
+  /* PORTME */
+}
