@@ -23,23 +23,10 @@
 
 #include "repodlgs.h"
 
-/**************************************************************************
-  Update all report dialogs.
-**************************************************************************/
-void update_report_dialogs(void)
-{
-  if (!is_report_dialogs_frozen()) {
-    activeunits_report_dialog_update();
-    economy_report_dialog_update();
-    city_report_dialog_update(); 
-    science_dialog_update();
-  }
-}
-
 /**************************************************************************
   Update the science report.
 **************************************************************************/
-void science_dialog_update(void)
+void real_science_report_dialog_update(void)
 {
   /* PORTME */
 }
@@ -48,7 +35,7 @@ void science_dialog_update(void)
   Display the science report.  Optionally raise it.
   Typically triggered by F6.
 **************************************************************************/
-void popup_science_dialog(bool raise)
+void science_report_dialog_popup(bool raise)
 {
   /* PORTME */
 }
@@ -56,7 +43,7 @@ void popup_science_dialog(bool raise)
 /**************************************************************************
   Update the economy report.
 **************************************************************************/
-void economy_report_dialog_update(void)
+void real_economy_report_dialog_update(void)
 {
   /* PORTME */
 }
@@ -65,7 +52,7 @@ void economy_report_dialog_update(void)
   Display the economy report.  Optionally raise it.
   Typically triggered by F5.
 **************************************************************************/
-void popup_economy_report_dialog(bool raise)
+void economy_report_dialog_popup(bool raise)
 {
   /* PORTME */
 }
@@ -73,7 +60,7 @@ void popup_economy_report_dialog(bool raise)
 /**************************************************************************
   Update the units report.
 **************************************************************************/
-void activeunits_report_dialog_update(void)
+void real_units_report_dialog_update(void)
 {
   /* PORTME */
 }
@@ -82,7 +69,7 @@ void activeunits_report_dialog_update(void)
   Display the units report.  Optionally raise it.
   Typically triggered by F2.
 **************************************************************************/
-void popup_activeunits_report_dialog(bool raise)
+void units_report_dialog_popup(bool raise)
 {
   /* PORTME */
 }
@@ -90,15 +77,7 @@ void popup_activeunits_report_dialog(bool raise)
 /****************************************************************
   Show a dialog with player statistics at endgame.
 *****************************************************************/
-void popup_endgame_report_dialog(struct packet_endgame_report *packet)
-{
-  /* PORTME */
-}
-
-/*************************************************************************
-  Show a dialog with the server options.
-*************************************************************************/
-void popup_settable_options_dialog(void)
+void endgame_report_dialog_popup(const struct packet_endgame_report *packet)
 {
   /* PORTME */
 }
@@ -106,7 +85,7 @@ void popup_settable_options_dialog(void)
 /****************************************************************************
   Resize and redraw the requirement tree.
 ****************************************************************************/
-void science_dialog_redraw(void)
+void science_report_dialog_redraw(void)
 {
   /* PORTME */
 }
