@@ -18,15 +18,20 @@
 #include <windows.h>
 #include <windowsx.h>
 
-#include "events.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
+#include "log.h"
 #include "mem.h"
 #include "support.h"
 
+/* common */
+#include "events.h"
+#include "game.h"
+
+/* client */
 #include "chatline_g.h"
 #include "citydlg_g.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "cma_fec.h"
 #include "messagewin_g.h"
 
@@ -52,9 +57,9 @@ struct cma_dialog {
   HWND change;
   HWND perm;
   HWND release;
-  HWND minimal_surplus[NUM_STATS];
+  HWND minimal_surplus[O_LAST];
   HWND happy;
-  HWND factor[NUM_STATS + 1];
+  HWND factor[O_LAST + 1];
   int id;
 };
 
@@ -76,7 +81,7 @@ enum cmagui_ids {
 
 static void handle_hscroll(HWND win, HWND winctl, UINT code, int pos);
 
-static struct dialog_list dialog_list;
+static struct dialog_list *dialog_list;
 static bool dialog_list_has_been_initialised = FALSE;
 static int allow_refreshes = 1;
 
@@ -87,7 +92,7 @@ static int allow_refreshes = 1;
 static void ensure_initialised_dialog_list(void)
 {
   if (!dialog_list_has_been_initialised) {
-    dialog_list_init(&dialog_list);
+    dialog_list = dialog_list_new();
     dialog_list_has_been_initialised = TRUE;
   }
 }
@@ -203,8 +208,9 @@ static void create_cma_sliders(struct cma_dialog *pdialog,
 			       HWND win, struct fcwin_box *box)
 {
   struct fcwin_box *hbox = fcwin_hbox_new(win, FALSE);
-  int i;
   struct fcwin_box *vbox[3];
+  int i;
+
   fcwin_box_add_box(box, hbox, FALSE,FALSE, 0);
   for (i = 0; i < 3; i++) {
     vbox[i] = fcwin_vbox_new(win, TRUE);
@@ -213,9 +219,9 @@ static void create_cma_sliders(struct cma_dialog *pdialog,
   fcwin_box_add_static(vbox[0], " ",0, SS_LEFT,TRUE, TRUE, 0);
   fcwin_box_add_static(vbox[1], _("Minimal Surplus"), 0, SS_LEFT, TRUE, TRUE, 0);
   fcwin_box_add_static(vbox[2], _("Factor"), 0, SS_LEFT, TRUE, TRUE, 0);
-  for (i = 0; i< NUM_STATS; i++) {
-    cmagui_add_slider(pdialog, win, vbox, FALSE, cm_get_stat_name(i), i);
-  } 
+  output_type_iterate(i) {
+    cmagui_add_slider(pdialog, win, vbox, FALSE, get_output_name(i), i);
+  } output_type_iterate_end;
   cmagui_add_slider(pdialog, win, vbox, TRUE, _("Celebrate"), i);
 }
 
@@ -226,17 +232,16 @@ static void create_cma_sliders(struct cma_dialog *pdialog,
 static void set_hscales(const struct cm_parameter *const parameter,
                         struct cma_dialog *pdialog)
 {
-  int i;
   allow_refreshes = 0;
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     handle_hscroll(pdialog->mainwin, pdialog->minimal_surplus[i],
 		   SB_THUMBTRACK, parameter->minimal_surplus[i]);
     handle_hscroll(pdialog->mainwin, pdialog->factor[i],
 		   SB_THUMBTRACK, parameter->factor[i]);
-  }
+  } output_type_iterate_end;
   Button_SetCheck(pdialog->happy,
 		  parameter->require_happy ? BST_CHECKED : BST_UNCHECKED);
-  handle_hscroll(pdialog->mainwin, pdialog->factor[i],
+  handle_hscroll(pdialog->mainwin, pdialog->factor[O_LAST],
 		 SB_THUMBTRACK, parameter->happy_factor);
   allow_refreshes = 1;
 }
@@ -246,7 +251,7 @@ static void set_hscales(const struct cm_parameter *const parameter,
 **************************************************************************/
 void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
 {
-  struct cm_result result;
+  struct cm_result *result = cm_result_new(pcity);
   struct cm_parameter param;
   struct cma_dialog *pdialog = get_cma_dialog(pcity);
  
@@ -256,9 +261,9 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   if (!pdialog)
     return;
   /* fill in result label */
-  cm_copy_result_from_city(pcity, &result);
+  cm_result_from_main_map(result, pcity);
   SetWindowText(pdialog->result_label,
-		(char *) cmafec_get_result_descr(pcity, &result, &param));
+		(char *) cmafec_get_result_descr(pcity, result, &param));
   /* if called from a hscale, we _don't_ want to do this */
   if (refresh != DONT_REFRESH_HSCALES) {
     set_hscales(&param, pdialog);
@@ -286,6 +291,8 @@ void refresh_cma_dialog(struct city *pcity, enum cma_refresh refresh)
   EnableWindow(pdialog->release,
 	       can_client_issue_orders() &&
 	       controlled);
+
+  cm_result_destroy(result);
 }
 
 /**************************************************************************
@@ -411,12 +418,14 @@ static void cma_del_preset_callback(struct cma_dialog *pdialog)
 **************************************************************************/
 static void cma_change_to_callback(struct cma_dialog *pdialog)
 {
-  struct cm_result result;
+  struct cm_result *result = cm_result_new(pdialog->pcity);
   struct cm_parameter param;
 
   cmafec_get_fe_parameter(pdialog->pcity, &param);
-  cm_query_result(pdialog->pcity, &param, &result);
-  cma_apply_result(pdialog->pcity, &result);
+  cm_query_result(pdialog->pcity, &param, result);
+  cma_apply_result(pdialog->pcity, result);
+
+  cm_result_destroy(result);
 }
 
 
@@ -495,7 +504,7 @@ static struct cma_dialog * create_cma_gui(HWND win)
   fcwin_set_box(win, vbox);
   ensure_initialised_dialog_list();
   
-  dialog_list_insert(&dialog_list, pdialog);
+  dialog_list_prepend(dialog_list, pdialog);
   
   
   return pdialog;
@@ -508,20 +517,19 @@ static struct cma_dialog * create_cma_gui(HWND win)
 static void hscale_changed(struct cma_dialog *pdialog)
 {
   struct cm_parameter param;
-  int i;
 
   if (!allow_refreshes) {
     return;
   }
   
   cmafec_get_fe_parameter(pdialog->pcity, &param);
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     param.minimal_surplus[i] = ScrollBar_GetPos(pdialog->minimal_surplus[i]);
     param.factor[i] = ScrollBar_GetPos(pdialog->factor[i]);
-  }
+  } output_type_iterate_end;
   
   param.require_happy = (Button_GetCheck(pdialog->happy) == BST_CHECKED) ? 1 : 0;
-  param.happy_factor = ScrollBar_GetPos(pdialog->factor[NUM_STATS]);
+  param.happy_factor = ScrollBar_GetPos(pdialog->factor[O_LAST]);
   
   /* save the change */
   cmafec_set_fe_parameter(pdialog->pcity, &param);
@@ -569,7 +577,7 @@ static void handle_hscroll(HWND win, HWND winctl, UINT code, int pos)
   if (poscur < posmin) poscur = posmin;
   if (poscur > posmax) poscur = posmax;
   ScrollBar_SetPos(winctl, poscur, TRUE);
-  my_snprintf(buf, sizeof(buf), "%d", poscur);
+  fc_snprintf(buf, sizeof(buf), "%d", poscur);
   SetWindowText(GetNextSibling(winctl), buf);
 
 }
@@ -592,7 +600,7 @@ LONG CALLBACK cma_proc(HWND win, UINT message,
     case WM_DESTROY:
       break;
       if (guidata) {
-	dialog_list_unlink(&dialog_list, guidata->pdialog);
+	dialog_list_remove(dialog_list, guidata->pdialog);
 	free(guidata->pdialog);
 	free(guidata);
       }
