@@ -617,18 +617,18 @@ static bool test_alphablend()
     if ((AlphaBlend = GetProcAddress(hmsimg32, "AlphaBlend"))) {
       /* fall through, do nothing */
     } else {
-      freelog(LOG_TEST, "No AlphaBlend() in msimg32.dll, alpha blending disabled");
+      log_test("No AlphaBlend() in msimg32.dll, alpha blending disabled");
       return FALSE;
     }
   } else {
-    freelog(LOG_TEST, "No msimg32.dll, alpha blending disabled");
+    log_test("No msimg32.dll, alpha blending disabled");
     return FALSE;
   }
 
   hdc = GetDC(map_window);
 
   if (GetDeviceCaps(hdc, BITSPIXEL) < 32) {
-    freelog(LOG_TEST, "Not running in 32 bit color, alpha blending disabled");
+    log_test("Not running in 32 bit color, alpha blending disabled");
     ReleaseDC(map_window, hdc);
     return FALSE;
   }
@@ -638,7 +638,7 @@ static bool test_alphablend()
 #define SB_NONE 0
 
   if (GetDeviceCaps(hdc, SHADEBLENDCAPS) == SB_NONE) {
-    freelog(LOG_TEST, "Device does not support alpha blending, alpha blending disabled");
+    log_test("Device does not support alpha blending, alpha blending disabled");
     ReleaseDC(map_window, hdc);
     return FALSE;
   }
@@ -799,7 +799,7 @@ ui_main(int argc, char *argv[])
      * and do animations */
     if (idle && callbacks && callback_list_size(callbacks) > 0) {
       struct callback *cb = callback_list_get(callbacks, 0);
-      callback_list_unlink(callbacks, cb);
+      callback_list_remove(callbacks, cb);
       (cb->callback)(cb->data);
       free(cb);
 
@@ -814,7 +814,7 @@ ui_main(int argc, char *argv[])
 
   free_timer(anim_timer);
   free_timer(callback_timer);
-  callback_list_free(callbacks);
+  callback_list_destroy(callbacks);
 
   FreeLibrary(hmsimg32);
 }
@@ -838,7 +838,7 @@ enum gui_type get_gui_type(void)
 /**************************************************************************
  Update the connected users list at pregame state.
 **************************************************************************/
-void update_conn_list_dialog(void)
+void real_update_conn_list_dialog(void)
 {
   /* PORTME */
 }
