@@ -253,19 +253,19 @@ static void my_add_menu_accelerator(char *item,int cmd)
   plus++;
   tab++;
   /* fkeys */
-  if ((*plus == 'F') && (my_isdigit(plus[1]))) {
-    if (my_isdigit(plus[2]))
+  if ((*plus == 'F') && (fc_isdigit(plus[1]))) {
+    if (fc_isdigit(plus[2]))
       newaccel.key=VK_F10+(plus[2]-'0');
     else
       newaccel.key=VK_F1+(plus[1]-'1');
     newaccel.fVirt=FVIRTKEY;
   } else if (*plus) { /* standard ascii */
-    newaccel.key = my_toupper(*plus);
+    newaccel.key = fc_toupper(*plus);
     newaccel.fVirt=FVIRTKEY;
   } else {
     return;
   }
-  if (mystrncasecmp(plus,"Space",5)==0)
+  if (fc_strncasecmp(plus,"Space",5)==0)
     newaccel.key=VK_SPACE;
   /* Modifiers (Alt,Shift,Ctl) */
   if (strstr(tab, "Shift") != NULL) {
@@ -1031,7 +1031,7 @@ static const char *get_tile_change_menu_text(struct tile *ptile,
   Update the status and names of all menu items.
 **************************************************************************/
 void
-update_menus(void)
+menus_update(void)
 {
   enum MenuID id;
   HMENU menu;
@@ -1071,7 +1071,7 @@ update_menus(void)
     /* add new government entries. */
     id = IDM_GOVERNMENT_CHANGE_FIRST;
 
-    government_iterate(g) {
+    governments_iterate(g) {
       if (g != game.government_during_revolution) {
 	AppendMenu(govts, MF_STRING, id + government_number(g),
 		   government_name_translation(g));
@@ -1079,7 +1079,7 @@ update_menus(void)
 		       can_change_to_government(client.conn.playing, g)
 		       && can_client_issue_orders());
       }
-    } government_iterate_end;
+    } governments_iterate_end;
 
     my_enable_menu(menu, IDM_GOVERNMENT_FIND_CITY, TRUE);
 
@@ -1266,7 +1266,7 @@ update_menus(void)
         pterrain = tile_terrain(punit->tile);
         if (pterrain->irrigation_result != T_NONE
             && pterrain->irrigation_result != pterrain) {
-          my_snprintf(irrtext, sizeof(irrtext), irrfmt,
+          fc_snprintf(irrtext, sizeof(irrtext), irrfmt,
                       get_tile_change_menu_text(punit->tile,
                                                 ACTIVITY_IRRIGATE));
           sz_strlcat(irrtext, "\tI");
@@ -1277,13 +1277,13 @@ update_menus(void)
         }
         if (pterrain->mining_result != T_NONE
             && pterrain->mining_result != pterrain) {
-          my_snprintf(mintext, sizeof(mintext), minfmt,
+          fc_snprintf(mintext, sizeof(mintext), minfmt,
                       get_tile_change_menu_text(punit->tile, ACTIVITY_MINE));
           sz_strlcat(mintext, "\tM");
         }
         if (pterrain->transform_result != T_NONE
             && pterrain->transform_result != pterrain) {
-          my_snprintf(transtext, sizeof(transtext), transfmt,
+          fc_snprintf(transtext, sizeof(transtext), transfmt,
                       get_tile_change_menu_text(punit->tile,
                                                 ACTIVITY_TRANSFORM));
           sz_strlcat(transtext, "\tO");
