@@ -36,18 +36,23 @@
 #include "pixcomm.h"
 #include "canvas.h"
 
-#include "city.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
 #include "genlist.h"
-#include "map.h"
+#include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "player.h"
 #include "shared.h"
 #include "support.h"
 
-#include "civclient.h"
+/* common */
+#include "city.h"
+#include "game.h"
+#include "map.h"
+#include "packets.h"
+#include "player.h"
+
+/* client */
+#include "client_main.h"
 #include "cma_fec.h"
 
 #include "cityrep.h"
@@ -70,20 +75,16 @@
 Widget *stat_surplus_label, *stat_factor_label;
 Widget control_button, change_button, preset_list, cma_dialog;
 Widget celebrate_toggle, result_label, release_button;
-Widget surplus_slider[NUM_STATS], factor_slider[NUM_STATS + 1];
+Widget surplus_slider[O_LAST], factor_slider[O_LAST + 1];
 
-int minimal_surplus[NUM_STATS], factors[NUM_STATS + 1];
+int minimal_surplus[O_LAST], factors[O_LAST + 1];
 struct city *current_city;
 char *initial_preset_list[] = {
-      N_("For information on:"),
-      N_("CMA and presets"),
-      N_("including sample presets,"),
-      N_("see README.cma."),
-      "                  ",
-      "                  ",
-      "                  ",
-      "                  ",
-      0};
+	N_("For information on\n"
+	"the citizen governor and governor presets,\n"
+	"including sample presets,\n"
+	"see README.cma."),
+	NULL};
 
 
 static void update_cma_preset_list(void);
@@ -129,12 +130,12 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
   Widget form, preset_viewport, close_button;
   Widget surplus_header, factor_header, prev;
   struct cm_parameter parameter;
-  struct cm_result result;
+  struct cm_result *result = cm_result_new(pcity);
   int i;
 
   current_city = pcity;
-  stat_surplus_label = fc_malloc((NUM_STATS + 1) * sizeof(Widget));
-  stat_factor_label = fc_malloc((NUM_STATS + 1) * sizeof(Widget));
+  stat_surplus_label = fc_malloc((O_LAST + 1) * sizeof(Widget));
+  stat_factor_label = fc_malloc((O_LAST + 1) * sizeof(Widget));
 
   cma_dialog = 
     I_T(XtCreatePopupShell("cmapopup",
@@ -172,22 +173,22 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
 
   /* Create labels in the minimal surplus column. */
   prev = surplus_header;
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     I_L(stat_surplus_label[i] = 
         XtVaCreateManagedWidget("cmastatlabel",
                                 labelWidgetClass, form,
                                 XtNfromHoriz, preset_viewport,
                                 XtNfromVert, prev,
                                 XtNvertDistance, 1,
-                                XtNlabel, (i == NUM_STATS) ? 
-                                  _("Celebrate") : cm_get_stat_name(i),
+                                XtNlabel, (i == O_LAST) ? 
+                                  _("Celebrate") : get_output_name(i),
                                 NULL));
     prev = stat_surplus_label[i];
   }
 
   /* Create scrollbars in the minimal surplus column. */
   prev = surplus_header;
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     surplus_slider[i] = 
       XtVaCreateManagedWidget("cmapresetscroll",
                                   scrollbarWidgetClass, form,
@@ -195,13 +196,13 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
                                   XtNfromVert, prev,
                                   NULL);
     prev = stat_surplus_label[i];
-  }
+  } output_type_iterate_end;
 
   celebrate_toggle = 
     I_L(XtVaCreateManagedWidget("cmapresettoggle",
                                 toggleWidgetClass,form,
                                 XtNfromHoriz, stat_surplus_label[0],
-                                XtNfromVert, stat_surplus_label[NUM_STATS - 1],
+                                XtNfromVert, stat_surplus_label[O_LAST - 1],
                                 NULL));
 
   /* Create header label in the factor column. */
@@ -214,22 +215,22 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
 
   /* Create stat labels in the factor column. */
   prev = factor_header;
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     I_L(stat_factor_label[i] =
         XtVaCreateManagedWidget("cmastatlabel",
                                 labelWidgetClass, form,
                                 XtNfromHoriz, surplus_slider[0],
                                 XtNfromVert, prev,
                                 XtNvertDistance, 1,
-                                XtNlabel, (i == NUM_STATS) ?
-                                  _("Celebrate") : cm_get_stat_name(i),
+                                XtNlabel, (i == O_LAST) ?
+                                  _("Celebrate") : get_output_name(i),
                                 NULL));
     prev = stat_factor_label[i];
   }
 
   /* Create scrollbars in the factor column. */
   prev = factor_header;
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     factor_slider[i] =
       XtVaCreateManagedWidget("cmapresetscroll",
                                   scrollbarWidgetClass, form,
@@ -306,14 +307,14 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
   XtAddCallback(close_button, XtNcallback,
                 close_callback, NULL);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     XtAddCallback(surplus_slider[i], XtNscrollProc,
                   sliders_scroll_callback, NULL);
     XtAddCallback(surplus_slider[i], XtNjumpProc,
                   sliders_jump_callback, NULL);
-  }
+  } output_type_iterate_end;
 
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     XtAddCallback(factor_slider[i], XtNscrollProc,
                   sliders_scroll_callback, NULL);
     XtAddCallback(factor_slider[i], XtNjumpProc,
@@ -323,23 +324,23 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
   /* Update dialog with CMA parameters from city.  */
   cmafec_get_fe_parameter(current_city, &parameter);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     minimal_surplus[i] = parameter.minimal_surplus[i];
-  }
+  } output_type_iterate_end;
 
   XtVaSetValues(celebrate_toggle, 
          XtNlabel, parameter.require_happy ? _("Yes") : _("No"),
          XtNstate, parameter.require_happy, NULL);
 
   if (parameter.happy_factor > 0) {
-    factors[NUM_STATS] = parameter.happy_factor;
+    factors[O_LAST] = parameter.happy_factor;
   } else {
-    factors[NUM_STATS] = 1;
+    factors[O_LAST] = 1;
   }
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     factors[i] = parameter.factor[i];
-  }
+  } output_type_iterate_end;
 
   set_slider_values();
   update_cma_preset_list();
@@ -350,24 +351,27 @@ void show_cma_dialog(struct city *pcity, Widget citydlg)
   XtVaSetValues(result_label, XtNwidth, 360, NULL);
   XtVaSetValues(result_label, XtNheight, 110, NULL);
 
-  for(i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     XtVaSetValues(stat_surplus_label[i], XtNwidth, 90, NULL);
     XtVaSetValues(stat_factor_label[i], XtNwidth, 90, NULL);
-  }
-  XtVaSetValues(stat_factor_label[i], XtNwidth, 90, NULL);
+  } output_type_iterate_end;
+  /* FIXME! Now we assume that output_type_iterate ends with O_LAST. */
+  XtVaSetValues(stat_factor_label[O_LAST], XtNwidth, 90, NULL);
 
   XtRealizeWidget(cma_dialog);
 
   update_stat_labels(True);
-  cm_copy_result_from_city(pcity, &result);
+  cm_result_from_main_map(result, pcity);
   xaw_set_label(result_label, 
-       (char *) cmafec_get_result_descr(current_city, &result, &parameter));
+       (char *) cmafec_get_result_descr(current_city, result, &parameter));
 
   XSetWMProtocols(display, XtWindow(cma_dialog),
                   &wm_delete_window, 1);
   XtVaSetValues(preset_viewport, XtNforceBars, True, NULL);
   xaw_set_relative_position(citydlg, cma_dialog, 5, 5);
   XtPopup(cma_dialog, XtGrabNone);
+
+  cm_result_destroy(result);
 }
 
 /**************************************************************************
@@ -382,8 +386,8 @@ static void update_cma_preset_list(void)
   if (cmafec_preset_num()) {
     /* Add all CMA presets to the list. */
     for (i = 0; i < cmafec_preset_num(); i++) {
-      my_snprintf(preset_text[i], sizeof(preset_text[i]),
-                  cmafec_preset_get_descr(i));
+      fc_strlcpy(preset_text[i], cmafec_preset_get_descr(i),
+		sizeof(preset_text[i]));
       preset_lines[i] = preset_text[i];
     }
     XawListChange(preset_list, preset_lines, cmafec_preset_num(), 0, True);
@@ -447,7 +451,7 @@ static void remove_preset(Widget w, XtPointer list,
   char buf[256]; 
 
   if (ret->list_index != XAW_LIST_NONE && cmafec_preset_num()) {
-    my_snprintf(buf, sizeof(buf), 
+    fc_snprintf(buf, sizeof(buf), 
                 _("Do you really want to remove %s?"),
                 cmafec_preset_get_descr(ret->list_index));
 
@@ -465,7 +469,7 @@ static void remove_preset(Widget w, XtPointer list,
 static void remove_preset_yes(Widget w, XtPointer row,
                               XtPointer call_data)
 {
-  cmafec_preset_remove((int)row);
+  cmafec_preset_remove(XTPOINTER_TO_INT(row));
   update_cma_preset_list();
   destroy_message_dialog(w);
 }
@@ -488,9 +492,12 @@ static void select_preset(Widget w, XtPointer list,
   XawListReturnStruct *ret;
   const struct cm_parameter *param;
   struct cm_parameter parameter;
-  struct cm_result result;
-  int i;
+  struct cm_result *result;
 
+  /* check gloabl variable current_city */
+  fc_assert_ret(current_city != NULL);
+
+  result = cm_result_new(current_city);
   ret = XawListShowCurrent(list);
 
   if (ret->list_index != XAW_LIST_NONE && cmafec_preset_num()) {
@@ -503,26 +510,27 @@ static void select_preset(Widget w, XtPointer list,
     }
 
     cmafec_get_fe_parameter(current_city, &parameter);
-    cm_copy_result_from_city(current_city, &result);
+    cm_result_from_main_map(result, current_city);
     xaw_set_label(result_label,
-        (char *) cmafec_get_result_descr(current_city, &result, &parameter));
+        (char *) cmafec_get_result_descr(current_city, result, &parameter));
 
-    for (i = 0; i < NUM_STATS; i++) {
+    output_type_iterate(i) {
       minimal_surplus[i] = param->minimal_surplus[i];
-    }
+    } output_type_iterate_end;
 
     XtVaSetValues(celebrate_toggle, XtNlabel,
             param->require_happy ? _("Yes") : _("No"),
             XtNstate, parameter.require_happy, NULL);
 
-    factors[NUM_STATS] = param->happy_factor;
+    factors[O_LAST] = param->happy_factor;
 
-    for (i = 0; i < NUM_STATS; i++) {
+    output_type_iterate(i) {
       factors[i] = param->factor[i];
-    }
+    } output_type_iterate_end;
   }
 
-  update_stat_labels(result.found_a_valid);
+  update_stat_labels(result->found_a_valid);
+  cm_result_destroy(result);
 }
 
 /****************************************************************
@@ -541,12 +549,12 @@ static void set_slider_values(void)
 {
   int i;
   
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     XawScrollbarSetThumb(surplus_slider[i],
                      (((20 + minimal_surplus[i])))*1/41.0f, 1/41.0f);
-  }
+  } output_type_iterate_end;
 
-  for (i = 0; i < (NUM_STATS + 1); i++) { 
+  for (i = 0; i < (O_LAST + 1); i++) { 
     XawScrollbarSetThumb(factor_slider[i],
                      (((factors[i])))*1/25.0f, 1/25.0f);
   }
@@ -560,10 +568,15 @@ static void sliders_scroll_callback(Widget w, XtPointer client_data,
 {
   int pos = XTPOINTER_TO_INT(position_val);
   struct cm_parameter parameter;
-  struct cm_result result;
+  struct cm_result *result;
   int i, preset_index;
 
-  for (i = 0; i < NUM_STATS; i++) {
+  /* check gloabl variable current_city */
+  fc_assert_ret(current_city != NULL);
+
+  result = cm_result_new(current_city);
+
+  output_type_iterate(i) {
     if (w == surplus_slider[i]) {
       if (pos > 0 ) {
         minimal_surplus[i]++;
@@ -577,9 +590,9 @@ static void sliders_scroll_callback(Widget w, XtPointer client_data,
         }
       }
     }
-  }
+  } output_type_iterate_end;
 
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     if (w == factor_slider[i]) {
       if (pos > 0 ) {
         factors[i]++;
@@ -597,13 +610,13 @@ static void sliders_scroll_callback(Widget w, XtPointer client_data,
 
   cmafec_get_fe_parameter(current_city, &parameter);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     parameter.minimal_surplus[i] = minimal_surplus[i];
     parameter.factor[i] = factors[i];
-  }
+  } output_type_iterate_end;
 
   XtVaGetValues(celebrate_toggle, XtNstate, &parameter.require_happy, NULL);
-  parameter.happy_factor = factors[NUM_STATS];
+  parameter.happy_factor = factors[O_LAST];
 
   cmafec_set_fe_parameter(current_city, &parameter);
 
@@ -613,11 +626,11 @@ static void sliders_scroll_callback(Widget w, XtPointer client_data,
   }
 
   cmafec_get_fe_parameter(current_city, &parameter);
-  cm_copy_result_from_city(current_city, &result);
+  cm_result_from_main_map(result, current_city);
   xaw_set_label(result_label,
-        (char *) cmafec_get_result_descr(current_city, &result, &parameter));
+        (char *) cmafec_get_result_descr(current_city, result, &parameter));
 
-  update_stat_labels(result.found_a_valid);
+  update_stat_labels(result->found_a_valid);
   update_cma_preset_list();
 
   /* highlight preset if parameter matches */
@@ -625,6 +638,8 @@ static void sliders_scroll_callback(Widget w, XtPointer client_data,
   if (preset_index != -1) {
     XawListHighlight(preset_list, preset_index);
   }
+
+  cm_result_destroy(result);
 }
 
 /**************************************************************************
@@ -635,18 +650,22 @@ void sliders_jump_callback(Widget w, XtPointer client_data,
 {
   float percent=*(float*)percent_ptr;
   struct cm_parameter parameter;
-  struct cm_result result;
+  struct cm_result *result;
   int i, preset_index;
 
+  /* check gloabl variable current_city */
+  fc_assert_ret(current_city != NULL);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  result = cm_result_new(current_city);
+
+  output_type_iterate(i) {
     if (w == surplus_slider[i]) {
       /* convert from percent to [-20..20] */
       minimal_surplus[i] = (int)(percent * 41) - 20;
     }
-  }
+  } output_type_iterate_end;
 
-  for (i = 0; i < (NUM_STATS + 1); i++) {
+  for (i = 0; i < (O_LAST + 1); i++) {
     if (w == factor_slider[i]) {
       /* convert from percent to [1..25] */
       factors[i] = (int)(percent * 25) + 1;  
@@ -655,13 +674,13 @@ void sliders_jump_callback(Widget w, XtPointer client_data,
 
   cmafec_get_fe_parameter(current_city, &parameter);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     parameter.minimal_surplus[i] = minimal_surplus[i];
     parameter.factor[i] = factors[i];
-  }
+  } output_type_iterate_end;
 
   XtVaGetValues(celebrate_toggle, XtNstate, &parameter.require_happy, NULL);
-  parameter.happy_factor = factors[NUM_STATS];
+  parameter.happy_factor = factors[O_LAST];
 
   cmafec_set_fe_parameter(current_city, &parameter);
 
@@ -671,18 +690,20 @@ void sliders_jump_callback(Widget w, XtPointer client_data,
   }
 
   cmafec_get_fe_parameter(current_city, &parameter);
-  cm_copy_result_from_city(current_city, &result);
+  cm_result_from_main_map(result, current_city);
   xaw_set_label(result_label,
-        (char *) cmafec_get_result_descr(current_city, &result, &parameter));
+        (char *) cmafec_get_result_descr(current_city, result, &parameter));
 
   update_cma_preset_list();
-  update_stat_labels(result.found_a_valid);
+  update_stat_labels(result->found_a_valid);
 
   /* highlight preset if parameter matches */
   preset_index = cmafec_preset_get_index_of_parameter(&parameter);
   if (preset_index != -1) {
     XawListHighlight(preset_list, preset_index);
   }
+
+  cm_result_destroy(result);
 }
 
 /**************************************************************************
@@ -691,25 +712,24 @@ void sliders_jump_callback(Widget w, XtPointer client_data,
 static void update_stat_labels(bool is_valid)
 {
   char buf[256]; 
-  int i;
 
-  for (i = 0; i < NUM_STATS; i++) {
-    my_snprintf(buf, sizeof(buf), "%-9s%3d",
-                cm_get_stat_name(i),
+  output_type_iterate(i) {
+    fc_snprintf(buf, sizeof(buf), "%-9s%3d",
+                get_output_name(i),
                 minimal_surplus[i]);
     xaw_set_label(stat_surplus_label[i], buf);
-  }
+  } output_type_iterate_end;
 
-  for (i = 0; i < NUM_STATS; i++) { 
-    my_snprintf(buf, sizeof(buf), "%-9s%3d",
-                cm_get_stat_name(i),
+  output_type_iterate(i) {
+    fc_snprintf(buf, sizeof(buf), "%-9s%3d",
+                get_output_name(i),
                 factors[i]);
     xaw_set_label(stat_factor_label[i], buf);
-  }
-  my_snprintf(buf, sizeof(buf), "%-9s%3d",
+  } output_type_iterate_end;
+  fc_snprintf(buf, sizeof(buf), "%-9s%3d",
               "Celebrate",
-              factors[NUM_STATS]);
-  xaw_set_label(stat_factor_label[NUM_STATS], buf);
+              factors[O_LAST]);
+  xaw_set_label(stat_factor_label[O_LAST], buf);
 
   XtSetSensitive(release_button, 
        (cma_is_city_under_agent(current_city, NULL)
@@ -736,21 +756,18 @@ static void new_preset_callback(Widget w, XtPointer save_preset,
 {
   Boolean celebrate_setting;
   struct cm_parameter parameter;
-  int i;
   
   if (save_preset) {
     /* The user entered a preset name and clicked OK  */
     XtVaGetValues(celebrate_toggle, XtNstate, &celebrate_setting, NULL);
     cmafec_get_fe_parameter(current_city, &parameter);
 
-    for (i = 0; i < NUM_STATS; i++) {
+    output_type_iterate(i) {
       parameter.minimal_surplus[i] = minimal_surplus[i];
-    }
-    for (i = 0; i < NUM_STATS; i++) {
       parameter.factor[i] = factors[i];
-    }
+    } output_type_iterate_end;
 
-    parameter.happy_factor = factors[NUM_STATS];
+    parameter.happy_factor = factors[O_LAST];
     parameter.require_happy = celebrate_setting;
 
     cmafec_preset_add(input_dialog_get_input(w), &parameter);
@@ -767,8 +784,10 @@ void celebrate_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   Boolean celebrate;
   struct cm_parameter parameter;
-  struct cm_result result;
-  int i;
+  struct cm_result *result;
+
+  /* check gloabl variable current_city */
+  fc_assert_ret(current_city != NULL);
 
   /* Change label on celebrate toggle. */
   XtVaGetValues(w, XtNstate, &celebrate, NULL);
@@ -776,13 +795,13 @@ void celebrate_callback(Widget w, XtPointer client_data, XtPointer call_data)
 
   /* Update parameters with new celebrate setting. */
   cmafec_get_fe_parameter(current_city, &parameter);
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     parameter.minimal_surplus[i] = minimal_surplus[i];
     parameter.factor[i] = factors[i];
-  }
+  } output_type_iterate_end;
 
   XtVaGetValues(celebrate_toggle, XtNstate, &parameter.require_happy, NULL);
-  parameter.happy_factor = factors[NUM_STATS];
+  parameter.happy_factor = factors[O_LAST];
 
   cmafec_set_fe_parameter(current_city, &parameter);
 
@@ -791,13 +810,16 @@ void celebrate_callback(Widget w, XtPointer client_data, XtPointer call_data)
     cma_put_city_under_agent(current_city, &parameter);
   }
 
+  result = cm_result_new(current_city);
   cmafec_get_fe_parameter(current_city, &parameter);
-  cm_copy_result_from_city(current_city, &result);
+  cm_result_from_main_map(result, current_city);
   xaw_set_label(result_label,
-        (char *) cmafec_get_result_descr(current_city, &result, &parameter));
+        (char *) cmafec_get_result_descr(current_city, result, &parameter));
 
-  update_stat_labels(result.found_a_valid);
+  update_stat_labels(result->found_a_valid);
   update_cma_preset_list();
+
+  cm_result_destroy(result);
 }
 
 /**************************************************************************
@@ -805,8 +827,8 @@ void celebrate_callback(Widget w, XtPointer client_data, XtPointer call_data)
 **************************************************************************/
 void popdown_cma_dialog(void)
 {
-  city_report_dialog_update_city(current_city);
   if (cma_dialog) {
+    city_report_dialog_update_city(current_city);
     XtDestroyWidget(cma_dialog);
     cma_dialog = 0;
   }
@@ -827,12 +849,17 @@ static void close_callback(Widget w, XtPointer client_data,
 static void change_callback(Widget w, XtPointer client_data,
                             XtPointer call_data)
 {
-  struct cm_result result;
+  struct cm_result *result;
   struct cm_parameter param;
 
-  cmafec_get_fe_parameter(current_city, &param);
-  cm_query_result(current_city, &param, &result);
-  cma_apply_result(current_city, &result);
-  refresh_city_dialog(current_city);
-}
+  /* check gloabl variable current_city */
+  fc_assert_ret(current_city != NULL);
 
+  result = cm_result_new(current_city);
+  cmafec_get_fe_parameter(current_city, &param);
+  cm_query_result(current_city, &param, result);
+  cma_apply_result(current_city, result);
+  refresh_city_dialog(current_city);
+
+  cm_result_destroy(result);
+}
