@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -33,12 +32,15 @@
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Viewport.h>	/* for racesdlg */
 
+/* utility */
+#include "bitvector.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "rand.h"
 #include "support.h"
 
+/* common */
 #include "game.h"
 #include "government.h"
 #include "map.h"
@@ -46,11 +48,11 @@
 #include "player.h"
 #include "unitlist.h"
 
+/* client */
 #include "chatline.h"
-#include "cityrep.h"	/* for popdown_city_report_dialog */
-#include "civclient.h"
+#include "cityrep.h"    /* city_report_dialog_popdown() */
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "control.h" /* request_xxx and set_unit_focus */
 #include "graphics.h"
 #include "gui_main.h"
@@ -58,7 +60,7 @@
 #include "mapctrl.h"
 #include "mapctrl_common.h"
 #include "mapview.h"
-#include "messagewin.h"	/* for popdown_meswin_dialog */
+#include "messagewin.h" /* meswin_dialog_popdown() */
 #include "options.h"
 #include "packhand.h"
 #include "plrdlg.h"	/* for popdown_players_dialog */
@@ -174,7 +176,7 @@ static void select_random_race(void)
   /* try to find a free nation */
   /* FIXME: this code should be done another way. -ev */
   while (1) {
-    unsigned int race_toggle_index = myrand(nation_count());
+    unsigned int race_toggle_index = fc_rand(nation_count());
 
     if (!is_nation_playable(nation_by_number(race_toggle_index))
 	|| !nation_by_number(race_toggle_index)->is_available
@@ -332,7 +334,8 @@ static void notify_no_goto_command_callback(Widget w, XtPointer client_data,
   location.
 **************************************************************************/
 void popup_notify_goto_dialog(const char *headline, const char *lines,
-			      struct tile *ptile)
+                              const struct text_tag_list *tags,
+                              struct tile *ptile)
 {
   Widget notify_dialog_shell, notify_form, notify_command, notify_goto_command;
   Widget notify_headline, notify_label;
@@ -389,7 +392,16 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
   /*  XtSetSensitive(toplevel, FALSE); */
 }
 
+/**************************************************************************
+  Popup a dialog to display connection message from server.
+**************************************************************************/
+void popup_connect_msg(const char *headline, const char *message)
+{
+  /* FIXME: Needs proper implementation.
+   *        Now just puts to chat window so message is not completely lost. */
 
+  output_window_append(ftc_client, message);
+}
 
 /****************************************************************
 ...
@@ -397,7 +409,7 @@ void popup_notify_goto_dialog(const char *headline, const char *lines,
 static void caravan_establish_trade_callback(Widget w, XtPointer client_data,
 					     XtPointer call_data)
 {
-  dsend_packet_unit_establish_trade(&aconnection, caravan_unit_id);
+  dsend_packet_unit_establish_trade(&client.conn, caravan_unit_id);
   destroy_message_dialog(w);
   caravan_dialog = 0;
   process_caravan_arrival(NULL);
@@ -411,7 +423,7 @@ static void caravan_help_build_wonder_callback(Widget w,
 					       XtPointer client_data,
 					       XtPointer call_data)
 {
-  dsend_packet_unit_help_build_wonder(&aconnection, caravan_unit_id);
+  dsend_packet_unit_help_build_wonder(&client.conn, caravan_unit_id);
 
   destroy_message_dialog(w);
   caravan_dialog = 0;
@@ -439,9 +451,10 @@ void popup_caravan_dialog(struct unit *punit,
 {
   char buf[128];
   
-  my_snprintf(buf, sizeof(buf),
-	      _("Your caravan from %s reaches the city of %s.\nWhat now?"),
-	      city_name(phomecity), city_name(pdestcity));
+  fc_snprintf(buf, sizeof(buf),
+              _("Your %s from %s reaches the city of %s.\nWhat now?"),
+              unit_name_translation(punit),
+              city_name(phomecity), city_name(pdestcity));
   
   caravan_city_id=pdestcity->id; /* callbacks need these */
   caravan_unit_id=punit->id;
@@ -529,10 +542,18 @@ static void pillage_callback(Widget w, XtPointer client_data,
   }
 
   if (client_data) {
-    struct unit *punit = game_find_unit_by_number (unit_to_use_to_pillage);
+    struct unit *punit = game_unit_by_number(unit_to_use_to_pillage);
     if (punit) {
+      Base_type_id pillage_base = -1;
+      int what = XTPOINTER_TO_INT(client_data);
+
+      if (what > S_LAST) {
+        pillage_base = what - S_LAST - 1;
+        what = S_LAST;
+      }
+
       request_new_unit_activity_targeted(punit, ACTIVITY_PILLAGE,
-					 XTPOINTER_TO_INT(client_data));
+					 what, pillage_base);
     }
   }
 
@@ -544,10 +565,11 @@ static void pillage_callback(Widget w, XtPointer client_data,
 ...
 *****************************************************************/
 void popup_pillage_dialog(struct unit *punit,
-			  bv_special may_pillage)
+			  bv_special may_pillage,
+                          bv_bases bases)
 {
   Widget shell, form, dlabel, button, prev;
-  enum tile_special_type what, prereq;
+  int what;
 
   if (is_showing_pillage_dialog) {
     return;
@@ -563,25 +585,35 @@ void popup_pillage_dialog(struct unit *punit,
   dlabel = I_L(XtVaCreateManagedWidget("dlabel", labelWidgetClass, form, NULL));
 
   prev = dlabel;
-  while ((what = get_preferred_pillage(may_pillage)) != S_LAST) {
+  while ((what = get_preferred_pillage(may_pillage, bases)) != S_LAST) {
     bv_special what_bv;
+    bv_bases what_base;
 
     BV_CLR_ALL(what_bv);
-    BV_SET(what_bv, what);
+    BV_CLR_ALL(what_base);
+
+    if (what > S_LAST) {
+      BV_SET(what_base, what - S_LAST - 1);
+    } else {
+      BV_SET(what_bv, what);
+    }
+
     button =
       XtVaCreateManagedWidget ("button", commandWidgetClass, form,
-			       XtNfromVert, prev,
-			       XtNlabel,
-			       (XtArgVal)(get_infrastructure_text(what_bv)),
-			       NULL);
+                               XtNfromVert, prev,
+                               XtNlabel,
+                               (XtArgVal)(get_infrastructure_text(what_bv,
+                                                                  what_base)),
+                               NULL);
     XtAddCallback(button, XtNcallback, pillage_callback,
-		  INT_TO_XTPOINTER(what));
-    prev = button;
-    clear_special(&may_pillage, what);
-    prereq = get_infrastructure_prereq(what);
-    if (prereq != S_LAST) {
-      clear_special(&may_pillage, prereq);
+                  INT_TO_XTPOINTER(what));
+
+    if (what > S_LAST) {
+      BV_CLR(bases, what - S_LAST - 1);
+    } else {
+      clear_special(&may_pillage, what);
     }
+    prev = button;
   }
   button =
     I_L(XtVaCreateManagedWidget("closebutton", commandWidgetClass, form,
@@ -627,7 +659,7 @@ Widget popup_message_dialog(Widget parent, const char *dialogname,
   while((fcb=((void(*)(Widget, XtPointer, XtPointer))(va_arg(args, void *))))) {
     client_data=va_arg(args, XtPointer);
     fixed_width=va_arg(args, int);
-    my_snprintf(button_name, sizeof(button_name), "button%d", i++);
+    fc_snprintf(button_name, sizeof(button_name), "button%d", i++);
     
     button=XtVaCreateManagedWidget(button_name, commandWidgetClass, 
 				   dform, NULL);
@@ -670,7 +702,7 @@ static int number_of_columns(int n)
   double sqrt(); double ceil();
   return ceil(sqrt((double)n/5.0));
 #else
-  assert(MAX_SELECT_UNITS == 100);
+  fc_assert(MAX_SELECT_UNITS == 100);
   if(n<=5) return 1;
   else if(n<=20) return 2;
   else if(n<=45) return 3;
@@ -734,9 +766,9 @@ void popup_unit_select_dialog(struct tile *ptile)
 
     unit_select_ids[i]=punit->id;
 
-    pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
+    pcity = player_city_by_number(client_player(), punit->homecity);
     
-    my_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s", 
+    fc_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s", 
 	    utype_name_translation(punittemp), 
 	    pcity ? city_name(pcity) : "",
 	    unit_activity_text(punit));
@@ -812,8 +844,8 @@ void unit_select_all_callback(Widget w, XtPointer client_data,
   XtDestroyWidget(unit_select_dialog_shell);
   
   for(i=0; i<unit_select_no; i++) {
-    struct unit *punit = player_find_unit_by_id(game.player_ptr,
-						unit_select_ids[i]);
+    struct unit *punit = player_unit_by_number(client_player(),
+                                               unit_select_ids[i]);
     if(punit) {
       set_unit_focus(punit);
     }
@@ -834,8 +866,8 @@ void unit_select_callback(Widget w, XtPointer client_data,
   for(i=0; i<unit_select_no; i++) {
 
     if(unit_select_commands[i]==w) {
-      struct unit *punit = player_find_unit_by_id(game.player_ptr,
-						  unit_select_ids[i]);
+      struct unit *punit = player_unit_by_number(client_player(),
+                                                 unit_select_ids[i]);
       if(punit) {
 	set_unit_focus(punit);
       }
@@ -902,20 +934,19 @@ void create_races_dialog(struct player *pplayer)
     }
   } nations_iterate_end;
   maxracelen = MIN(maxracelen, MAX_LEN_NAME-1);
-  my_snprintf(maxracename, sizeof(maxracename), "%*s", maxracelen+2, "W");
+  fc_snprintf(maxracename, sizeof(maxracename), "%*s", maxracelen+2, "W");
 
-  races_dialog_shell =
-    I_T(XtCreatePopupShell("racespopup", 
-			   transientShellWidgetClass,
-			   toplevel, NULL, 0));
+  races_dialog_shell = I_T(XtCreatePopupShell("racespopup", 
+					  transientShellWidgetClass,
+					  toplevel, NULL, 0));
 
   races_form = XtVaCreateManagedWidget("racesform", 
 				       formWidgetClass, 
 				       races_dialog_shell, NULL);   
 
-  races_label =
-    I_LW(XtVaCreateManagedWidget("raceslabel", labelWidgetClass, 
-				 races_form, NULL));  
+  races_label = I_L(XtVaCreateManagedWidget("raceslabel", 
+				       labelWidgetClass, 
+				       races_form, NULL));  
 
   races_toggles_viewport =
     XtVaCreateManagedWidget("racestogglesviewport",
@@ -930,13 +961,9 @@ void create_races_dialog(struct player *pplayer)
 			    races_toggles_viewport,
 			    NULL);
 
-  if (races_toggles) {
-    free(races_toggles);
-  }
+  free(races_toggles);
   races_toggles = fc_calloc(nation_count(), sizeof(Widget));
-  if (races_toggles_to_nations) {
-    free(races_toggles_to_nations);
-  }
+  free(races_toggles_to_nations);
   races_toggles_to_nations = fc_calloc(nation_count(),
 				       sizeof(struct nation_type *));
 
@@ -950,7 +977,7 @@ void create_races_dialog(struct player *pplayer)
 
     if (j == 0) {
       index = i * per_row;
-      my_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
+      fc_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
       if (i == 0) {
 	races_toggles[index] =
 	  XtVaCreateManagedWidget(namebuf,
@@ -972,7 +999,7 @@ void create_races_dialog(struct player *pplayer)
       }
     } else {
       index = i * per_row + j;
-      my_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
+      fc_snprintf(namebuf, sizeof(namebuf), "racestoggle%d", index);
       if (i == 0) {
 	races_toggles[index] =
 	  XtVaCreateManagedWidget(namebuf,
@@ -1015,6 +1042,7 @@ void create_races_dialog(struct player *pplayer)
 			    formWidgetClass,
 			    races_form,
 			    XtNfromVert, races_toggles_viewport,
+/*			    XtNfromHoriz, races_toggles_viewport,*/
 			    NULL);
 
   XtVaGetValues(races_leader_form, XtNdefaultDistance, &space, NULL);
@@ -1032,40 +1060,40 @@ void create_races_dialog(struct player *pplayer)
   races_leader_pick_popupmenu = 0;
 
   races_leader_pick_menubutton =
-    I_LW(XtVaCreateManagedWidget("racesleaderpickmenubutton",
-				 menuButtonWidgetClass,
-				 races_leader_form,
-/*				 XtNfromVert, races_leader,*/
-				 XtNfromHoriz, races_leader,
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racesleaderpickmenubutton",
+				menuButtonWidgetClass,
+				races_leader_form,
+/*				XtNfromVert, races_leader,*/
+				XtNfromHoriz, races_leader,
+				NULL));
 
-  races_sex_label =
-    I_LW(XtVaCreateManagedWidget("racessexlabel", labelWidgetClass,
-				 races_form,
-				 XtNfromVert, races_leader_form,
-				 NULL));
+  races_sex_label = I_L(XtVaCreateManagedWidget("racessexlabel",
+				            labelWidgetClass,
+				            races_form,
+					    XtNfromVert, races_leader_form,
+					    NULL));
 
-  races_sex_form =
-    XtVaCreateManagedWidget("racessexform", formWidgetClass, races_form,
-			    XtNfromVert, races_leader_form,
-			    XtNfromHoriz, races_sex_label,
-			    NULL);
+  races_sex_form = XtVaCreateManagedWidget("racessexform",
+					   formWidgetClass,
+					   races_form,
+					   XtNfromVert, races_sex_label,
+					   NULL);
 
   races_sex_toggles[0] =
-    I_LW(XtVaCreateManagedWidget("racessextoggle0",
-				 toggleWidgetClass,
-				 races_sex_form,
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racessextoggle0",
+				toggleWidgetClass,
+				races_sex_form,
+				NULL));
 
   races_sex_toggles[1] =
-    I_LW(XtVaCreateManagedWidget("racessextoggle1",
-				 toggleWidgetClass,
-				 races_sex_form,
-				 XtNfromHoriz,
-				 (XtArgVal)races_sex_toggles[0],
-				 XtNradioGroup,
-				 races_sex_toggles[0],
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racessextoggle1",
+				toggleWidgetClass,
+				races_sex_form,
+				XtNfromHoriz,
+				(XtArgVal)races_sex_toggles[0],
+				XtNradioGroup,
+				races_sex_toggles[0],
+				NULL));
 
   /* find out styles that can be used at the game beginning */
   /* Limit of 64 city_styles should be deleted. -ev */
@@ -1078,11 +1106,12 @@ void create_races_dialog(struct player *pplayer)
   }
 
   races_style_label =
-    I_LW(XtVaCreateManagedWidget("racesstylelabel", 
-				 labelWidgetClass, 
-				 races_form,
-				 XtNfromVert, races_sex_form,
-				 NULL));  
+    I_L(XtVaCreateManagedWidget("racesstylelabel", 
+				labelWidgetClass, 
+				races_form,
+				XtNfromVert, races_sex_form,
+/*				XtNfromHoriz, races_toggles_viewport,*/
+				NULL));  
 
   races_style_form =
     XtVaCreateManagedWidget("racesstyleform", 
@@ -1097,7 +1126,7 @@ void create_races_dialog(struct player *pplayer)
 
   for( i = 0; i < ((b_s_num-1)/per_row)+1; i++) {
     index = i * per_row;
-    my_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
+    fc_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
     if( i == 0 ) {
       races_style_toggles[index] =
 	XtVaCreateManagedWidget(namebuf,
@@ -1121,7 +1150,7 @@ void create_races_dialog(struct player *pplayer)
     for( j = 1; j < per_row; j++) {
       index = i * per_row + j;
       if( index >= b_s_num ) break;
-      my_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
+      fc_snprintf(namebuf, sizeof(namebuf), "racesstyle%d", index);
       if( i == 0 ) {
 	races_style_toggles[index] =
 	  XtVaCreateManagedWidget(namebuf,
@@ -1157,24 +1186,24 @@ void create_races_dialog(struct player *pplayer)
 					      NULL);
 
   races_ok_command =
-    I_LW(XtVaCreateManagedWidget("racesokcommand",
-				 commandWidgetClass,
-				 races_action_form,
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racesokcommand",
+				commandWidgetClass,
+				races_action_form,
+				NULL));
 
   races_random_command =
-    I_LW(XtVaCreateManagedWidget("racesdisconnectcommand",
-				 commandWidgetClass,
-				 races_action_form,
-				 XtNfromHoriz, races_ok_command,
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racesdisconnectcommand",
+				commandWidgetClass,
+				races_action_form,
+				XtNfromHoriz, races_ok_command,
+				NULL));
 
   races_quit_command =
-    I_LW(XtVaCreateManagedWidget("racesquitcommand",
-				 commandWidgetClass,
-				 races_action_form,
-				 XtNfromHoriz, races_random_command,
-				 NULL));
+    I_L(XtVaCreateManagedWidget("racesquitcommand",
+				commandWidgetClass,
+				races_action_form,
+				XtNfromHoriz, races_random_command,
+				NULL));
 
   XtAddCallback(races_random_command, XtNcallback,
 		races_random_command_callback, NULL);
@@ -1278,10 +1307,8 @@ void races_toggles_set_sensitive(void)
         nation_index(races_toggles_to_nations[races_buttons_get_current()]);
     }
 
-    freelog(LOG_DEBUG, "  [%d]: %d = %s",
-	    selected_nation,
-	    nation_number(nation),
-	    nation_rule_name(nation));
+    log_debug("  [%d]: %d = %s", selected_nation,
+              nation_number(nation), nation_rule_name(nation));
 
     if (this_index == selected_nation) {
       XawToggleUnsetCurrent(races_toggles[0]);
@@ -1305,8 +1332,6 @@ void races_toggles_callback(Widget w, XtPointer client_data,
   int index = XTPOINTER_TO_INT(client_data);
   struct nation_type *race = races_toggles_to_nations[index];
   int j;
-  int leader_count;
-  struct nation_leader *leaders = get_nation_leaders(race, &leader_count);
   Widget entry;
 
   if(races_leader_pick_popupmenu)
@@ -1320,17 +1345,20 @@ void races_toggles_callback(Widget w, XtPointer client_data,
 
   local_nation_count = nation_count();
 
-  for(j=0; j<leader_count; j++) {
+  j = 0;
+  nation_leader_list_iterate(nation_leaders(race), pleader) {
     entry =
-      XtVaCreateManagedWidget(leaders[j].name,
-			      smeBSBObjectClass,
-			      races_leader_pick_popupmenu,
-			      NULL);
+      XtVaCreateManagedWidget(nation_leader_name(pleader),
+                              smeBSBObjectClass,
+                              races_leader_pick_popupmenu,
+                              NULL);
     XtAddCallback(entry, XtNcallback, races_leader_pick_callback,
-		  INT_TO_XTPOINTER(local_nation_count * j + nation_index(race)));
-  }
+                  INT_TO_XTPOINTER(local_nation_count * j
+                                   + nation_index(race)));
+    j++;
+  } nation_leader_list_iterate_end;
 
-  races_leader_set_values(race, myrand(leader_count));
+  races_leader_set_values(race, fc_rand(j));
 
   x_simulate_button_click
   (
@@ -1355,13 +1383,14 @@ void races_leader_pick_callback(Widget w, XtPointer client_data,
 **************************************************************************/
 void races_leader_set_values(struct nation_type *race, int lead)
 {
-  int leader_count;
-  struct nation_leader *leaders = get_nation_leaders(race, &leader_count);
+  const struct nation_leader *pleader =
+      nation_leader_list_get(nation_leaders(race), lead);
 
-  XtVaSetValues(races_leader, XtNstring, leaders[lead].name, NULL);
-  XawTextSetInsertionPoint(races_leader, strlen(leaders[lead].name));
+  XtVaSetValues(races_leader, XtNstring, nation_leader_name(pleader), NULL);
+  XawTextSetInsertionPoint(races_leader,
+                           strlen(nation_leader_name(pleader)));
 
-  races_sex_buttons_set_current(!leaders[lead].is_male);
+  races_sex_buttons_set_current(!nation_leader_is_male(pleader));
 }
 
 /**************************************************************************
@@ -1459,7 +1488,7 @@ int races_indirect_compare(const void *first, const void *second)
   first_string = nation_adjective_translation(first_nation);
   second_string = nation_adjective_translation(second_nation);
 
-  return mystrcasecmp(first_string, second_string);
+  return fc_strcasecmp(first_string, second_string);
 }
 
 /**************************************************************************
@@ -1472,17 +1501,17 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
   XtPointer dp;
 
   if((selected_index=races_buttons_get_current())==-1) {
-    append_output_window(_("You must select a nation."));
+    output_window_append(ftc_client, _("You must select a nation."));
     return;
   }
 
   if((selected_sex=races_sex_buttons_get_current())==-1) {
-    append_output_window(_("You must select your sex."));
+    output_window_append(ftc_client, _("You must select your sex."));
     return;
   }
 
   if((selected_style=races_style_buttons_get_current())==-1) {
-    append_output_window(_("You must select your city style."));
+    output_window_append(ftc_client, _("You must select your city style."));
     return;
   }
 
@@ -1490,12 +1519,12 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
 
   /* perform a minimum of sanity test on the name */
   if (strlen(dp) == 0) {
-    append_output_window(_("You must type a legal name."));
+    output_window_append(ftc_client, _("You must type a legal name."));
     return;
   }
 
-  dsend_packet_nation_select_req(&aconnection,
-				 races_player->player_no,
+  dsend_packet_nation_select_req(&client.conn,
+				 player_number(races_player),
 				 nation_index(races_toggles_to_nations[selected_index]),
 				 selected_sex ? FALSE : TRUE,
 				 dp, city_style_idx[selected_style]);
@@ -1575,11 +1604,11 @@ bool popup_theme_suggestion_dialog(const char *theme_name)
 ***********************************************************************/
 void popdown_all_game_dialogs(void)
 {
-  popdown_city_report_dialog();
-  popdown_meswin_dialog();
-  popdown_science_dialog();
-  popdown_economy_report_dialog();
-  popdown_activeunits_report_dialog();
+  city_report_dialog_popdown();
+  meswin_dialog_popdown();
+  science_report_dialog_popdown();
+  economy_report_dialog_popdown();
+  units_report_dialog_popdown();
   popdown_players_dialog();
   popdown_notify_dialog();
 }
