@@ -26,15 +26,22 @@
 #include <X11/Xaw/SimpleMenu.h>
 #include <X11/Xaw/List.h>
 
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
-#include "packets.h"
-#include "player.h"
+#include "log.h"
 #include "shared.h"
 #include "support.h"
 
-#include "clinet.h"
+/* common */
+#include "government.h"
+#include "packets.h"
+#include "player.h"
+#include "research.h"
+
+/* client */
+#include "client_main.h"
+
+/* gui-xaw */
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -69,7 +76,7 @@ struct intel_dialog {
     TYPED_LIST_ITERATE(struct intel_dialog, dialoglist, pdialog)
 #define dialog_list_iterate_end  LIST_ITERATE_END
 
-static struct dialog_list dialog_list;
+static struct dialog_list *dialog_list = NULL;
 static bool dialog_list_has_been_initialised = FALSE;
 
 /******************************************************************/
@@ -96,7 +103,7 @@ void intel_diplo_close_callback(Widget w, XtPointer client_data,
 struct intel_dialog *get_intel_dialog(struct player *pplayer)
 {
   if (!dialog_list_has_been_initialised) {
-    dialog_list_init(&dialog_list);
+    dialog_list = dialog_list_new();
     dialog_list_has_been_initialised = TRUE;
   }
 
@@ -122,7 +129,7 @@ void popup_intel_dialog(struct player *pplayer)
     pdialog->pplayer = pplayer;
     pdialog->intel_dialog_shell = 0;
     pdialog->intel_diplo_dialog_shell = 0;
-    dialog_list_insert(&dialog_list, pdialog);
+    dialog_list_prepend(dialog_list, pdialog);
   }
 
   if (!(pdialog->intel_dialog_shell)) {
@@ -148,22 +155,31 @@ void popdown_intel_dialog(struct intel_dialog *pdialog)
   pdialog->intel_dialog_shell = 0;
 
   if (!(pdialog->intel_diplo_dialog_shell)) {
-    dialog_list_unlink(&dialog_list, pdialog);
+    dialog_list_remove(dialog_list, pdialog);
     free(pdialog);
   }
 }
 
+/**************************************************************************
+  Close an intelligence dialog for the given player.
+**************************************************************************/
+void close_intel_dialog(struct player *p)
+{
+  struct intel_dialog *pdialog = get_intel_dialog(p);
+  popdown_intel_dialog(pdialog);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
 void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 {
-  char buf[64];
+  char buf[4 * MAX_LEN_NAME], plr_buf[4 * MAX_LEN_NAME];
   struct city *pcity;
 
   static char *tech_list_names_ptrs[A_LAST+1];
   static char tech_list_names[A_LAST+1][200];
-  int i, j;
+  int j = 0;
 
   pdialog->intel_dialog_shell_is_raised = raise;
 
@@ -178,9 +194,9 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 						pdialog->intel_dialog_shell,
 						NULL);
 
-  my_snprintf(buf, sizeof(buf),
+  fc_snprintf(buf, sizeof(buf),
 	      _("Intelligence Information for the %s Empire"),
-	      get_nation_name(pdialog->pplayer->nation));
+	      nation_adjective_for_player(pdialog->pplayer));
 
   pdialog->intel_label = I_L(XtVaCreateManagedWidget("inteltitlelabel",
 						     labelWidgetClass,
@@ -188,26 +204,24 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 						     XtNlabel, buf,
 						     NULL));
 
-  my_snprintf(buf, sizeof(buf), _("Ruler: %s %s"),
-	      get_ruler_title(pdialog->pplayer->government,
-			      pdialog->pplayer->is_male,
-			      pdialog->pplayer->nation),
-	      pdialog->pplayer->name);
+  fc_snprintf(buf, sizeof(buf), _("Ruler: %s"),
+              ruler_title_for_player(pdialog->pplayer,
+                                     plr_buf, sizeof(plr_buf)));
   XtVaCreateManagedWidget("intelnamelabel",
 			  labelWidgetClass,
 			  pdialog->intel_form,
 			  XtNlabel, buf,
 			  NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Government: %s"),
-	      get_government_name(pdialog->pplayer->government));
+  fc_snprintf(buf, sizeof(buf), _("Government: %s"),
+	      government_name_for_player(pdialog->pplayer));
   XtVaCreateManagedWidget("intelgovlabel",
 			  labelWidgetClass,
 			  pdialog->intel_form,
 			  XtNlabel, buf,
 			  NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Gold: %d"),
+  fc_snprintf(buf, sizeof(buf), _("Gold: %d"),
 	      pdialog->pplayer->economic.gold);
   XtVaCreateManagedWidget("intelgoldlabel",
 			  labelWidgetClass,
@@ -215,7 +229,7 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 			  XtNlabel, buf,
 			  NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Tax: %d%%"),
+  fc_snprintf(buf, sizeof(buf), _("Tax: %d%%"),
 	      pdialog->pplayer->economic.tax);
   XtVaCreateManagedWidget("inteltaxlabel",
 			  labelWidgetClass,
@@ -223,7 +237,7 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 			  XtNlabel, buf,
 			  NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Science: %d%%"),
+  fc_snprintf(buf, sizeof(buf), _("Science: %d%%"),
 	      pdialog->pplayer->economic.science);
   XtVaCreateManagedWidget("intelscilabel",
 			  labelWidgetClass,
@@ -231,7 +245,7 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 			  XtNlabel, buf,
 			  NULL);
 
-  my_snprintf(buf, sizeof(buf), _("Luxury: %d%%"),
+  fc_snprintf(buf, sizeof(buf), _("Luxury: %d%%"),
 	      pdialog->pplayer->economic.luxury);
   XtVaCreateManagedWidget("intelluxlabel",
 			  labelWidgetClass,
@@ -239,18 +253,21 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 			  XtNlabel, buf,
 			  NULL);
 
-  if (pdialog->pplayer->research.researching == A_UNSET) {
-    my_snprintf(buf, sizeof(buf), _("Researching: %s(%d/%d)"),
-		advances[A_NONE].name,
-		pdialog->pplayer->research.bulbs_researched,
+  switch (player_research_get(pdialog->pplayer)->researching) {
+  case A_UNKNOWN:
+    fc_snprintf(buf, sizeof(buf), _("Researching: (Unknown)"));
+    break;
+  case A_UNSET:
+    fc_snprintf(buf, sizeof(buf), _("Researching: Unknown(%d/-)"),
+		player_research_get(pdialog->pplayer)->bulbs_researched);
+    break;
+  default:
+    fc_snprintf(buf, sizeof(buf), _("Researching: %s(%d/%d)"),
+		advance_name_researching(pdialog->pplayer),
+		player_research_get(pdialog->pplayer)->bulbs_researched,
 		total_bulbs_required(pdialog->pplayer));
-  } else {
-    my_snprintf(buf, sizeof(buf), _("Researching: %s(%d/%d)"),
-		get_tech_name(pdialog->pplayer,
-			      pdialog->pplayer->research.researching),
-		pdialog->pplayer->research.bulbs_researched,
-		total_bulbs_required(pdialog->pplayer));
-  }
+    break;
+  };
 
   XtVaCreateManagedWidget("intelreslabel",
 			  labelWidgetClass,
@@ -258,26 +275,28 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 			  XtNlabel, buf,
 			  NULL);
 
-  pcity = find_palace(pdialog->pplayer);
-  my_snprintf(buf, sizeof(buf), _("Capital: %s"),
-	      (!pcity)?_("(Unknown)"):pcity->name);
+  pcity = player_palace(pdialog->pplayer);
+  fc_snprintf(buf, sizeof(buf), _("Capital: %s"),
+	      /* TRANS: "unknown" location */
+	      (!pcity) ? _("(unknown)") : city_name(pcity));
   XtVaCreateManagedWidget("intelcapitallabel",
 			  labelWidgetClass,
 			  pdialog->intel_form,
 			  XtNlabel, buf,
 			  NULL);
 
-  for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
-    if (get_invention(pdialog->pplayer, i) == TECH_KNOWN) {
-      if(get_invention(game.player_ptr, i)==TECH_KNOWN) {
-	sz_strlcpy(tech_list_names[j], advances[i].name);
+  advance_index_iterate(A_FIRST, i) {
+    if (player_invention_state(pdialog->pplayer, i) == TECH_KNOWN) {
+      if (TECH_KNOWN == player_invention_state(client.conn.playing, i)) {
+	sz_strlcpy(tech_list_names[j], advance_name_translation(advance_by_number(i)));
       } else {
-	my_snprintf(tech_list_names[j], sizeof(tech_list_names[j]),
-		    "%s*", advances[i].name);
+	fc_snprintf(tech_list_names[j], sizeof(tech_list_names[j]),
+		    "%s*", advance_name_translation(advance_by_number(i)));
       }
       tech_list_names_ptrs[j]=tech_list_names[j];
       j++;
     }
+  } advance_index_iterate_end;
   tech_list_names_ptrs[j]=0;
 
   XtVaCreateManagedWidget("inteltechlist",
@@ -300,7 +319,7 @@ void create_intel_dialog(struct intel_dialog *pdialog, bool raise)
 		intel_close_callback, (XtPointer)pdialog);
   XtAddCallback(pdialog->intel_diplo_command, XtNcallback,
 		intel_diplo_callback,
-		INT_TO_XTPOINTER(pdialog->pplayer->player_no));
+		INT_TO_XTPOINTER(player_number(pdialog->pplayer)));
   XtRealizeWidget(pdialog->intel_dialog_shell);
 
   xaw_horiz_center(pdialog->intel_label);
@@ -335,7 +354,7 @@ void intel_close_callback(Widget w, XtPointer client_data,
 void intel_diplo_callback(Widget w, XtPointer client_data,
 			  XtPointer call_data)
 {
-  popup_intel_diplo_dialog(&game.players[XTPOINTER_TO_INT(client_data)],
+  popup_intel_diplo_dialog(player_by_number(XTPOINTER_TO_INT(client_data)),
 			   FALSE);
 }
 
@@ -364,7 +383,7 @@ void popup_intel_diplo_dialog(struct player *pplayer, bool raise)
     pdialog->pplayer = pplayer;
     pdialog->intel_dialog_shell = 0;
     pdialog->intel_diplo_dialog_shell = 0;
-    dialog_list_insert(&dialog_list, pdialog);
+    dialog_list_prepend(dialog_list, pdialog);
   }
 
   if (!(pdialog->intel_diplo_dialog_shell)) {
@@ -391,7 +410,7 @@ void popdown_intel_diplo_dialog(struct intel_dialog *pdialog)
   pdialog->intel_diplo_dialog_shell = 0;
 
   if (!(pdialog->intel_dialog_shell)) {
-    dialog_list_unlink(&dialog_list, pdialog);
+    dialog_list_remove(dialog_list, pdialog);
     free(pdialog);
   }
 }
@@ -401,7 +420,7 @@ void popdown_intel_diplo_dialog(struct intel_dialog *pdialog)
 *****************************************************************/
 void create_intel_diplo_dialog(struct intel_dialog *pdialog, bool raise)
 {
-  char buf[64];
+  char buf[4 * MAX_LEN_NAME], plr_buf[4 * MAX_LEN_NAME];
 
   pdialog->intel_diplo_dialog_shell_is_raised = raise;
 
@@ -417,9 +436,9 @@ void create_intel_diplo_dialog(struct intel_dialog *pdialog, bool raise)
 			    pdialog->intel_diplo_dialog_shell,
 			    NULL);
 
-  my_snprintf(buf, sizeof(buf),
+  fc_snprintf(buf, sizeof(buf),
 	      _("Intelligence Diplomacy Information for the %s Empire"),
-	      get_nation_name(pdialog->pplayer->nation));
+	      nation_adjective_for_player(pdialog->pplayer));
 
   pdialog->intel_diplo_label =
     I_L(XtVaCreateManagedWidget("inteldiplolabel",
@@ -427,12 +446,10 @@ void create_intel_diplo_dialog(struct intel_dialog *pdialog, bool raise)
 				pdialog->intel_diplo_form,
 				XtNlabel, buf,
 				NULL));
-   
-  my_snprintf(buf, sizeof(buf), _("Ruler: %s %s"), 
-	      get_ruler_title(pdialog->pplayer->government,
-			      pdialog->pplayer->is_male,
-			      pdialog->pplayer->nation),
-	      pdialog->pplayer->name);
+
+  fc_snprintf(buf, sizeof(buf), _("Ruler: %s"),
+              ruler_title_for_player(pdialog->pplayer,
+                                     plr_buf, sizeof(plr_buf)));
   XtVaCreateManagedWidget("inteldiplonamelabel", 
 			  labelWidgetClass, 
 			  pdialog->intel_diplo_form, 
@@ -481,11 +498,11 @@ void update_intel_diplo_dialog(struct intel_dialog *pdialog)
       if (other == pdialog->pplayer) {
 	continue;
       }
-      state = pplayer_get_diplstate(pdialog->pplayer, other);
-      my_snprintf(namelist_text[i], sizeof(namelist_text[i]),
+      state = player_diplstate_get(pdialog->pplayer, other);
+      fc_snprintf(namelist_text[i], sizeof(namelist_text[i]),
 		  "%-32s %-16s %-16s",
-		  other->name,
-		  get_nation_name(other->nation),
+		  player_name(other),
+		  nation_adjective_for_player(other),
 		  diplstate_text(state->type));
       namelist_ptrs[i] = namelist_text[i];
       i++;
