@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -210,7 +209,7 @@ void update_unit_info_label(struct unit_list *punitlist)
 {
   char buffer[512];
 
-  my_snprintf(buffer, sizeof(buffer), "%s\n%s",
+  fc_snprintf(buffer, sizeof(buffer), "%s\n%s",
               get_unit_info_label_text1(punitlist),
               get_unit_info_label_text2(punitlist, 0));
   xaw_set_label(unit_info_label, buffer);
@@ -862,3 +861,11 @@ void tileset_changed(void)
   update_info_label();
   reset_unit_below_pixmaps();
 }
+
+/**************************************************************************
+ Sets the position of the overview scroll window based on mapview position.
+**************************************************************************/
+void update_overview_scroll_window_pos(int x, int y)
+{
+  /* TODO: PORTME. */
+}
