@@ -207,6 +207,7 @@ static struct MenuEntry order_menu_entries[]={
     { { N_("Go to/Airlift to City..."), 0 }, "t", MENU_ORDER_GOTO_CITY, 0 },
     { { N_("Return to Nearest City"), 0 },   "G", MENU_ORDER_RETURN, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
+    { { N_("Convert Unit"), 0         },     "O", MENU_ORDER_CONVERT, 0 },
     { { N_("Disband Unit"), 0         },     "D", MENU_ORDER_DISBAND, 0 },
     { { N_("Help Build Wonder"), 0    },     "b", MENU_ORDER_BUILD_WONDER, 0 },
     { { N_("Establish Trade Route"), 0 },    "r", MENU_ORDER_TRADE_ROUTE, 0 },
@@ -259,6 +260,7 @@ static struct MenuEntry help_menu_entries[]={
     { { N_("Technology"), 0           },      "", MENU_HELP_TECH, 0 },
     { { N_("Space Race"), 0           },      "", MENU_HELP_SPACE_RACE, 0 },
     { { N_("About Ruleset"), 0        },      "", MENU_HELP_RULESET, 0 },
+    { { N_("About Nations"), 0        },      "", MENU_HELP_NATIONS, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Connecting"), 0           },      "", MENU_HELP_CONNECTING, 0 },
     { { N_("Controls"), 0             },      "", MENU_HELP_CONTROLS, 0 },
@@ -285,10 +287,19 @@ static char *menu_entry_text(enum MenuIndex menu, int ent, int var,
 static void revolution_menu_callback(Widget w, XtPointer client_data,
 				     XtPointer garbage);
 
+/**************************************************************************
+  Initialize menus (sensitivity, name, etc.) based on the
+  current state and current ruleset, etc.  Call menus_update().
+**************************************************************************/
+void real_menus_init(void)
+{
+  /* PORTME */
+}
+
 /****************************************************************
 ...
 *****************************************************************/
-void update_menus(void)
+void real_menus_update(void)
 {
   if (!can_client_change_view()) {
     XtSetSensitive(menus[MENU_REPORT]->button, False);
@@ -331,7 +342,7 @@ void update_menus(void)
       XtDestroyWidget(government_widgets[i]);
     }
     i = 0;
-    government_iterate(pgovernment) {
+    governments_iterate(pgovernment) {
       Widget w;
 
       if (pgovernment == game.government_during_revolution) {
@@ -347,7 +358,7 @@ void update_menus(void)
 
       government_widgets[i] = w;
       i++;
-    } government_iterate_end;
+    } governments_iterate_end;
     num_government_entries = i;
 
     menu_entry_sensitive(MENU_VIEW, MENU_VIEW_SHOW_CITY_GROWTH,
@@ -391,7 +402,7 @@ void update_menus(void)
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_SELECT_SAME_TYPE, True);
 
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_BUILD_CITY,
-			   can_units_do(punits, can_unit_add_or_build_city));
+                           can_units_do(punits, unit_can_add_or_build_city));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_ROAD, 
 			   can_units_do_activity(punits, ACTIVITY_ROAD)
 			   || can_units_do_activity(punits,
@@ -431,6 +442,8 @@ void update_menus(void)
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_AUTO_SETTLER,
 			   can_units_do(punits, can_unit_do_autosettlers));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_AUTO_ATTACK, false);
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_CONVERT,
+			   units_can_convert(punits));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_DISBAND,
 			   units_have_flag(punits, F_UNDISBANDABLE, FALSE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_AUTO_EXPLORE, 
@@ -538,7 +551,7 @@ static void game_menu_callback(Widget w, XtPointer client_data,
 
   switch(pane_num) {
   case MENU_GAME_OPTIONS:
-    popup_option_dialog();
+    option_dialog_popup(_("Local Options"), client_optset);
     break;
   case MENU_GAME_MSG_OPTIONS:
     popup_messageopt_dialog();
@@ -547,7 +560,7 @@ static void game_menu_callback(Widget w, XtPointer client_data,
     options_save();
     break;
   case MENU_GAME_SERVER_OPTIONS:
-    popup_settable_options_dialog();
+    option_dialog_popup(_("Game Settings"), server_optset);
     break;
   case MENU_GAME_OUTPUT_LOG:
     log_output_window();
@@ -777,6 +790,9 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
       request_unit_return(punit);
     } unit_list_iterate_end;
     break;
+  case MENU_ORDER_CONVERT:
+    key_unit_convert();
+    break;
   case MENU_ORDER_DISBAND:
     key_unit_disband();
     break;
@@ -814,19 +830,19 @@ static void reports_menu_callback(Widget w, XtPointer client_data,
 
   switch(pane_num) {
    case MENU_REPORT_CITIES:
-    popup_city_report_dialog(0);
+    city_report_dialog_popup(FALSE);
     break;
    case MENU_REPORT_UNITS:
-    popup_activeunits_report_dialog(0);
+    units_report_dialog_popup(FALSE);
     break;
   case MENU_REPORT_PLAYERS:
     popup_players_dialog(FALSE);
     break;
    case MENU_REPORT_ECONOMY:
-    popup_economy_report_dialog(0);
+    economy_report_dialog_popup(FALSE);
     break;
    case MENU_REPORT_SCIENCE:
-    popup_science_dialog(0);
+    science_report_dialog_popup(FALSE);
     break;
    case MENU_REPORT_WOW:
     send_report_request(REPORT_WONDERS_OF_THE_WORLD);
@@ -835,7 +851,7 @@ static void reports_menu_callback(Widget w, XtPointer client_data,
     send_report_request(REPORT_TOP_5_CITIES);
     break;
   case MENU_REPORT_MESSAGES:
-    popup_meswin_dialog(FALSE);
+    meswin_dialog_popup(FALSE);
     break;
    case MENU_REPORT_DEMOGRAPHIC:
     send_report_request(REPORT_DEMOGRAPHIC);
@@ -937,6 +953,9 @@ static void help_menu_callback(Widget w, XtPointer client_data,
   case MENU_HELP_RULESET:
     popup_help_dialog_string(HELP_RULESET_ITEM);
     break;
+  case MENU_HELP_NATIONS:
+    popup_help_dialog_string(HELP_NATIONS_ITEM);
+    break;
   case MENU_HELP_COPYING:
     popup_help_dialog_string(HELP_COPYING_ITEM);
     break;
@@ -1110,11 +1129,11 @@ static char *menu_entry_text(enum MenuIndex menu, int ent, int var,
   xlt=Q_(pmenu->entries[ent].text[var]);
 
   if (strstr(xlt, "%s")) {
-    my_snprintf(tmp, sizeof(tmp), xlt, terr);
+    fc_snprintf(tmp, sizeof(tmp), xlt, terr);
     xlt=tmp;
   }
 
-  my_snprintf(retbuf, sizeof(retbuf), "%*.*s%*.*s",
+  fc_snprintf(retbuf, sizeof(retbuf), "%*.*s%*.*s",
 	  -pmenu->maxitemlen, pmenu->maxitemlen, xlt,
 	  pmenu->maxacellen, pmenu->maxacellen, pmenu->entries[ent].acel);
 
