@@ -729,7 +729,7 @@ const char *science_dialog_text(void)
     int total = total_bulbs_required(client_player());
 
     if (perturn > 0) {
-      int turns = MAX(1, ceil((double) (total - done) / perturn));
+      int turns = MAX(1, ceil((double)total) / perturn);
 
       astr_add(&str, PL_("Progress: %d turn/advance",
                          "Progress: %d turns/advance",
@@ -826,8 +826,8 @@ const char *get_science_target_text(double *percent)
 const char *get_science_goal_text(Tech_type_id goal)
 {
   int steps = num_unknown_techs_for_goal(client.conn.playing, goal);
-  int bulbs = total_bulbs_required_for_goal(client.conn.playing, goal);
-  int bulbs_needed = bulbs, turns;
+  int bulbs_needed = total_bulbs_required_for_goal(client.conn.playing, goal);
+  int turns;
   int perturn = get_bulbs_per_turn(NULL, NULL, NULL);
   char buf1[256], buf2[256], buf3[256];
   struct player_research* research = player_research_get(client_player());
@@ -848,7 +848,7 @@ const char *get_science_goal_text(Tech_type_id goal)
   fc_snprintf(buf1, sizeof(buf1),
               PL_("%d step", "%d steps", steps), steps);
   fc_snprintf(buf2, sizeof(buf2),
-              PL_("%d bulb", "%d bulbs", bulbs), bulbs);
+              PL_("%d bulb", "%d bulbs", bulbs_needed), bulbs_needed);
   if (perturn > 0) {
     turns = (bulbs_needed + perturn - 1) / perturn;
     fc_snprintf(buf3, sizeof(buf3),
@@ -961,7 +961,7 @@ const char *get_info_label_text_popup(void)
     astr_add_line(&str, _("Global warming chance: %d%% (%+d%%/turn)"),
                   chance, rate);
   } else {
-    astr_add_line(&str, _("Global Warming deactivated."));
+    astr_add_line(&str, _("Global warming deactivated."));
   }
 
   if (game.info.nuclear_winter) {
@@ -1259,7 +1259,7 @@ const char *get_global_warming_tooltip(void)
   astr_clear(&str);
 
   if (!game.info.global_warming) {
-    astr_add_line(&str, _("Global Warming deactivated."));
+    astr_add_line(&str, _("Global warming deactivated."));
   } else {
     int chance, rate;
     global_warming_scaled(&chance, &rate, 100);
