@@ -15,10 +15,10 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <string.h>
 
 /* utility */
+#include "bitvector.h"
 #include "log.h"
 #include "mem.h"
 
@@ -44,7 +44,7 @@ static inline int single_move_cost(const struct pf_parameter *param,
       && !is_safe_ocean(dest_tile)) {
     return PF_IMPOSSIBLE_MC;
   } else if (uclass_has_flag(param->uclass, UCF_TERRAIN_SPEED)) {
-    return map_move_cost(src_tile, dest_tile);
+    return map_move_cost(param->owner, src_tile, dest_tile);
   } else {
     return SINGLE_MOVE;
   }
@@ -324,7 +324,7 @@ static int igter_move_unit(const struct tile *ptile,
       move_cost = MOVE_COST_ROAD;
     }
   } else if (uclass_has_flag(param->uclass, UCF_TERRAIN_SPEED)) {
-    move_cost = (map_move_cost(ptile, ptile1) != 0
+    move_cost = (map_move_cost(param->owner, ptile, ptile1) != 0
                  ? MOVE_COST_ROAD : 0);
   } else {
     move_cost = SINGLE_MOVE;
@@ -671,7 +671,6 @@ static void pft_fill_default_parameter(struct pf_parameter *parameter,
 {
   struct unit_class *punitclass = utype_class(punittype);
 
-  parameter->turn_mode = TM_CAPPED;
   parameter->unknown_MC = SINGLE_MOVE;
 
   if (uclass_has_flag(punitclass, UCF_TERRAIN_SPEED)) {
@@ -738,6 +737,12 @@ static void pft_fill_utype_default_parameter(struct pf_parameter *parameter,
   }
   parameter->owner = powner;
 
+  if (!BV_ISSET(parameter->unit_flags, F_CIVILIAN)) {
+    parameter->can_invade_tile = player_can_invade_tile;
+  } else {
+    parameter->can_invade_tile = NULL;
+  }
+
   parameter->omniscience = !ai_handicap(powner, H_MAP);
 }
 
@@ -795,14 +800,13 @@ static void pft_fill_parameter(struct pf_parameter *parameter,
     parameter->get_MC = airmove;
     break;
   default:
-    freelog(LOG_ERROR, "pft_fill_parameter() impossible move type!");
+    log_error("pft_fill_parameter() impossible move type!");
     break;
   }
 
   if (!parameter->get_moves_left_req && utype_fuel(punittype)) {
     /* Unit needs fuel */
     parameter->get_moves_left_req = get_fuel_moves_left_req;
-    parameter->turn_mode = TM_WORST_TIME;
   }
 
   if (!unit_type_really_ignores_zoc(punittype)) {
@@ -856,7 +860,7 @@ static void pft_fill_overlap_param(struct pf_parameter *parameter,
     parameter->get_MC = airmove; /* very crude */
     break;
   default:
-    freelog(LOG_ERROR, "pft_fill_overlap_param() impossible move type!");
+    log_error("pft_fill_overlap_param() impossible move type!");
     break;
   }
 
@@ -912,7 +916,7 @@ static void pft_fill_attack_param(struct pf_parameter *parameter,
     parameter->get_MC = airmove; /* very crude */
     break;
   default:
-    freelog(LOG_ERROR, "pft_fill_attack_param() impossible move type!");
+    log_error("pft_fill_attack_param() impossible move type!");
     break;
   }
 
@@ -975,8 +979,6 @@ void pft_fill_amphibious_parameter(struct pft_amphibious *parameter)
   parameter->sea_scale = move_rate / parameter->sea.move_rate;
   parameter->combined.moves_left_initially *= parameter->sea_scale;
   parameter->combined.move_rate = move_rate;
-  /* To ensure triremes behave correctly: FIXME: Probably incorrect now */
-  parameter->combined.turn_mode = TM_WORST_TIME;
   parameter->combined.get_MC = amphibious_move;
   parameter->combined.get_TB = amphibious_behaviour;
   parameter->combined.get_EC = amphibious_extra_cost;
@@ -1007,10 +1009,10 @@ struct pf_path *pft_concat(struct pf_path *dest_path,
   } else {
     int old_length = dest_path->length;
 
-    assert(pf_path_get_last_position(dest_path)->tile
-	   == src_path->positions[0].tile);
-    assert(pf_path_get_last_position(dest_path)->moves_left
-	   == src_path->positions[0].moves_left);
+    fc_assert_ret_val(pf_path_last_position(dest_path)->tile
+                      == src_path->positions[0].tile, NULL);
+    fc_assert_ret_val(pf_path_last_position(dest_path)->moves_left
+                      == src_path->positions[0].moves_left, NULL);
     dest_path->length += src_path->length - 1;
     dest_path->positions =
 	fc_realloc(dest_path->positions,
@@ -1040,7 +1042,7 @@ bool pft_advance_path(struct pf_path *path, struct tile *ptile)
       return FALSE;
     }
   }
-  assert(i < path->length);
+  fc_assert_ret_val(i < path->length, FALSE);
   path->length -= i;
   new_positions = fc_malloc(sizeof(*path->positions) * path->length);
   memcpy(new_positions, path->positions + i,
