@@ -13,8 +13,10 @@
 #ifndef FC__FEATURED_TEXT_H
 #define FC__FEATURED_TEXT_H
 
-#include "shared.h"             /* bool type. */
+/* utility */
+#include "support.h"            /* bool type. */
 
+/* common */
 #include "fc_types.h"
 
 /*
@@ -136,18 +138,14 @@ struct ft_color {
   const char *foreground;
   const char *background;
 };
-
+#define FT_COLOR(fg, bg) { .foreground = fg, .background = bg }
 /**************************************************************************
-  Builds a featured text color.
+  Constructor.
 **************************************************************************/
 static inline struct ft_color ft_color(const char *foreground,
                                        const char *background)
 {
-  struct ft_color color = {
-    .foreground = foreground,
-    .background = background
-  };
-
+  struct ft_color color = FT_COLOR(foreground, background);
   return color;
 }
 
@@ -189,7 +187,7 @@ extern const struct ft_color ftc_vote_abstain;
 /* Main functions. */
 size_t featured_text_to_plain_text(const char *featured_text,
                                    char *plain_text, size_t plain_text_len,
-                                   struct text_tag_list *tags);
+                                   struct text_tag_list **tags);
 size_t featured_text_apply_tag(const char *text_source,
                                char *featured_text, size_t featured_text_len,
                                enum text_tag_type tag_type,
@@ -197,15 +195,19 @@ size_t featured_text_apply_tag(const char *text_source,
                                ft_offset_t stop_offset,
                                ...);
 
-/* Text tag list functions. */
-void text_tag_list_clear_all(struct text_tag_list *tags);
-struct text_tag_list *text_tag_list_dup(const struct text_tag_list *tags);
+/* Text tag list functions. NB: Overwrite the ones defined in speclist.h. */
+#define text_tag_list_new() \
+  text_tag_list_new_full(text_tag_destroy)
+#define text_tag_list_copy(tags) \
+  text_tag_list_copy_full(tags, text_tag_copy, text_tag_destroy)
+
 
 /* Text tag functions. */
 struct text_tag *text_tag_new(enum text_tag_type tag_type,
                               ft_offset_t start_offset,
                               ft_offset_t stop_offset,
                               ...);
+struct text_tag *text_tag_copy(const struct text_tag *ptag);
 void text_tag_destroy(struct text_tag *ptag);
 
 enum text_tag_type text_tag_type(const struct text_tag *ptag);
