@@ -15,29 +15,21 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
+/* utility */
 #include "fcintl.h"
-#include "game.h"
 #include "log.h"
-#include "map.h"
 #include "mem.h"
-#include "shared.h" /* ARRAY_SIZE */
+#include "shared.h"     /* ARRAY_SIZE */
+#include "string_vector.h"
 #include "support.h"
+
+/* common */
+#include "game.h"
+#include "map.h"
 #include "tech.h"
 
 #include "improvement.h"
 
-static const char *genus_names[IG_LAST] = {
-  "GreatWonder", "SmallWonder", "Improvement", "Special"
-};
-
-static const char *flag_names[] = {
-  "VisibleByOthers", "SaveSmallWonder", "Gold"
-};
-/* Note that these strings must correspond with the enums in impr_flag_id,
-   in common/improvement.h */
-
 /**************************************************************************
 All the city improvements:
 Use improvement_by_number(id) to access the array.
@@ -47,22 +39,6 @@ The improvement_types array is now setup in:
 **************************************************************************/
 static struct impr_type improvement_types[B_LAST];
 
-/**************************************************************************
-  Convert impr genus names to enum; case insensitive;
-  returns IG_LAST if can't match.
-**************************************************************************/
-enum impr_genus_id find_genus_by_rule_name(const char *s)
-{
-  enum impr_genus_id i;
-
-  for(i = 0; i < ARRAY_SIZE(genus_names); i++) {
-    if (mystrcasecmp(genus_names[i], s) == 0) {
-      break;
-    }
-  }
-  return i;
-}
-
 /****************************************************************************
   Initialize building structures.
 ****************************************************************************/
@@ -85,8 +61,10 @@ void improvements_init(void)
 **************************************************************************/
 static void improvement_free(struct impr_type *p)
 {
-  free(p->helptext);
-  p->helptext = NULL;
+  if (NULL != p->helptext) {
+    strvec_destroy(p->helptext);
+    p->helptext = NULL;
+  }
 
   requirement_vector_free(&p->reqs);
 }
@@ -139,7 +117,7 @@ Impr_type_id improvement_count(void)
 **************************************************************************/
 Impr_type_id improvement_index(const struct impr_type *pimprove)
 {
-  assert(pimprove);
+  fc_assert_ret_val(NULL != pimprove, -1);
   return pimprove - improvement_types;
 }
 
@@ -148,7 +126,7 @@ Impr_type_id improvement_index(const struct impr_type *pimprove)
 **************************************************************************/
 Impr_type_id improvement_number(const struct impr_type *pimprove)
 {
-  assert(pimprove);
+  fc_assert_ret_val(NULL != pimprove, -1);
   return pimprove->item_number;
 }
 
@@ -205,15 +183,9 @@ struct impr_type *valid_improvement_by_number(const Impr_type_id id)
   Return the (translated) name of the given improvement. 
   You don't have to free the return pointer.
 **************************************************************************/
-const char *improvement_name_translation(struct impr_type *pimprove)
+const char *improvement_name_translation(const struct impr_type *pimprove)
 {
-  if (NULL == pimprove->name.translated) {
-    /* delayed (unified) translation */
-    pimprove->name.translated = ('\0' == pimprove->name.vernacular[0])
-				? pimprove->name.vernacular
-				: Q_(pimprove->name.vernacular);
-  }
-  return pimprove->name.translated;
+  return name_translation(&pimprove->name);
 }
 
 /****************************************************************************
@@ -222,7 +194,7 @@ const char *improvement_name_translation(struct impr_type *pimprove)
 ****************************************************************************/
 const char *improvement_rule_name(const struct impr_type *pimprove)
 {
-  return Qn_(pimprove->name.vernacular); 
+  return rule_name(&pimprove->name);
 }
 
 /****************************************************************************
@@ -281,7 +253,7 @@ bool is_wonder(const struct impr_type *pimprove)
   Does a linear search of improvement_types[].name.translated
   Returns NULL when none match.
 **************************************************************************/
-struct impr_type *find_improvement_by_translated_name(const char *name)
+struct impr_type *improvement_by_translated_name(const char *name)
 {
   improvement_iterate(pimprove) {
     if (0 == strcmp(improvement_name_translation(pimprove), name)) {
@@ -296,12 +268,12 @@ struct impr_type *find_improvement_by_translated_name(const char *name)
   Does a linear search of improvement_types[].name.vernacular
   Returns NULL when none match.
 ****************************************************************************/
-struct impr_type *find_improvement_by_rule_name(const char *name)
+struct impr_type *improvement_by_rule_name(const char *name)
 {
   const char *qname = Qn_(name);
 
   improvement_iterate(pimprove) {
-    if (0 == mystrcasecmp(improvement_rule_name(pimprove), qname)) {
+    if (0 == fc_strcasecmp(improvement_rule_name(pimprove), qname)) {
       return pimprove;
     }
   } improvement_iterate_end;
@@ -315,26 +287,8 @@ struct impr_type *find_improvement_by_rule_name(const char *name)
 bool improvement_has_flag(const struct impr_type *pimprove,
 			  enum impr_flag_id flag)
 {
-  assert(flag >= 0 && flag < IF_LAST);
-  return TEST_BIT(pimprove->flags, flag);
-}
-
-/**************************************************************************
- Convert flag names to enum; case insensitive;
- returns IF_LAST if can't match.
-**************************************************************************/
-enum impr_flag_id find_improvement_flag_by_rule_name(const char *s)
-{
-  enum impr_flag_id i;
-
-  assert(ARRAY_SIZE(flag_names) == IF_LAST);
-  
-  for(i = 0; i < IF_LAST; i++) {
-    if (mystrcasecmp(flag_names[i], s) == 0) {
-      return i;
-    }
-  }
-  return IF_LAST;
+  fc_assert_ret_val(impr_flag_id_is_valid(flag), FALSE);
+  return BV_ISSET(pimprove->flags, flag);
 }
 
 /**************************************************************************
@@ -387,6 +341,39 @@ bool impr_provides_buildable_units(const struct player *pplayer,
   return FALSE;
 }
 
+/**************************************************************************
+  Returns TRUE if an improvement in a city is redundant, that is, the
+  city wouldn't lose anything by losing the improvement, or gain anything
+  by building it. This means:
+   - all of its effects (if any) are provided by other means, or it's
+     obsolete (and thus assumed to have no effect); and
+   - it's not enabling the city to build some kind of units; and
+   - it's not Coinage (IF_GOLD).
+  (Note that it's not impossible that this improvement could become useful
+  if circumstances changed, say if a new government enabled the building
+  of its special units.)
+**************************************************************************/
+bool is_improvement_redundant(const struct city *pcity,
+                              struct impr_type *pimprove)
+{
+  /* A capitalization production is never redundant. */
+  if (improvement_has_flag(pimprove, IF_GOLD)) {
+    return FALSE;
+  }
+
+  /* If an improvement allows building of units, don't claim it's redundant.
+   * (FIXME: if enabling units were done via the effects system, then we
+   * could decide which of two improvements enabling a unit was "better".) */
+  if (impr_provides_buildable_units(city_owner(pcity), pimprove)) {
+    return FALSE;
+  }
+
+  /* Otherwise, it's redundant if its effects are available by other means,
+   * or if it's an obsolete wonder (great or small). */
+  return is_building_replaced(pcity, pimprove, RPT_CERTAIN)
+      || improvement_obsolete(city_owner(pcity), pimprove);
+}
+
 /**************************************************************************
    Whether player can build given building somewhere, ignoring whether it
    is obsolete.
@@ -532,8 +519,8 @@ void wonder_built(const struct city *pcity, const struct impr_type *pimprove)
   struct player *pplayer;
   int index = improvement_number(pimprove);
 
-  RETURN_IF_FAIL(NULL != pcity);
-  RETURN_IF_FAIL(is_wonder(pimprove));
+  fc_assert_ret(NULL != pcity);
+  fc_assert_ret(is_wonder(pimprove));
 
   pplayer = city_owner(pcity);
   pplayer->wonders[index] = pcity->id;
@@ -553,15 +540,15 @@ void wonder_destroyed(const struct city *pcity,
   struct player *pplayer;
   int index = improvement_number(pimprove);
 
-  RETURN_IF_FAIL(NULL != pcity);
-  RETURN_IF_FAIL(is_wonder(pimprove));
+  fc_assert_ret(NULL != pcity);
+  fc_assert_ret(is_wonder(pimprove));
 
   pplayer = city_owner(pcity);
-  RETURN_IF_FAIL(pplayer->wonders[index] == pcity->id);
+  fc_assert_ret(pplayer->wonders[index] == pcity->id);
   pplayer->wonders[index] = WONDER_NOT_BUILT;
 
   if (is_great_wonder(pimprove)) {
-    RETURN_IF_FAIL(game.info.great_wonder_owners[index]
+    fc_assert_ret(game.info.great_wonder_owners[index]
                    == player_number(pplayer));
     game.info.great_wonder_owners[index] = WONDER_DESTROYED;
   }
@@ -573,8 +560,8 @@ void wonder_destroyed(const struct city *pcity,
 bool wonder_is_built(const struct player *pplayer,
                      const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(NULL != pplayer, NULL);
-  RETURN_VAL_IF_FAIL(is_wonder(pimprove), NULL);
+  fc_assert_ret_val(NULL != pplayer, NULL);
+  fc_assert_ret_val(is_wonder(pimprove), NULL);
 
   return WONDER_BUILT(pplayer->wonders[improvement_index(pimprove)]);
 }
@@ -583,13 +570,13 @@ bool wonder_is_built(const struct player *pplayer,
   Get the world city with this wonder (small or great).  This doesn't
   always success on the client side.
 **************************************************************************/
-struct city *find_city_from_wonder(const struct player *pplayer,
-                                   const struct impr_type *pimprove)
+struct city *city_from_wonder(const struct player *pplayer,
+                              const struct impr_type *pimprove)
 {
   int city_id = pplayer->wonders[improvement_index(pimprove)];
 
-  RETURN_VAL_IF_FAIL(NULL != pplayer, NULL);
-  RETURN_VAL_IF_FAIL(is_wonder(pimprove), NULL);
+  fc_assert_ret_val(NULL != pplayer, NULL);
+  fc_assert_ret_val(is_wonder(pimprove), NULL);
 
   if (!WONDER_BUILT(city_id)) {
     return NULL;
@@ -598,20 +585,20 @@ struct city *find_city_from_wonder(const struct player *pplayer,
 #ifdef DEBUG
   if (is_server()) {
     /* On client side, this info is not always known. */
-    struct city *pcity = player_find_city_by_id(pplayer, city_id);
+    struct city *pcity = player_city_by_number(pplayer, city_id);
 
     if (NULL == pcity) {
-      freelog(LOG_ERROR, "Player %s (nb %d) has outdated wonder info for "
-              "%s (nb %d), it points to city nb %d.",
-              player_name(pplayer), player_number(pplayer),
-              improvement_rule_name(pimprove), improvement_number(pimprove),
-              city_id);
+      log_error("Player %s (nb %d) has outdated wonder info for "
+                "%s (nb %d), it points to city nb %d.",
+                player_name(pplayer), player_number(pplayer),
+                improvement_rule_name(pimprove),
+                improvement_number(pimprove), city_id);
     } else if (!city_has_building(pcity, pimprove)) {
-      freelog(LOG_ERROR, "Player %s (nb %d) has outdated wonder info for "
-              "%s (nb %d), the city %s (nb %d) doesn't have this wonder.",
-              player_name(pplayer), player_number(pplayer),
-              improvement_rule_name(pimprove), improvement_number(pimprove),
-              city_name(pcity), pcity->id);
+      log_error("Player %s (nb %d) has outdated wonder info for "
+                "%s (nb %d), the city %s (nb %d) doesn't have this wonder.",
+                player_name(pplayer), player_number(pplayer),
+                improvement_rule_name(pimprove),
+                improvement_number(pimprove), city_name(pcity), pcity->id);
       return NULL;
     }
 
@@ -619,7 +606,7 @@ struct city *find_city_from_wonder(const struct player *pplayer,
   }
 #endif /* DEBUG */
 
-  return player_find_city_by_id(pplayer, city_id);
+  return player_city_by_number(pplayer, city_id);
 }
 
 /**************************************************************************
@@ -627,7 +614,7 @@ struct city *find_city_from_wonder(const struct player *pplayer,
 **************************************************************************/
 bool great_wonder_is_built(const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+  fc_assert_ret_val(is_great_wonder(pimprove), FALSE);
 
   return WONDER_OWNED(game.info.great_wonder_owners
                       [improvement_index(pimprove)]);
@@ -638,7 +625,7 @@ bool great_wonder_is_built(const struct impr_type *pimprove)
 **************************************************************************/
 bool great_wonder_is_destroyed(const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+  fc_assert_ret_val(is_great_wonder(pimprove), FALSE);
 
   return (WONDER_DESTROYED
           == game.info.great_wonder_owners[improvement_index(pimprove)]);
@@ -649,7 +636,7 @@ bool great_wonder_is_destroyed(const struct impr_type *pimprove)
 **************************************************************************/
 bool great_wonder_is_available(const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), FALSE);
+  fc_assert_ret_val(is_great_wonder(pimprove), FALSE);
 
   return (WONDER_NOT_OWNED
           == game.info.great_wonder_owners[improvement_index(pimprove)]);
@@ -659,27 +646,28 @@ bool great_wonder_is_available(const struct impr_type *pimprove)
   Get the world city with this great wonder.  This doesn't always success
   on the client side.
 **************************************************************************/
-struct city *find_city_from_great_wonder(const struct impr_type *pimprove)
+struct city *city_from_great_wonder(const struct impr_type *pimprove)
 {
   int player_id = game.info.great_wonder_owners[improvement_index(pimprove)];
 
-  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), NULL);
+  fc_assert_ret_val(is_great_wonder(pimprove), NULL);
 
   if (WONDER_OWNED(player_id)) {
 #ifdef DEBUG
     const struct player *pplayer = player_by_number(player_id);
-    struct city *pcity = find_city_from_wonder(pplayer, pimprove);
+    struct city *pcity = city_from_wonder(pplayer, pimprove);
 
     if (is_server() && NULL == pcity) {
-      freelog(LOG_ERROR, "Game has outdated wonder info for %s (nb %d), "
-              "the player %s (nb %d) doesn't have this wonder.",
-              improvement_rule_name(pimprove), improvement_number(pimprove),
-              player_name(pplayer), player_number(pplayer));
+      log_error("Game has outdated wonder info for %s (nb %d), "
+                "the player %s (nb %d) doesn't have this wonder.",
+                improvement_rule_name(pimprove),
+                improvement_number(pimprove),
+                player_name(pplayer), player_number(pplayer));
     }
 
     return pcity;
 #else
-    return find_city_from_wonder(player_by_number(player_id), pimprove);
+    return city_from_wonder(player_by_number(player_id), pimprove);
 #endif /* DEBUG */
   } else {
     return NULL;
@@ -694,7 +682,7 @@ struct player *great_wonder_owner(const struct impr_type *pimprove)
 {
   int player_id = game.info.great_wonder_owners[improvement_index(pimprove)];
 
-  RETURN_VAL_IF_FAIL(is_great_wonder(pimprove), NULL);
+  fc_assert_ret_val(is_great_wonder(pimprove), NULL);
 
   if (WONDER_OWNED(player_id)) {
     return player_by_number(player_id);
@@ -709,7 +697,7 @@ struct player *great_wonder_owner(const struct impr_type *pimprove)
 bool small_wonder_is_built(const struct player *pplayer,
                            const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(is_small_wonder(pimprove), FALSE);
+  fc_assert_ret_val(is_small_wonder(pimprove), FALSE);
 
   return (NULL != pplayer
           && wonder_is_built(pplayer, pimprove));
@@ -718,15 +706,15 @@ bool small_wonder_is_built(const struct player *pplayer,
 /**************************************************************************
   Get the player city with this small wonder.
 **************************************************************************/
-struct city *find_city_from_small_wonder(const struct player *pplayer,
-                                         const struct impr_type *pimprove)
+struct city *city_from_small_wonder(const struct player *pplayer,
+                                    const struct impr_type *pimprove)
 {
-  RETURN_VAL_IF_FAIL(is_small_wonder(pimprove), NULL);
+  fc_assert_ret_val(is_small_wonder(pimprove), NULL);
 
   if (NULL == pplayer) {
     return NULL; /* Used in some places in the client. */
   } else {
-    return find_city_from_wonder(pplayer, pimprove);
+    return city_from_wonder(pplayer, pimprove);
   }
 }
 
