@@ -18,29 +18,36 @@ struct data_in;
 
 #include "connection.h"		/* struct connection, MAX_LEN_* */
 #include "diptreaty.h"
+#include "effects.h"
 #include "events.h"
-#include "improvement.h"
+#include "improvement.h"	/* bv_imprs */
 #include "map.h"
 #include "player.h"
-#include "shared.h"		/* MAX_LEN_NAME, MAX_LEN_ADDR */
+#include "requirements.h"
+#include "shared.h"		/* MAX_LEN_ADDR */
 #include "spaceship.h"
+#include "team.h"
 #include "unittype.h"
 #include "worklist.h"
 
 
 #define MAX_LEN_USERNAME        10        /* see below */
 #define MAX_LEN_MSG             1536
-#define MAX_ATTRIBUTE_BLOCK     (256*1024)	/* largest attribute block */
-#define ATTRIBUTE_CHUNK_SIZE    (1024*2)  /* attribute chunk size to use */
 #define MAX_LEN_ROUTE		2000	  /* MAX_LEN_PACKET/2 - header */
 
+/* The size of opaque (void *) data sent in the network packet.  To avoid
+ * fragmentation issues, this SHOULD NOT be larger than the standard
+ * ethernet or PPP 1500 byte frame size (with room for headers).
+ *
+ * Do not spend much time optimizing, you have no idea of the actual dynamic
+ * path characteristics between systems, such as VPNs and tunnels.
+ */
+#define ATTRIBUTE_CHUNK_SIZE    (1400)
+
 enum report_type {
   REPORT_WONDERS_OF_THE_WORLD,
   REPORT_TOP_5_CITIES,
-  REPORT_DEMOGRAPHIC,
-  REPORT_SERVER_OPTIONS,   /* obsolete */
-  REPORT_SERVER_OPTIONS1,
-  REPORT_SERVER_OPTIONS2
+  REPORT_DEMOGRAPHIC
 };
 
 enum spaceship_place_type {
@@ -76,19 +83,12 @@ void generic_handle_player_attribute_chunk(struct player *pplayer,
 					   const struct
 					   packet_player_attribute_chunk
 					   *chunk);
-const char *get_packet_name(enum packet_type type);
+const char *packet_name(enum packet_type type);
+bool packet_has_game_info_flag(enum packet_type type);
 
-void pre_send_packet_chat_msg(struct connection *pc,
-			      struct packet_chat_msg *packet);
-void post_receive_packet_chat_msg(struct connection *pc,
-				  struct packet_chat_msg *packet);
 void pre_send_packet_player_attribute_chunk(struct connection *pc,
 					    struct packet_player_attribute_chunk
 					    *packet);
-void post_receive_packet_game_state(struct connection *pc,
-				    struct packet_game_state *packet);
-void post_send_packet_game_state(struct connection *pc,
-				 const struct packet_game_state *packet);
 
 #define SEND_PACKET_START(type) \
   unsigned char buffer[MAX_LEN_PACKET]; \
@@ -129,4 +129,20 @@ void post_send_packet_game_state(struct connection *pc,
 int send_packet_data(struct connection *pc, unsigned char *data, int len);
 void check_packet(struct data_in *din, struct connection *pc);
 
+/* Utilities to exchange strings and string vectors. */
+#define PACKET_STRVEC_SEPARATOR '\3'
+#define PACKET_STRVEC_COMPUTE(str, strvec)                                  \
+  if (NULL != strvec) {                                                     \
+    strvec_to_str(strvec, PACKET_STRVEC_SEPARATOR, str, sizeof(str));       \
+  } else {                                                                  \
+    str[0] = '\0';                                                          \
+  }
+#define PACKET_STRVEC_EXTRACT(strvec, str)                                  \
+  if ('\0' != str[0]) {                                                     \
+    strvec = strvec_new();                                                  \
+    strvec_from_str(strvec, PACKET_STRVEC_SEPARATOR, str);                  \
+  } else {                                                                  \
+    strvec = NULL;                                                          \
+  }
+
 #endif  /* FC__PACKETS_H */
