@@ -15,14 +15,16 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
+/* utility */
 #include "astring.h"
+#include "bitvector.h"
 #include "fcintl.h"
 #include "mem.h"
 #include "shared.h"
 #include "support.h"
 
+/* common */
+#include "ai.h"
 #include "base.h"
 #include "city.h"
 #include "game.h"
@@ -32,9 +34,10 @@
 #include "packets.h"
 #include "player.h"
 #include "tech.h"
-#include "unit.h"
 #include "unitlist.h"
 
+#include "unit.h"
+
 /**************************************************************************
 bribe unit
 investigate
@@ -137,42 +140,135 @@ bool is_diplomat_action_available(const struct unit *pdiplomat,
   return FALSE;
 }
 
-/**************************************************************************
-FIXME: Maybe we should allow airlifts between allies
-**************************************************************************/
-bool unit_can_airlift_to(const struct unit *punit, const struct city *pcity)
+/****************************************************************************
+  Determines if punit can be airlifted to dest_city now!  So punit needs
+  to be in a city now.
+  If pdest_city is NULL, just indicate whether it's possible for the unit
+  to be airlifted at all from its current position.
+  The 'restriction' parameter specifies which player's knowledge this is
+  based on -- one player can't see whether another's cities are currently
+  able to airlift.  (Clients other than global observers should only call
+  this with a non-NULL 'restriction'.)
+****************************************************************************/
+enum unit_airlift_result
+    test_unit_can_airlift_to(const struct player *restriction,
+                             const struct unit *punit,
+                             const struct city *pdest_city)
 {
-  struct city *acity = tile_city(punit->tile);
+  const struct city *psrc_city = tile_city(punit->tile);
+  const struct player *punit_owner;
+  enum unit_airlift_result ok_result = AR_OK;
 
   if (0 == punit->moves_left) {
-    return FALSE;
+    /* No moves left. */
+    return AR_NO_MOVES;
   }
 
   if (!is_ground_unit(punit)) {
-    return FALSE;
+    /* Only land units can be airlifted currently. */
+    return AR_WRONG_UNITTYPE;
   }
 
-  if (NULL == acity) {
-    return FALSE;
+  if (0 < get_transporter_occupancy(punit)) {
+    /* Units with occupants can't be airlifted currently. */
+    return AR_OCCUPIED;
   }
 
-  if (acity == pcity) {
-    return FALSE;
+  if (NULL == psrc_city) {
+    /* No city there. */
+    return AR_NOT_IN_CITY;
   }
 
-  if (city_owner(acity) != unit_owner(punit)) {
-    return FALSE;
+  if (psrc_city == pdest_city) {
+    /* Airlifting to our current position doesn't make sense. */
+    return AR_BAD_DST_CITY;
   }
 
-  if (city_owner(acity) != city_owner(pcity)) {
-    return FALSE;
+  punit_owner = unit_owner(punit);
+
+  /* Check validity of both source and destination before checking capacity,
+   * to avoid misleadingly optimistic returns. */
+
+  if (punit_owner != city_owner(psrc_city)
+      && !(game.info.airlifting_style & AIRLIFTING_ALLIED_SRC
+           && pplayers_allied(punit_owner, city_owner(psrc_city)))) {
+    /* Not allowed to airlift from this source. */
+    return AR_BAD_SRC_CITY;
   }
 
-  if (acity->airlift <= 0 || pcity->airlift <= 0) {
-    return FALSE;
+  if (pdest_city &&
+      punit_owner != city_owner(pdest_city)
+      && !(game.info.airlifting_style & AIRLIFTING_ALLIED_DEST
+           && pplayers_allied(punit_owner, city_owner(pdest_city)))) {
+    /* Not allowed to airlift to this destination. */
+    return AR_BAD_DST_CITY;
   }
 
-  return TRUE;
+  if (NULL == restriction || city_owner(psrc_city) == restriction) {
+    /* We know for sure whether or not src can airlift this turn. */
+    if (0 >= psrc_city->airlift) {
+      /* The source cannot airlift for this turn (maybe already airlifted
+       * or no airport).
+       *
+       * Note that (game.info.airlifting_style & AIRLIFTING_UNLIMITED_SRC)
+       * is not handled here because it always needs an airport to airlift.
+       * See also do_airline() in server/unittools.h. */
+      return AR_SRC_NO_FLIGHTS;
+    } /* else, there is capacity; continue to other checks */
+  } else {
+    /* We don't have access to the 'airlift' field. Assume it's OK; can
+     * only find out for sure by trying it. */
+    ok_result = AR_OK_SRC_UNKNOWN;
+  }
+
+  if (pdest_city) {
+    if (NULL == restriction || city_owner(pdest_city) == restriction) {
+      if (0 >= pdest_city->airlift
+          && !(game.info.airlifting_style & AIRLIFTING_UNLIMITED_DEST)) {
+        /* The destination cannot support airlifted units for this turn
+         * (maybe already airlifed or no airport).
+         * See also do_airline() in server/unittools.h. */
+        return AR_DST_NO_FLIGHTS;
+      } /* else continue */
+    } else {
+      ok_result = AR_OK_DST_UNKNOWN;
+    }
+  }
+
+  return ok_result;
+}
+
+/****************************************************************************
+  Encapsulates whether a return from test_unit_can_airlift_to() should be
+  treated as a successful result.
+****************************************************************************/
+bool is_successful_airlift_result(enum unit_airlift_result result)
+{
+  switch (result) {
+  case AR_OK:
+  case AR_OK_SRC_UNKNOWN:
+  case AR_OK_DST_UNKNOWN:
+    return TRUE;
+  default:  /* everything else is failure */
+    return FALSE;
+  }
+}
+
+/****************************************************************************
+  Determines if punit can be airlifted to dest_city now!  So punit needs
+  to be in a city now.
+  On the server this gives correct information; on the client it errs on the
+  side of saying airlifting is possible even if it's not certain given
+  player knowledge.
+****************************************************************************/
+bool unit_can_airlift_to(const struct unit *punit,
+                         const struct city *pdest_city)
+{
+  /* FIXME: really we want client_player(), not unit_owner(). */
+  struct player *restriction = is_server() ? NULL : unit_owner(punit);
+  fc_assert_ret_val(pdest_city, FALSE);
+  return is_successful_airlift_result(
+      test_unit_can_airlift_to(restriction, punit, pdest_city));
 }
 
 /****************************************************************************
@@ -225,9 +321,9 @@ bool unit_can_est_trade_route_here(const struct unit *punit)
   struct city *phomecity, *pdestcity;
 
   return (unit_has_type_flag(punit, F_TRADE_ROUTE)
-	  && (pdestcity = tile_city(punit->tile))
-	  && (phomecity = game_find_city_by_number(punit->homecity))
-	  && can_cities_trade(phomecity, pdestcity));
+          && (pdestcity = tile_city(punit->tile))
+          && (phomecity = game_city_by_number(punit->homecity))
+          && can_cities_trade(phomecity, pdestcity));
 }
 
 /**************************************************************************
@@ -248,7 +344,7 @@ bool is_attack_unit(const struct unit *punit)
 
 /**************************************************************************
   Military units are capable of enforcing martial law. Military ground
-  and heli units can occupy empty cities -- see COULD_OCCUPY(punit).
+  and heli units can occupy empty cities -- see unit_can_take_over(punit).
   Some military units, like the Galleon, have no attack strength.
 **************************************************************************/
 bool is_military_unit(const struct unit *punit)
@@ -312,7 +408,7 @@ bool is_field_unit(const struct unit *punit)
 **************************************************************************/
 bool is_hiding_unit(const struct unit *punit)
 {
-  struct unit *transporter = game_find_unit_by_number(punit->transported_by);
+  struct unit *transporter = game_unit_by_number(punit->transported_by);
 
   return (unit_has_type_flag(punit, F_PARTIAL_INVIS)
 	  || (transporter && unit_has_type_flag(transporter, F_PARTIAL_INVIS)));
@@ -328,77 +424,94 @@ bool kills_citizen_after_attack(const struct unit *punit)
                   uclass_move_type(unit_class(punit)));
 }
 
-/**************************************************************************
+/****************************************************************************
   Return TRUE iff this unit may be disbanded to add its pop_cost to a
   city at its current location.
-**************************************************************************/
-bool can_unit_add_to_city(const struct unit *punit)
+****************************************************************************/
+bool unit_can_add_to_city(const struct unit *punit)
 {
-  return (test_unit_add_or_build_city(punit) == AB_ADD_OK);
+  return (UAB_ADD_OK == unit_add_or_build_city_test(punit));
 }
 
-/**************************************************************************
+/****************************************************************************
   Return TRUE iff this unit is capable of building a new city at its
   current location.
-**************************************************************************/
-bool can_unit_build_city(const struct unit *punit)
+****************************************************************************/
+bool unit_can_build_city(const struct unit *punit)
 {
-  return (test_unit_add_or_build_city(punit) == AB_BUILD_OK);
+  return (UAB_BUILD_OK == unit_add_or_build_city_test(punit));
 }
 
-/**************************************************************************
+/****************************************************************************
   Return TRUE iff this unit can add to a current city or build a new city
   at its current location.
-**************************************************************************/
-bool can_unit_add_or_build_city(const struct unit *punit)
+****************************************************************************/
+bool unit_can_add_or_build_city(const struct unit *punit)
 {
-  enum add_build_city_result r = test_unit_add_or_build_city(punit);
+  enum unit_add_build_city_result res = unit_add_or_build_city_test(punit);
 
-  return (r == AB_BUILD_OK || r == AB_ADD_OK);
+  return (UAB_BUILD_OK == res || UAB_ADD_OK == res);
 }
 
-/**************************************************************************
+/****************************************************************************
   See if the unit can add to an existing city or build a new city at
   its current location, and return a 'result' value telling what is
   allowed.
-**************************************************************************/
-enum add_build_city_result test_unit_add_or_build_city(const struct unit *
-						       punit)
+****************************************************************************/
+enum unit_add_build_city_result
+unit_add_or_build_city_test(const struct unit *punit)
 {
-  struct city *pcity = tile_city(punit->tile);
+  struct tile *ptile = unit_tile(punit);
+  struct city *pcity = tile_city(ptile);
   bool is_build = unit_has_type_flag(punit, F_CITIES);
   bool is_add = unit_has_type_flag(punit, F_ADD_TO_CITY);
   int new_pop;
 
-  /* See if we can build */
-  if (!pcity) {
-    if (!is_build)
-      return AB_NOT_BUILD_UNIT;
-    if (punit->moves_left == 0)
-      return AB_NO_MOVES_BUILD;
-    if (!city_can_be_built_here(punit->tile, punit)) {
-      return AB_NOT_BUILD_LOC;
+  /* Test if we can build. */
+  if (NULL == pcity) {
+    if (!is_build) {
+      return UAB_NOT_BUILD_UNIT;
     }
-    return AB_BUILD_OK;
+    if (punit->moves_left == 0) {
+      return UAB_NO_MOVES_BUILD;
+    }
+    switch (city_build_here_test(ptile, punit)) {
+    case CB_OK:
+      return UAB_BUILD_OK;
+    case CB_BAD_CITY_TERRAIN:
+      return UAB_BAD_CITY_TERRAIN;
+    case CB_BAD_UNIT_TERRAIN:
+      return UAB_BAD_UNIT_TERRAIN;
+    case CB_BAD_BORDERS:
+      return UAB_BAD_BORDERS;
+    case CB_NO_MIN_DIST:
+      return UAB_NO_MIN_DIST;
+    }
+    log_error("%s(): Internal error.", __FUNCTION__);
+    return UAB_NO_MOVES_BUILD; /* Returns something prohibitive. */
   }
-  
-  /* See if we can add */
 
-  if (!is_add)
-    return AB_NOT_ADDABLE_UNIT;
-  if (punit->moves_left == 0)
-    return AB_NO_MOVES_ADD;
+  /* Test if we can add. */
+  if (!is_add) {
+    return UAB_NOT_ADDABLE_UNIT;
+  }
+  if (punit->moves_left == 0) {
+    return UAB_NO_MOVES_ADD;
+  }
 
-  assert(unit_pop_value(punit) > 0);
+  fc_assert(unit_pop_value(punit) > 0);
   new_pop = pcity->size + unit_pop_value(punit);
 
-  if (new_pop > game.info.add_to_size_limit)
-    return AB_TOO_BIG;
-  if (city_owner(pcity) != unit_owner(punit))
-    return AB_NOT_OWNER;
-  if (!city_can_grow_to(pcity, new_pop))
-    return AB_NO_SPACE;
-  return AB_ADD_OK;
+  if (new_pop > game.info.add_to_size_limit) {
+    return UAB_TOO_BIG;
+  }
+  if (city_owner(pcity) != unit_owner(punit)) {
+    return UAB_NOT_OWNER;
+  }
+  if (!city_can_grow_to(pcity, new_pop)) {
+    return UAB_NO_SPACE;
+  }
+  return UAB_ADD_OK;
 }
 
 /**************************************************************************
@@ -473,7 +586,8 @@ int get_activity_rate_this_turn(const struct unit *punit)
 /**************************************************************************
   Return the estimated number of turns for the worker unit to start and
   complete the activity at the given location.  This assumes no other
-  worker units are helping out.
+  worker units are helping out, and doesn't take account of any work
+  already done by this unit.
 **************************************************************************/
 int get_turns_for_activity_at(const struct unit *punit,
 			      enum unit_activity activity,
@@ -491,6 +605,38 @@ int get_turns_for_activity_at(const struct unit *punit,
   }
 }
 
+/**************************************************************************
+  Return TRUE if activity requires some sort of target to be specified.
+**************************************************************************/
+bool activity_requires_target(enum unit_activity activity)
+{
+  switch (activity) {
+  case ACTIVITY_PILLAGE:
+  case ACTIVITY_BASE:
+    return TRUE;
+  case ACTIVITY_IDLE:
+  case ACTIVITY_POLLUTION:
+  case ACTIVITY_ROAD:
+  case ACTIVITY_MINE:
+  case ACTIVITY_IRRIGATE:
+  case ACTIVITY_FORTIFIED:
+  case ACTIVITY_SENTRY:
+  case ACTIVITY_RAILROAD:
+  case ACTIVITY_GOTO:
+  case ACTIVITY_EXPLORE:
+  case ACTIVITY_TRANSFORM:
+  case ACTIVITY_FORTIFYING:
+  case ACTIVITY_FALLOUT:
+    return FALSE;
+  /* These shouldn't be kicking around internally. */
+  case ACTIVITY_FORTRESS:
+  case ACTIVITY_AIRBASE:
+  case ACTIVITY_PATROL_UNUSED:
+  default:
+    fc_assert_ret_val(FALSE, FALSE);
+  }
+}
+
 /**************************************************************************
   Return whether the unit can be put in auto-settler mode.
 
@@ -567,7 +713,7 @@ const char *get_activity_text(enum unit_activity activity)
     break;
   }
 
-  assert(0);
+  fc_assert(FALSE);
   return _("Unknown");
 }
 
@@ -952,7 +1098,7 @@ bool can_unit_do_activity_targeted_at(const struct unit *punit,
             /* Needs to match what unit_activity_assign_target chooses */
             int pre_target = get_preferred_pillage(pspossible, bspossible);
             Base_type_id pre_base = BASE_NONE;
-            assert(pre_target != S_LAST);
+            fc_assert_ret_val(pre_target != S_LAST, FALSE);
             if (pre_target > S_LAST) {
               pre_base = pre_target - S_LAST - 1;
               pre_target = S_LAST;
@@ -998,20 +1144,19 @@ bool can_unit_do_activity_targeted_at(const struct unit *punit,
   case ACTIVITY_UNKNOWN:
     break;
   }
-  freelog(LOG_ERROR,
-	  "can_unit_do_activity_targeted_at() unknown activity %d",
-	  activity);
+  log_error("can_unit_do_activity_targeted_at() unknown activity %d",
+            activity);
   return FALSE;
 }
 
 /**************************************************************************
-  assign a new task to a unit.
+  Assign a new task to a unit. Doesn't account for changed_from.
 **************************************************************************/
-static void set_unit_activity_no_checks(struct unit *punit,
-                                        enum unit_activity new_activity)
+static void set_unit_activity_internal(struct unit *punit,
+                                       enum unit_activity new_activity)
 {
-  assert(new_activity != ACTIVITY_FORTRESS
-         && new_activity != ACTIVITY_AIRBASE);
+  fc_assert_ret(new_activity != ACTIVITY_FORTRESS
+                && new_activity != ACTIVITY_AIRBASE);
 
   punit->activity=new_activity;
   punit->activity_count=0;
@@ -1028,11 +1173,16 @@ static void set_unit_activity_no_checks(struct unit *punit,
 **************************************************************************/
 void set_unit_activity(struct unit *punit, enum unit_activity new_activity)
 {
-  /* Targets must be specified for these activities */
-  assert(new_activity != ACTIVITY_PILLAGE
-         && new_activity != ACTIVITY_BASE);
+  fc_assert_ret(!activity_requires_target(new_activity));
 
-  set_unit_activity_no_checks(punit, new_activity);
+  if (new_activity == ACTIVITY_FORTIFYING
+      && punit->changed_from == ACTIVITY_FORTIFIED) {
+    new_activity = ACTIVITY_FORTIFIED;
+  }
+  set_unit_activity_internal(punit, new_activity);
+  if (new_activity == punit->changed_from) {
+    punit->activity_count = punit->changed_from_count;
+  }
 }
 
 /**************************************************************************
@@ -1043,12 +1193,19 @@ void set_unit_activity_targeted(struct unit *punit,
 				enum tile_special_type new_target,
                                 Base_type_id base)
 {
-  assert(new_target != S_OLD_FORTRESS
-         && new_target != S_OLD_AIRBASE);
+  fc_assert_ret(activity_requires_target(new_activity));
+  fc_assert_ret(new_activity != ACTIVITY_BASE);
+  fc_assert_ret(new_target != S_OLD_FORTRESS
+                && new_target != S_OLD_AIRBASE);
 
-  set_unit_activity_no_checks(punit, new_activity);
+  set_unit_activity_internal(punit, new_activity);
   punit->activity_target = new_target;
   punit->activity_base = base;
+  if (new_activity == punit->changed_from
+      && (new_target == punit->changed_from_target)
+      && (new_target != S_LAST || (base == punit->changed_from_base))) {
+    punit->activity_count = punit->changed_from_count;
+  }
 }
 
 /**************************************************************************
@@ -1057,8 +1214,12 @@ void set_unit_activity_targeted(struct unit *punit,
 void set_unit_activity_base(struct unit *punit,
                             Base_type_id base)
 {
-  set_unit_activity_no_checks(punit, ACTIVITY_BASE);
+  set_unit_activity_internal(punit, ACTIVITY_BASE);
   punit->activity_base = base;
+  if (ACTIVITY_BASE == punit->changed_from
+      && (base == punit->changed_from_base)) {
+    punit->activity_count = punit->changed_from_count;
+  }
 }
 
 /**************************************************************************
@@ -1087,7 +1248,7 @@ bv_special get_unit_tile_pillage_set(const struct tile *ptile)
   unit_list_iterate(ptile->units, punit) {
     if (punit->activity == ACTIVITY_PILLAGE
         && punit->activity_target != S_LAST) {
-      assert(punit->activity_target < S_LAST);
+      fc_assert_action(punit->activity_target < S_LAST, continue);
       BV_SET(tgt_ret, punit->activity_target);
     }
   } unit_list_iterate_end;
@@ -1108,7 +1269,7 @@ bv_bases get_unit_tile_pillage_base_set(const struct tile *ptile)
     if (punit->activity == ACTIVITY_PILLAGE
         && punit->activity_target == S_LAST
         && punit->activity_base != BASE_NONE) {
-      assert(punit->activity_base < base_count());
+      fc_assert(punit->activity_base < base_count());
       BV_SET(tgt_ret, punit->activity_base);
     }
   } unit_list_iterate_end;
@@ -1128,7 +1289,7 @@ const char *unit_activity_text(const struct unit *punit) {
   astr_clear(&str);
   unit_activity_astr(punit, &str);
 
-  return str.str;
+  return astr_str(&str);
 }
 
 /**************************************************************************
@@ -1178,7 +1339,7 @@ void unit_activity_astr(const struct unit *punit, struct astring *astr)
                       punit->moves_left / SINGLE_MOVE);
       }
     }
-    break;
+    return;
   case ACTIVITY_POLLUTION:
   case ACTIVITY_FALLOUT:
   case ACTIVITY_ROAD:
@@ -1194,7 +1355,7 @@ void unit_activity_astr(const struct unit *punit, struct astring *astr)
   case ACTIVITY_GOTO:
   case ACTIVITY_EXPLORE:
     astr_add_line(astr, "%s", get_activity_text(punit->activity));
-    break;
+    return;
   case ACTIVITY_PILLAGE:
     if (punit->activity_target == S_LAST) {
       astr_add_line(astr, "%s", get_activity_text(punit->activity));
@@ -1211,7 +1372,7 @@ void unit_activity_astr(const struct unit *punit, struct astring *astr)
       astr_add_line(astr, "%s: %s", get_activity_text(punit->activity),
                     get_infrastructure_text(pset, bases));
     }
-    break;
+    return;
   case ACTIVITY_BASE:
     {
       struct base_type *pbase;
@@ -1219,11 +1380,15 @@ void unit_activity_astr(const struct unit *punit, struct astring *astr)
       astr_add_line(astr, "%s: %s", get_activity_text(punit->activity),
                     base_name_translation(pbase));
     }
+    return;
+  case ACTIVITY_UNKNOWN:
+  case ACTIVITY_PATROL_UNUSED:
+  case ACTIVITY_LAST:
     break;
-  default:
-    die("Unknown unit activity %d in unit_activity_text()",
-        punit->activity);
   }
+
+  log_error("Unknown unit activity %d for %s (nb %d) in %s()",
+            punit->activity, unit_rule_name(punit), punit->id, __FUNCTION__);
 }
 
 /**************************************************************************
@@ -1248,8 +1413,8 @@ void unit_upkeep_astr(const struct unit *punit, struct astring *astr)
 **************************************************************************/
 struct player *unit_owner(const struct unit *punit)
 {
-  assert(NULL != punit);
-  assert(NULL != punit->owner);
+  fc_assert_ret_val(NULL != punit, NULL);
+  fc_assert(NULL != punit->owner);
   return punit->owner;
 }
 
@@ -1259,7 +1424,7 @@ struct player *unit_owner(const struct unit *punit)
 **************************************************************************/
 struct tile *unit_tile(const struct unit *punit)
 {
-  assert(NULL != punit);
+  fc_assert_ret_val(NULL != punit, NULL);
   return punit->tile;
 }
 
@@ -1423,7 +1588,7 @@ bool unit_being_aggressive(const struct unit *punit)
   if (tile_city(punit->tile)) {
     return FALSE;
   }
-  if (game.info.borders > 0
+  if (BORDERS_DISABLED != game.info.borders
       && game.info.happyborders
       && tile_owner(punit->tile) == unit_owner(punit)) {
     return FALSE;
@@ -1461,21 +1626,23 @@ bool is_build_or_clean_activity(enum unit_activity activity)
 
 /**************************************************************************
   Create a virtual unit skeleton. pcity can be NULL, but then you need
-  to set x, y and homecity yourself.
+  to set tile and homecity yourself.
 **************************************************************************/
 struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
                                  struct unit_type *punittype,
-				 int veteran_level)
+                                 int veteran_level)
 {
+  /* Make sure that contents of unit structure are correctly initialized,
+   * if you ever allocate it by some other mean than fc_calloc() */
   struct unit *punit = fc_calloc(1, sizeof(*punit));
 
   /* It does not register the unit so the id is set to 0. */
   punit->id = IDENTITY_NUMBER_ZERO;
 
-  CHECK_UNIT_TYPE(punittype); /* No untyped units! */
+  fc_assert_ret_val(NULL != punittype, NULL);   /* No untyped units! */
   punit->utype = punittype;
 
-  assert(pplayer != NULL); /* No unowned units! */
+  fc_assert_ret_val(NULL != pplayer, NULL);     /* No unowned units! */
   punit->owner = pplayer;
 
   if (pcity) {
@@ -1485,40 +1652,46 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
     punit->tile = NULL;
     punit->homecity = IDENTITY_NUMBER_ZERO;
   }
+
   memset(punit->upkeep, 0, O_LAST * sizeof(*punit->upkeep));
   punit->goto_tile = NULL;
   punit->veteran = veteran_level;
   /* A unit new and fresh ... */
-  punit->debug = FALSE;
   punit->fuel = utype_fuel(unit_type(punit));
-  punit->birth_turn = game.info.turn;
   punit->hp = unit_type(punit)->hp;
   punit->moves_left = unit_move_rate(punit);
   punit->moved = FALSE;
+
+  punit->ai_controlled = FALSE;
   punit->paradropped = FALSE;
   punit->done_moving = FALSE;
-  punit->ai.done = FALSE;
-  punit->ai.cur_pos = NULL;
-  punit->ai.prev_pos = NULL;
-  punit->ai.target = 0;
-  punit->ai.hunted = 0;
-  punit->ai.control = FALSE;
-  punit->ai.ai_role = AIUNIT_NONE;
-  punit->ai.ferryboat = 0;
-  punit->ai.passenger = 0;
-  punit->ai.bodyguard = 0;
-  punit->ai.charge = 0;
+
   punit->transported_by = -1;
-  punit->focus_status = FOCUS_AVAIL;
-  punit->ord_map = 0;
-  punit->ord_city = 0;
   set_unit_activity(punit, ACTIVITY_IDLE);
   punit->occupy = 0;
   punit->battlegroup = BATTLEGROUP_NONE;
-  punit->client.colored = FALSE;
-  punit->server.vision = NULL; /* No vision. */
   punit->has_orders = FALSE;
 
+  if (is_server()) {
+    punit->server.debug = FALSE;
+    punit->server.birth_turn = game.info.turn;
+
+    punit->server.ord_map = 0;
+    punit->server.ord_city = 0;
+
+    punit->server.vision = NULL; /* No vision. */
+    punit->server.action_timestamp = 0;
+    punit->server.action_turn = 0;
+
+    punit->server.adv = fc_calloc(1, sizeof(*punit->server.adv));
+
+    CALL_FUNC_EACH_AI(unit_alloc, punit);
+    CALL_PLR_AI_FUNC(unit_got, pplayer, punit);
+  } else {
+    punit->client.focus_status = FOCUS_AVAIL;
+    punit->client.colored = FALSE;
+  }
+
   return punit;
 }
 
@@ -1529,8 +1702,15 @@ struct unit *create_unit_virtual(struct player *pplayer, struct city *pcity,
 void destroy_unit_virtual(struct unit *punit)
 {
   free_unit_orders(punit);
-  memset(punit, 0, sizeof(*punit)); /* ensure no pointers remain */
-  free(punit);
+
+  CALL_PLR_AI_FUNC(unit_lost, punit->owner, punit);
+  CALL_FUNC_EACH_AI(unit_free, punit);
+
+  if (is_server() && punit->server.adv) {
+    FC_FREE(punit->server.adv);
+  }
+
+  FC_FREE(punit);
 }
 
 /**************************************************************************
@@ -1566,7 +1746,7 @@ int get_transporter_occupancy(const struct unit *ptrans)
 /****************************************************************************
   Find a transporter at the given location for the unit.
 ****************************************************************************/
-struct unit *find_transporter_for_unit(const struct unit *pcargo)
+struct unit *transporter_for_unit(const struct unit *pcargo)
 {
   struct tile *ptile = pcargo->tile;
 
@@ -1579,17 +1759,17 @@ struct unit *find_transporter_for_unit(const struct unit *pcargo)
   return NULL;
 }
 
-/***************************************************************************
-  Tests if the unit could be updated. Returns UR_OK if is this is
+/****************************************************************************
+  Tests if the unit could be updated. Returns UU_OK if is this is
   possible.
 
   is_free should be set if the unit upgrade is "free" (e.g., Leonardo's).
   Otherwise money is needed and the unit must be in an owned city.
 
   Note that this function is strongly tied to unittools.c:upgrade_unit().
-***************************************************************************/
-enum unit_upgrade_result test_unit_upgrade(const struct unit *punit,
-					   bool is_free)
+****************************************************************************/
+enum unit_upgrade_result unit_upgrade_test(const struct unit *punit,
+                                           bool is_free)
 {
   struct player *pplayer = unit_owner(punit);
   struct unit_type *to_unittype = can_upgrade_unittype(pplayer, unit_type(punit));
@@ -1597,22 +1777,22 @@ enum unit_upgrade_result test_unit_upgrade(const struct unit *punit,
   int cost;
 
   if (!to_unittype) {
-    return UR_NO_UNITTYPE;
+    return UU_NO_UNITTYPE;
   }
 
   if (!is_free) {
     cost = unit_upgrade_price(pplayer, unit_type(punit), to_unittype);
     if (pplayer->economic.gold < cost) {
-      return UR_NO_MONEY;
+      return UU_NO_MONEY;
     }
 
     pcity = tile_city(punit->tile);
     if (!pcity) {
-      return UR_NOT_IN_CITY;
+      return UU_NOT_IN_CITY;
     }
     if (city_owner(pcity) != pplayer) {
       /* TODO: should upgrades in allied cities be possible? */
-      return UR_NOT_CITY_OWNER;
+      return UU_NOT_CITY_OWNER;
     }
   }
 
@@ -1620,59 +1800,88 @@ enum unit_upgrade_result test_unit_upgrade(const struct unit *punit,
     /* TODO: allow transported units to be reassigned.  Check for
      * unit_class_transporter_capacity() here and make changes to
      * upgrade_unit. */
-    return UR_NOT_ENOUGH_ROOM;
+    return UU_NOT_ENOUGH_ROOM;
   }
 
-  return UR_OK;
+  if (!can_exist_at_tile(to_unittype, unit_tile(punit))) {
+    /* The new unit type can't survive on this terrain. */
+    return UU_NOT_TERRAIN;
+  }
+
+  return UU_OK;
+}
+
+/**************************************************************************
+  Tests if unit can be converted to another type.
+**************************************************************************/
+bool unit_can_convert(const struct unit *punit)
+{
+  return unit_type(punit)->converted_to != NULL;
 }
 
 /**************************************************************************
   Find the result of trying to upgrade the unit, and a message that
   most callers can use directly.
 **************************************************************************/
-enum unit_upgrade_result get_unit_upgrade_info(char *buf, size_t bufsz,
-					       const struct unit *punit)
+enum unit_upgrade_result unit_upgrade_info(const struct unit *punit,
+                                           char *buf, size_t bufsz)
 {
   struct player *pplayer = unit_owner(punit);
-  enum unit_upgrade_result result = test_unit_upgrade(punit, FALSE);
+  enum unit_upgrade_result result = unit_upgrade_test(punit, FALSE);
   int upgrade_cost;
   struct unit_type *from_unittype = unit_type(punit);
   struct unit_type *to_unittype = can_upgrade_unittype(pplayer,
-						  unit_type(punit));
+                                                       unit_type(punit));
+  char tbuf[MAX_LEN_MSG];
+
+  fc_snprintf(tbuf, ARRAY_SIZE(tbuf), PL_("Treasury contains %d gold.",
+                                          "Treasury contains %d gold.",
+                                          pplayer->economic.gold),
+              pplayer->economic.gold);
 
   switch (result) {
-  case UR_OK:
+  case UU_OK:
     upgrade_cost = unit_upgrade_price(pplayer, from_unittype, to_unittype);
     /* This message is targeted toward the GUI callers. */
-    my_snprintf(buf, bufsz, _("Upgrade %s to %s for %d gold?\n"
-			      "Treasury contains %d gold."),
-		utype_name_translation(from_unittype),
-		utype_name_translation(to_unittype),
-		upgrade_cost, pplayer->economic.gold);
+    /* TRANS: Last %s is pre-pluralised "Treasury contains %d gold." */
+    fc_snprintf(buf, bufsz, PL_("Upgrade %s to %s for %d gold?\n%s",
+                                "Upgrade %s to %s for %d gold?\n%s",
+                                upgrade_cost),
+                utype_name_translation(from_unittype),
+                utype_name_translation(to_unittype),
+                upgrade_cost, tbuf);
     break;
-  case UR_NO_UNITTYPE:
-    my_snprintf(buf, bufsz,
-		_("Sorry, cannot upgrade %s (yet)."),
-		utype_name_translation(from_unittype));
+  case UU_NO_UNITTYPE:
+    fc_snprintf(buf, bufsz,
+                _("Sorry, cannot upgrade %s (yet)."),
+                utype_name_translation(from_unittype));
     break;
-  case UR_NO_MONEY:
+  case UU_NO_MONEY:
     upgrade_cost = unit_upgrade_price(pplayer, from_unittype, to_unittype);
-    my_snprintf(buf, bufsz,
-		_("Upgrading %s to %s costs %d gold.\n"
-		  "Treasury contains %d gold."),
-		utype_name_translation(from_unittype),
-		utype_name_translation(to_unittype),
-		upgrade_cost, pplayer->economic.gold);
+    /* TRANS: Last %s is pre-pluralised "Treasury contains %d gold." */
+    fc_snprintf(buf, bufsz, PL_("Upgrading %s to %s costs %d gold.\n%s",
+                                "Upgrading %s to %s costs %d gold.\n%s",
+                                upgrade_cost),
+                utype_name_translation(from_unittype),
+                utype_name_translation(to_unittype),
+                upgrade_cost, tbuf);
     break;
-  case UR_NOT_IN_CITY:
-  case UR_NOT_CITY_OWNER:
-    my_snprintf(buf, bufsz,
-		_("You can only upgrade units in your cities."));
+  case UU_NOT_IN_CITY:
+  case UU_NOT_CITY_OWNER:
+    fc_snprintf(buf, bufsz,
+                _("You can only upgrade units in your cities."));
     break;
-  case UR_NOT_ENOUGH_ROOM:
-    my_snprintf(buf, bufsz,
-		_("Upgrading this %s would strand units it transports."),
-		utype_name_translation(from_unittype));
+  case UU_NOT_ENOUGH_ROOM:
+    fc_snprintf(buf, bufsz,
+                _("Upgrading this %s would strand units it transports."),
+                utype_name_translation(from_unittype));
+    break;
+  case UU_NOT_TERRAIN:
+    fc_snprintf(buf, bufsz,
+                _("Upgrading this %s would result in a %s which can not "
+                  "survive at this place."),
+                utype_name_translation(from_unittype),
+                utype_name_translation(to_unittype));
     break;
   }
 
@@ -1705,7 +1914,7 @@ bool unit_type_is_losing_hp(const struct player *pplayer,
 bool unit_alive(int id)
 {
   /* Check if unit exist in game */
-  if (game_find_unit_by_number(id)) {
+  if (game_unit_by_number(id)) {
     return TRUE;
   }
 
@@ -1725,5 +1934,22 @@ bool unit_is_virtual(const struct unit *punit)
     return FALSE;
   }
 
-  return punit != game_find_unit_by_number(punit->id);
+  return punit != game_unit_by_number(punit->id);
+}
+
+/**************************************************************************
+  Return pointer to ai data of given unit and ai type.
+**************************************************************************/
+void *unit_ai_data(const struct unit *punit, const struct ai_type *ai)
+{
+  return punit->server.ais[ai_type_number(ai)];
+}
+
+/**************************************************************************
+  Attach ai data to unit
+**************************************************************************/
+void unit_set_ai_data(struct unit *punit, const struct ai_type *ai,
+                      void *data)
+{
+  punit->server.ais[ai_type_number(ai)] = data;
 }
