@@ -13,8 +13,13 @@
 #ifndef FC__UNIT_H
 #define FC__UNIT_H
 
-#include "fc_types.h"
+/* utility */
+#include "bitvector.h"
+
+/* common */
+#include "ai.h"                 /* FC_AI_LAST */
 #include "base.h"
+#include "fc_types.h"
 #include "terrain.h"		/* enum tile_special_type */
 #include "unittype.h"
 #include "vision.h"
@@ -65,66 +70,50 @@ enum goto_route_type {
   ROUTE_GOTO, ROUTE_PATROL
 };
 
-enum unit_move_result {
-  MR_OK,
-  MR_DEATH,
-  MR_PAUSE,
-  MR_BAD_TYPE_FOR_CITY_TAKE_OVER,
-  MR_NO_WAR, /* Can't move here without declaring war. */
-  MR_PEACE, /* Can't move here because of a peace treaty. */
-  MR_ZOC,
-  MR_BAD_ACTIVITY,
-  MR_BAD_DESTINATION,
-  MR_BAD_MAP_POSITION,
-  MR_DESTINATION_OCCUPIED_BY_NON_ALLIED_CITY,
-  MR_DESTINATION_OCCUPIED_BY_NON_ALLIED_UNIT,
-  MR_NO_TRANSPORTER_CAPACITY,
-  MR_TRIREME,
-};
-
-enum add_build_city_result {
-  AB_BUILD_OK,			/* Unit OK to build city */
-  AB_ADD_OK,			/* Unit OK to add to city */
-  AB_NOT_BUILD_LOC,		/* City is not allowed to be built at
-				   this location */
-  AB_NOT_ADDABLE_UNIT,		/* Unit is not one that can be added
-				   to cities */
-  AB_NOT_BUILD_UNIT,		/* Unit is not one that can build
-				   cities */
-  AB_NO_MOVES_BUILD,		/* Unit does not have moves left to
-				   build a city */
-  AB_NO_MOVES_ADD,		/* Unit does not have moves left to
-				   add to city */
-  AB_NOT_OWNER,			/* Owner of unit is not owner of
-				   city */
-  AB_TOO_BIG,			/* City is too big to be added to */
-  AB_NO_SPACE			/* Adding takes city past limit */
+enum unit_add_build_city_result {
+  UAB_BUILD_OK,         /* Unit OK to build city. */
+  UAB_ADD_OK,           /* Unit OK to add to city. */
+  UAB_BAD_CITY_TERRAIN, /* Equivalent to 'CB_BAD_CITY_TERRAIN'. */
+  UAB_BAD_UNIT_TERRAIN, /* Equivalent to 'CB_BAD_UNIT_TERRAIN'. */
+  UAB_BAD_BORDERS,      /* Equivalent to 'CB_BAD_BORDERS'. */
+  UAB_NO_MIN_DIST,      /* Equivalent to 'CB_NO_MIN_DIST'. */
+  UAB_NOT_ADDABLE_UNIT, /* Unit is not one that can be added to cities. */
+  UAB_NOT_BUILD_UNIT,   /* Unit is not one that can build cities. */
+  UAB_NO_MOVES_BUILD,   /* Unit does not have moves left to build a city. */
+  UAB_NO_MOVES_ADD,     /* Unit does not have moves left to add to city. */
+  UAB_NOT_OWNER,        /* Owner of unit is not owner of city. */
+  UAB_TOO_BIG,          /* City is too big to be added to. */
+  UAB_NO_SPACE          /* Adding takes city past limit. */
 };
 
 enum unit_upgrade_result {
-  UR_OK,
-  UR_NO_UNITTYPE,
-  UR_NO_MONEY,
-  UR_NOT_IN_CITY,
-  UR_NOT_CITY_OWNER,
-  UR_NOT_ENOUGH_ROOM
+  UU_OK,
+  UU_NO_UNITTYPE,
+  UU_NO_MONEY,
+  UU_NOT_IN_CITY,
+  UU_NOT_CITY_OWNER,
+  UU_NOT_ENOUGH_ROOM,
+  UU_NOT_TERRAIN        /* The upgraded unit could not survive. */
 };
-    
-struct unit_ai {
-  bool control; /* 0: not automated    1: automated */
-  enum ai_unit_task ai_role;
-  /* The following are unit ids or special indicator values (<=0) */
-  int ferryboat; /* the ferryboat assigned to us */
-  int passenger; /* the unit assigned to this ferryboat */
-  int bodyguard; /* the unit bodyguarding us */
-  int charge; /* the unit this unit is bodyguarding */
 
-  struct tile *prev_struct, *cur_struct;
-  struct tile **prev_pos, **cur_pos;
+enum unit_airlift_result {
+  /* Codes treated as success: */
+  AR_OK,                /* This will definitely work */
+  AR_OK_SRC_UNKNOWN,    /* Source city's airlift capability is unknown */
+  AR_OK_DST_UNKNOWN,    /* Dest city's airlift capability is unknown */
+  /* Codes treated as failure: */
+  AR_NO_MOVES,          /* Unit has no moves left */
+  AR_WRONG_UNITTYPE,    /* Can't airlift this type of unit */
+  AR_OCCUPIED,          /* Can't airlift units with passengers */
+  AR_NOT_IN_CITY,       /* Unit not in a city */
+  AR_BAD_SRC_CITY,      /* Can't airlift from this src city */
+  AR_BAD_DST_CITY,      /* Can't airlift to this dst city */
+  AR_SRC_NO_FLIGHTS,    /* No flights available from src */
+  AR_DST_NO_FLIGHTS     /* No flights available to dst */
+};
 
-  int target; /* target we hunt */
-  int hunted; /* if a player is hunting us, set by that player */
-  bool done;  /* we are done controlling this unit this turn */
+struct unit_adv {
+  enum ai_unit_task role;
 };
 
 struct unit_order {
@@ -147,11 +136,11 @@ struct unit {
   int hp;
   int veteran;
   int fuel;
-  int birth_turn;
-  struct unit_ai ai;
-  enum unit_activity activity;
+
   struct tile *goto_tile; /* May be NULL. */
 
+  enum unit_activity activity;
+
   /* The amount of work that has been done on the current activity.  This
    * is counted in turns but is multiplied by ACTIVITY_COUNT (which allows
    * fractional values in some cases). */
@@ -159,11 +148,15 @@ struct unit {
 
   enum tile_special_type activity_target;
   Base_type_id           activity_base;
-  enum unit_focus_status focus_status;
-  int ord_map, ord_city;
-  /* ord_map and ord_city are the order index of this unit in tile.units
-     and city.units_supported; they are only used for save/reload */
-  bool debug;
+
+  /* Previous activity, so it can be resumed without loss of progress
+   * if the user changes their mind during a turn. */
+  enum unit_activity changed_from;
+  int changed_from_count;
+  enum tile_special_type changed_from_target;
+  Base_type_id           changed_from_base;
+
+  bool ai_controlled; /* 0: not automated; 1: automated */
   bool moved;
   bool paradropped;
 
@@ -182,24 +175,46 @@ struct unit {
 #define BATTLEGROUP_NONE (-1)
   int battlegroup;
 
-  struct {
-    /* Equivalent to pcity->client.color.  Only for F_CITIES units. */
-    bool colored;
-    int color_index;
-
-    bool asking_city_name;
-  } client;
-  struct {
-    struct vision *vision;
-  } server;
-
   bool has_orders;
   struct {
     int length, index;
-    bool repeat;	/* The path is to be repeated on completion. */
-    bool vigilant;	/* Orders should be cleared if an enemy is met. */
+    bool repeat;   /* The path is to be repeated on completion. */
+    bool vigilant; /* Orders should be cleared if an enemy is met. */
     struct unit_order *list;
   } orders;
+
+  union {
+    struct {
+      /* Only used at the client (the server is omniscient; ./client/). */
+
+      enum unit_focus_status focus_status;
+
+      /* Equivalent to pcity->client.color. Only for F_CITIES units. */
+      bool colored;
+      int color_index;
+
+      bool asking_city_name;
+    } client;
+
+    struct {
+      /* Only used in the server (./ai/ and ./server/). */
+
+      bool debug;
+
+      struct unit_adv *adv;
+      void *ais[FC_AI_LAST];
+      int birth_turn;
+
+      /* ord_map and ord_city are the order index of this unit in tile.units
+       * and city.units_supported; they are only used for save/reload */
+      int ord_map;
+      int ord_city;
+
+      struct vision *vision;
+      time_t action_timestamp;
+      int action_turn;
+    } server;
+  };
 };
 
 bool is_real_activity(enum unit_activity activity);
@@ -228,6 +243,11 @@ bool unit_can_help_build_wonder(const struct unit *punit,
 				const struct city *pcity);
 bool unit_can_help_build_wonder_here(const struct unit *punit);
 bool unit_can_est_trade_route_here(const struct unit *punit);
+enum unit_airlift_result
+    test_unit_can_airlift_to(const struct player *restriction,
+                             const struct unit *punit,
+                             const struct city *pdest_city);
+bool is_successful_airlift_result(enum unit_airlift_result result);
 bool unit_can_airlift_to(const struct unit *punit, const struct city *pcity);
 bool unit_has_orders(const struct unit *punit);
 
@@ -266,6 +286,7 @@ int get_activity_rate_this_turn(const struct unit *punit);
 int get_turns_for_activity_at(const struct unit *punit,
 			      enum unit_activity activity,
 			      const struct tile *ptile);
+bool activity_requires_target(enum unit_activity activity);
 bool can_unit_do_autosettlers(const struct unit *punit); 
 bool is_unit_activity_on_tile(enum unit_activity activity,
 			      const struct tile *ptile);
@@ -278,14 +299,11 @@ bool is_square_threatened(const struct player *pplayer,
 			  const struct tile *ptile);
 bool is_field_unit(const struct unit *punit);              /* ships+aero */
 bool is_hiding_unit(const struct unit *punit);
-#define COULD_OCCUPY(punit) \
-  (uclass_has_flag(unit_class(punit), UCF_CAN_OCCUPY_CITY) \
-   && is_military_unit(punit))
-bool can_unit_add_to_city (const struct unit *punit);
-bool can_unit_build_city (const struct unit *punit);
-bool can_unit_add_or_build_city (const struct unit *punit);
-enum add_build_city_result test_unit_add_or_build_city(const struct unit *
-						       punit);
+bool unit_can_add_to_city(const struct unit *punit);
+bool unit_can_build_city(const struct unit *punit);
+bool unit_can_add_or_build_city(const struct unit *punit);
+enum unit_add_build_city_result
+unit_add_or_build_city_test(const struct unit *punit);
 bool kills_citizen_after_attack(const struct unit *punit);
 
 struct astring; /* Forward declaration. */
@@ -323,16 +341,22 @@ bool unit_is_virtual(const struct unit *punit);
 void free_unit_orders(struct unit *punit);
 
 int get_transporter_occupancy(const struct unit *ptrans);
-struct unit *find_transporter_for_unit(const struct unit *pcargo);
+struct unit *transporter_for_unit(const struct unit *pcargo);
+
+enum unit_upgrade_result unit_upgrade_test(const struct unit *punit,
+                                           bool is_free);
+enum unit_upgrade_result unit_upgrade_info(const struct unit *punit,
+                                           char *buf, size_t bufsz);
+bool unit_can_convert(const struct unit *punit);
 
-enum unit_upgrade_result test_unit_upgrade(const struct unit *punit,
-					   bool is_free);
-enum unit_upgrade_result get_unit_upgrade_info(char *buf, size_t bufsz,
-					       const struct unit *punit);
 bool is_losing_hp(const struct unit *punit);
 bool unit_type_is_losing_hp(const struct player *pplayer,
                             const struct unit_type *punittype);
 
 bool unit_alive(int id);
 
+void *unit_ai_data(const struct unit *punit, const struct ai_type *ai);
+void unit_set_ai_data(struct unit *punit, const struct ai_type *ai,
+                      void *data);
+
 #endif  /* FC__UNIT_H */
