@@ -15,8 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
 /* utility */
 #include "log.h"
 #include "mem.h"
@@ -50,8 +48,8 @@ struct vision *vision_new(struct player *pplayer, struct tile *ptile)
 ****************************************************************************/
 void vision_free(struct vision *vision)
 {
-  assert(-1 == vision->radius_sq[V_MAIN]);
-  assert(-1 == vision->radius_sq[V_INVIS]);
+  fc_assert(-1 == vision->radius_sq[V_MAIN]);
+  fc_assert(-1 == vision->radius_sq[V_INVIS]);
   free(vision);
 }
 
