@@ -43,11 +43,8 @@
 #include "terrain.h"
 #include "unitlist.h"
 
-/* ai */
-#include "aidata.h"
-#include "aitools.h"
-
 /* server */
+#include "citytools.h"
 #include "gamehand.h"
 #include "maphand.h"
 #include "notify.h"
@@ -58,6 +55,9 @@
 #include "unithand.h"
 #include "unittools.h"
 
+/* server/advisors */
+#include "advdata.h"
+
 #include "barbarian.h"
 
 #define BARBARIAN_INITIAL_VISION_RADIUS 3
@@ -68,7 +68,7 @@
 **************************************************************************/
 bool is_land_barbarian(struct player *pplayer)
 {
-  return (pplayer->ai_data.barbarian_type == LAND_BARBARIAN);
+  return (pplayer->ai_common.barbarian_type == LAND_BARBARIAN);
 }
 
 /**************************************************************************
@@ -76,7 +76,7 @@ bool is_land_barbarian(struct player *pplayer)
 **************************************************************************/
 bool is_sea_barbarian(struct player *pplayer)
 {
-  return (pplayer->ai_data.barbarian_type == SEA_BARBARIAN);
+  return (pplayer->ai_common.barbarian_type == SEA_BARBARIAN);
 }
 
 /**************************************************************************
@@ -97,9 +97,10 @@ static struct player *create_barbarian_player(enum barbarian_type type)
       if (!barbarians->is_alive) {
         barbarians->economic.gold = 0;
         barbarians->is_alive = TRUE;
-        barbarians->is_dying = FALSE;
-        pick_random_player_name(nation_of_player(barbarians), barbarians->name);
-	sz_strlcpy(barbarians->username, ANON_USER_NAME);
+        player_status_reset(barbarians);
+        sz_strlcpy(barbarians->name,
+                   pick_random_player_name(nation_of_player(barbarians)));
+        sz_strlcpy(barbarians->username, ANON_USER_NAME);
         /* I need to make them to forget the map, I think */
 	whole_map_iterate(ptile) {
 	  map_clear_known(ptile, barbarians);
@@ -111,7 +112,7 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   } players_iterate_end;
 
   /* make a new player, or not */
-  barbarians = server_create_player();
+  barbarians = server_create_player(-1);
   if (!barbarians) {
     return NULL;
   }
@@ -120,23 +121,23 @@ static struct player *create_barbarian_player(enum barbarian_type type)
 
   nation = pick_a_nation(NULL, FALSE, TRUE, type);
   player_set_nation(barbarians, nation);
-  pick_random_player_name(nation, barbarians->name);
+  sz_strlcpy(barbarians->name, pick_random_player_name(nation));
 
   server.nbarbarians++;
-  game.info.max_players = MAX(player_count(), game.info.max_players);
+  game.server.max_players = MAX(player_count(), game.server.max_players);
 
   sz_strlcpy(barbarians->username, ANON_USER_NAME);
   barbarians->is_connected = FALSE;
-  barbarians->government = nation->init_government;
-  assert(barbarians->revolution_finishes < 0);
-  barbarians->capital = FALSE;
+  barbarians->government = nation->server.init_government;
+  fc_assert(barbarians->revolution_finishes < 0);
+  barbarians->server.capital = FALSE;
   barbarians->economic.gold = 100;
 
   barbarians->phase_done = TRUE;
 
   /* Do the ai */
-  barbarians->ai_data.control = TRUE;
-  barbarians->ai_data.barbarian_type = type;
+  barbarians->ai_controlled = TRUE;
+  barbarians->ai_common.barbarian_type = type;
   set_ai_level_directer(barbarians, game.info.skill_level);
   init_tech(barbarians, TRUE);
   give_global_initial_techs(barbarians);
@@ -145,21 +146,20 @@ static struct player *create_barbarian_player(enum barbarian_type type)
   /* Ensure that we are at war with everyone else */
   players_iterate(pplayer) {
     if (pplayer != barbarians) {
-      pplayer->diplstates[player_index(barbarians)].type = DS_WAR;
-      barbarians->diplstates[player_index(pplayer)].type = DS_WAR;
+      player_diplstate_get(pplayer, barbarians)->type = DS_WAR;
+      player_diplstate_get(barbarians, pplayer)->type = DS_WAR;
     }
   } players_iterate_end;
 
-  freelog(LOG_VERBOSE, "Created barbarian %s, player %d",
-          player_name(barbarians),
-          player_number(barbarians));
+  log_verbose("Created barbarian %s, player %d", player_name(barbarians),
+              player_number(barbarians));
   notify_player(NULL, NULL, E_UPRISING, ftc_server,
                 _("%s gain a leader by the name %s. Dangerous "
                   "times may lie ahead."),
                 nation_plural_for_player(barbarians),
                 player_name(barbarians));
 
-  send_player_info(barbarians, NULL);
+  send_player_all_c(barbarians, NULL);
 
   return barbarians;
 }
@@ -208,12 +208,12 @@ static int random_unchecked_direction(int possibilities, const bool *checked)
   int j = -1;
   int i;
 
-  int num = myrand(possibilities);
+  int num = fc_rand(possibilities);
   for (i = 0; i <= num; i++) {
     j++;
     while (checked[j]) {
       j++;
-      assert(j < 8);
+      fc_assert(j < 8);
     }
   }
 
@@ -249,8 +249,8 @@ bool unleash_barbarians(struct tile *ptile)
    *        but L_BARBARIAN_TECH is already available,
    *        we should unleash those.
    *        Doesn't affect any ruleset I'm aware of. */
-  if (game.info.barbarianrate == 0
-      || game.info.turn < game.info.onsetbarbarian
+  if (BARBS_DISABLED == game.server.barbarianrate
+      || game.info.turn < game.server.onsetbarbarian
       || num_role_units(L_BARBARIAN) == 0) {
     unit_list_iterate_safe((ptile)->units, punit) {
       wipe_unit(punit);
@@ -263,7 +263,9 @@ bool unleash_barbarians(struct tile *ptile)
     return FALSE;
   }
 
-  unit_cnt = 3 + myrand(4);
+  ai_data_phase_init(barbarians, TRUE);
+
+  unit_cnt = 3 + fc_rand(4);
   for (i = 0; i < unit_cnt; i++) {
     struct unit_type *punittype
       = find_a_unit_type(L_BARBARIAN, L_BARBARIAN_TECH);
@@ -275,8 +277,7 @@ bool unleash_barbarians(struct tile *ptile)
       struct unit *barb_unit;
 
       barb_unit = create_unit(barbarians, ptile, punittype, 0, 0, -1);
-      freelog(LOG_DEBUG, "Created barbarian unit %s",
-              utype_rule_name(punittype));
+      log_debug("Created barbarian unit %s", utype_rule_name(punittype));
       send_unit_info(NULL, barb_unit);
     }
   }
@@ -286,7 +287,7 @@ bool unleash_barbarians(struct tile *ptile)
   for (dir = 0; dir < 8; dir++) {
     dir_tiles[dir] = mapstep(ptile, dir);
     if (dir_tiles[dir] == NULL) {
-      terrainc[dir] = TC_LAST;
+      terrainc[dir] = terrain_class_invalid();
     } else if (is_free_land(dir_tiles[dir], barbarians)) {
       terrainc[dir] = TC_LAND;
       land_tiles++;
@@ -294,7 +295,7 @@ bool unleash_barbarians(struct tile *ptile)
       terrainc[dir] = TC_OCEAN;
       ocean_tiles++;
     } else {
-      terrainc[dir] = TC_LAST;
+      terrainc[dir] = terrain_class_invalid();
     }
   }
 
@@ -312,10 +313,10 @@ bool unleash_barbarians(struct tile *ptile)
              checked_count++) {
           int rdir = random_unchecked_direction(land_tiles - checked_count, checked);
 
-          if (can_unit_move_to_tile(punit2, dir_tiles[rdir], TRUE)) {
+          if (unit_can_move_to_tile(punit2, dir_tiles[rdir], TRUE)) {
             (void) unit_move_handling(punit2, dir_tiles[rdir], TRUE, FALSE);
-            freelog(LOG_DEBUG, "Moved barbarian unit from %d %d to %d, %d", 
-                    ptile->x, ptile->y, dir_tiles[rdir]->x, dir_tiles[rdir]->y);
+            log_debug("Moved barbarian unit from (%d, %d) to (%d, %d)", 
+                      TILE_XY(ptile), TILE_XY(dir_tiles[rdir]));
             dest_found = TRUE;
           }
 
@@ -355,7 +356,7 @@ bool unleash_barbarians(struct tile *ptile)
         /* We do have a boat. Try to get everybody in */
         unit_list_iterate_safe((ptile)->units, punit2) {
           if (unit_owner(punit2) == barbarians) {
-            if (can_unit_move_to_tile(punit2, btile, TRUE)) {
+            if (unit_can_move_to_tile(punit2, btile, TRUE)) {
               (void) unit_move_handling(punit2, btile, TRUE, FALSE);
             }
           }
@@ -376,7 +377,7 @@ bool unleash_barbarians(struct tile *ptile)
             int rdir;
             rdir = random_unchecked_direction(land_tiles - checked_count, checked);
 
-            if (can_unit_move_to_tile(punit2, dir_tiles[rdir], TRUE)) {
+            if (unit_can_move_to_tile(punit2, dir_tiles[rdir], TRUE)) {
               (void) unit_move_handling(punit2, dir_tiles[rdir], TRUE, FALSE);
               dest_found = TRUE;
             }
@@ -482,8 +483,8 @@ static void try_summon_barbarians(void)
     return;
   }
 
-
-  if (!(pc = dist_nearest_city(NULL, ptile, TRUE, FALSE))) {
+  if (!(pc = find_closest_city(ptile, NULL, NULL, FALSE, FALSE, FALSE, FALSE,
+                               FALSE))) {
     /* any city */
     return;
   }
@@ -491,8 +492,8 @@ static void try_summon_barbarians(void)
   victim = city_owner(pc);
 
   dist = real_map_distance(ptile, pc->tile);
-  freelog(LOG_DEBUG,"Closest city (to %d,%d) is %s (at %d,%d) distance %d.", 
-          TILE_XY(ptile), city_name(pc), TILE_XY(pc->tile), dist);
+  log_debug("Closest city (to %d,%d) is %s (at %d,%d) distance %d.",
+            TILE_XY(ptile), city_name(pc), TILE_XY(pc->tile), dist);
   if (dist > MAX_UNREST_DIST || dist < MIN_UNREST_DIST) {
     return;
   }
@@ -505,14 +506,16 @@ static void try_summon_barbarians(void)
     return;
   }
 
+  fc_assert(1 < game.server.barbarianrate);
+
   /* do not harass small civs - in practice: do not uprise at the beginning */
-  if ((int)myrand(UPRISE_CIV_MORE) >
+  if ((int)fc_rand(UPRISE_CIV_MORE) >
            (int)city_list_size(victim->cities) -
-                UPRISE_CIV_SIZE/(game.info.barbarianrate-1)
-      || myrand(100) > get_player_bonus(victim, EFT_CIVIL_WAR_CHANCE)) {
+                UPRISE_CIV_SIZE/(game.server.barbarianrate-1)
+      || fc_rand(100) > get_player_bonus(victim, EFT_CIVIL_WAR_CHANCE)) {
     return;
   }
-  freelog(LOG_DEBUG, "Barbarians are willing to fight");
+  log_debug("Barbarians are willing to fight");
 
   if (tile_has_special(utile, S_HUT)) {
     /* remove the hut in place of uprising */
@@ -520,7 +523,7 @@ static void try_summon_barbarians(void)
     update_tile_knowledge(utile);
   }
 
-  barb_count = myrand(3) + uprise * game.info.barbarianrate;
+  barb_count = fc_rand(3) + uprise * game.server.barbarianrate;
   leader_type = get_role_unit(L_BARBARIAN_LEADER, 0);
 
   if (!is_ocean_tile(utile)) {
@@ -542,8 +545,7 @@ static void try_summon_barbarians(void)
       if (is_native_tile(punittype, utile)) {
         (void) create_unit(barbarians, utile, punittype, 0, 0, -1);
         really_created++;
-        freelog(LOG_DEBUG, "Created barbarian unit %s",
-                utype_rule_name(punittype));
+        log_debug("Created barbarian unit %s",utype_rule_name(punittype));
       }
     }
  
@@ -555,12 +557,23 @@ static void try_summon_barbarians(void)
   } else {                   /* sea raiders - their units will be veteran */
     struct unit *ptrans;
     struct unit_type *boat;
+    bool miniphase;
 
     barbarians = create_barbarian_player(SEA_BARBARIAN);
     if (!barbarians) {
       return;
     }
-    ai_data_phase_init(barbarians, TRUE); /* Created ferry may need ai data */
+
+    /* Setup data phase if it's not already set up. Created ferries may
+       need that data.
+       We don't know if create_barbarian_player() above created completely
+       new player or did it just return existing one. If it was existing
+       one, phase has already been set up at turn begin and will be closed
+       at turn end. If this is completely new player, we have to take care
+       of both opening and closing the data phase. Return value of
+       ai_data_phase_init() tells us if data phase was already initialized
+       at turn beginning. */
+    miniphase = ai_data_phase_init(barbarians, TRUE);
 
     boat = find_a_unit_type(L_BARBARIAN_BOAT,-1);
 
@@ -581,8 +594,7 @@ static void try_summon_barbarians(void)
           (void) create_unit_full(barbarians, utile, barb, 0, 0, -1, -1,
                                   ptrans);
           really_created++;
-          freelog(LOG_DEBUG, "Created barbarian unit %s",
-                  utype_rule_name(barb));
+          log_debug("Created barbarian unit %s", utype_rule_name(barb));
         }
       }
 
@@ -593,7 +605,9 @@ static void try_summon_barbarians(void)
       }
     }
 
-    ai_data_phase_done(barbarians);
+    if (miniphase) {
+      ai_data_phase_done(barbarians);
+    }
   }
 
   if (really_created == 0) {
@@ -631,11 +645,12 @@ void summon_barbarians(void)
 {
   int i, n;
 
-  if (game.info.barbarianrate == 0) {
+  if (BARBS_DISABLED == game.server.barbarianrate
+      || BARBS_HUTS_ONLY == game.server.barbarianrate) {
     return;
   }
 
-  if (game.info.turn < game.info.onsetbarbarian) {
+  if (game.info.turn < game.server.onsetbarbarian) {
     return;
   }
 
@@ -645,7 +660,7 @@ void summon_barbarians(void)
     n = 1;
   }
 
-  for (i = 0; i < n * (game.info.barbarianrate - 1); i++) {
+  for (i = 0; i < n * (game.server.barbarianrate - 1); i++) {
     try_summon_barbarians();
   }
 }
