@@ -14,7 +14,7 @@
 #ifndef FC__BARBARIAN_H
 #define FC__BARBARIAN_H
 
-#include "shared.h"		/* bool type */
+#include "support.h"            /* bool type */
 
 #include "fc_types.h"
 
@@ -27,9 +27,11 @@
 
 #define MAP_FACTOR     2000  /* adjust this to get a good uprising frequency */
 
-struct nation_type *pick_barbarian_nation(void);
+#define BARBARIAN_MIN_LIFESPAN 5
+
 bool unleash_barbarians(struct tile *ptile);
 void summon_barbarians(void);
 bool is_land_barbarian(struct player *pplayer);
+bool is_sea_barbarian(struct player *pplayer);
 
 #endif  /* FC__BARBARIAN_H */
