@@ -262,22 +262,26 @@ static struct command commands[] = {
    /* TRANS: translate text between [] and <> only */
    N_("take [connection-name] <player-name>"),
    N_("Take over a player's place in the game."),
+   /* TRANS: Don't translate text between '' */
    N_("Only the console and connections with cmdlevel 'hack' can force "
       "other connections to take over a player. If you're not one of these, "
       "only the <player-name> argument is allowed. If '-' is given for the "
       "player name and the connection does not already control a player, one "
-      "is created and assigned to the connection."),
+      "is created and assigned to the connection. The 'allowtake' option "
+      "controls which players may be taken and in what circumstances."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
   {"observe",    ALLOW_INFO,
    /* TRANS: translate text between [] only */
    N_("observe [connection-name] [player-name]"),
    N_("Observe a player or the whole game."),
+   /* TRANS: Don't translate text between '' */
    N_("Only the console and connections with cmdlevel 'hack' can force "
       "other connections to observe a player. If you're not one of these, "
       "only the [player-name] argument is allowed. If the console gives no "
       "player-name or the connection uses no arguments, then the connection "
-      "is attached to a global observer."),
+      "is attached to a global observer. The 'allowtake' option controls "
+      "which players may be observed and in what circumstances."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
   {"detach",    ALLOW_INFO,
@@ -433,10 +437,10 @@ static struct command commands[] = {
    N_("ignore [type=]<pattern>"),
    N_("Block all messages from users matching the pattern."),
    N_("The given pattern will be added to your ignore list; you will not "
-      "receive any messages from this users matching this pattern. The type "
+      "receive any messages from users matching this pattern. The type "
       "may be either \"user\", \"host\", or \"ip\". The default type "
-      "(if ommited) is to match against the username. The pattern supports "
-      "unix glob style wildcards, i.e. * matches zero or more character, ? "
+      "(if omitted) is to match against the username. The pattern supports "
+      "unix glob style wildcards, i.e., * matches zero or more character, ? "
       "exactly one character, [abc] exactly one of 'a' 'b' or 'c', etc. "
       "To access your current ignore list, issue \"/list ignore\"."),
    CMD_ECHO_NONE, VCF_NONE, 0
@@ -448,8 +452,8 @@ static struct command commands[] = {
    N_("The ignore list entries in the given range will be removed; "
       "you will be able to receive messages from the respective users. "
       "The range argument may be a single number or a pair of numbers "
-      "separated by a dash '-'. If the first number is ommitted, it is "
-      "assumed to be 1, if the last is ommitted, it is assumed to be "
+      "separated by a dash '-'. If the first number is omitted, it is "
+      "assumed to be 1; if the last is omitted, it is assumed to be "
       "the last valid ignore list index. To access your current ignore "
       "list, issue \"/list ignore\"."),
    CMD_ECHO_NONE, VCF_NONE, 0
