@@ -15,10 +15,11 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
+/* utility */
+#include "bitvector.h"
 #include "log.h"
 
+/* common */
 #include "city.h"
 #include "game.h"
 #include "government.h"
@@ -30,64 +31,49 @@
 #include "unit.h"
 #include "unitlist.h"
 
+/* server */
 #include "citytools.h"
-#include "cityturn.h"		/* city_repair_size() */
+#include "cityturn.h"           /* city_repair_size() */
 #include "maphand.h"
 #include "srv_main.h"
 #include "unittools.h"
 
 #include "sanitycheck.h"
 
+
 #ifdef SANITY_CHECKING
 
-#define SANITY_(_s)							\
-      freelog(LOG_ERROR, "Failed %s:%d at %s:%d! " _s,			\
-	      __FILE__,__LINE__, file, line
+#define SANITY_(format, ...) \
+  fc_assert_fail(file, function, line, NULL, format, ## __VA_ARGS__)
 
-#define SANITY_CHECK(check)						\
-  do {									\
-    if (!(check)) {							\
-      SANITY_("!(%s)"),							\
-      #check);								\
-    }									\
-  } while(0)
+#define SANITY_CHECK(check) \
+  fc_assert_full(file, function, line, check, , NULL)
 
-#define SANITY_CITY(_city, check)					\
-  do {									\
-    if (!(check)) {							\
-      SANITY_("(%4d,%4d) !(%s) in \"%s\"[%d]"),				\
-	      TILE_XY((_city)->tile),					\
-	      #check,							\
-	      city_name(_city),						\
-	      (_city)->size);						\
-    }									\
-  } while(0)
+#define SANITY_CITY(_city, check)                                           \
+  fc_assert_full(file, function, line, check, ,                             \
+                 "(%4d, %4d) in \"%s\"[%d]", TILE_XY((_city)->tile),        \
+                 city_name(_city), (_city)->size)
 
-#define SANITY_TERRAIN(_tile, check)					\
-  do {									\
-    if (!(check)) {							\
-      SANITY_("(%4d,%4d) !(%s) at \"%s\""),				\
-	      TILE_XY(_tile),						\
-	      #check,							\
-	      terrain_rule_name(tile_terrain(_tile)));			\
-    }									\
-  } while(0)
+#define SANITY_TERRAIN(_tile, check)                                        \
+  fc_assert_full(file, function, line, check, ,                             \
+                 "(%4d, %4d) at \"%s\"", TILE_XY(_tile),                    \
+                 terrain_rule_name(tile_terrain(_tile)))
 
-#define SANITY_TILE(_tile, check)					\
-  do {									\
-    struct city *_tile##_city = tile_city(_tile);			\
-    if (NULL != _tile##_city) {						\
-      SANITY_CITY(_tile##_city, check);					\
-    } else {								\
-      SANITY_TERRAIN(_tile, check);					\
-    }									\
+#define SANITY_TILE(_tile, check)                                           \
+  do {                                                                      \
+    struct city *_tile##_city = tile_city(_tile);                           \
+    if (NULL != _tile##_city) {                                             \
+      SANITY_CITY(_tile##_city, check);                                     \
+    } else {                                                                \
+      SANITY_TERRAIN(_tile, check);                                         \
+    }                                                                       \
   } while(0)
 
 
 /**************************************************************************
   Sanity checking on map (tile) specials.
 **************************************************************************/
-static void check_specials(const char *file, int line)
+static void check_specials(const char *file, const char *function, int line)
 {
   whole_map_iterate(ptile) {
     const struct terrain *pterrain = tile_terrain(ptile);
@@ -113,29 +99,22 @@ static void check_specials(const char *file, int line)
 /**************************************************************************
   Sanity checking on fog-of-war (visibility, shared vision, etc.).
 **************************************************************************/
-static void check_fow(const char *file, int line)
+static void check_fow(const char *file, const char *function, int line)
 {
+  if (!game_was_started()) {
+    /* The private map of the players is only allocated at game start. */
+    return;
+  }
+
   whole_map_iterate(ptile) {
     players_iterate(pplayer) {
       struct player_tile *plr_tile = map_get_player_tile(ptile, pplayer);
 
       vision_layer_iterate(v) {
-        /* Underflow of unsigned int. */
+        /* underflow of unsigned int */
         SANITY_TILE(ptile, plr_tile->seen_count[v] < 30000);
         SANITY_TILE(ptile, plr_tile->own_seen[v] < 30000);
-
-	/* FIXME: It's not entirely clear that this is correct.  The
-	 * invariants for when fogofwar is turned off are not clearly
-	 * defined. */
-	if (game.info.fogofwar) {
-	  if (plr_tile->seen_count[v] > 0) {
-	    SANITY_TILE(ptile, BV_ISSET(ptile->tile_seen[v], player_index(pplayer)));
-	  } else {
-	    SANITY_TILE(ptile, !BV_ISSET(ptile->tile_seen[v], player_index(pplayer)));
-	  }
-	}
-
-	SANITY_TILE(ptile, plr_tile->own_seen[v] <= plr_tile->seen_count[v]);
+        SANITY_TILE(ptile, plr_tile->own_seen[v] <= plr_tile->seen_count[v]);
       } vision_layer_iterate_end;
 
       /* Lots of server bits depend on this. */
@@ -154,24 +133,31 @@ static void check_fow(const char *file, int line)
 /**************************************************************************
   Miscellaneous sanity checks.
 **************************************************************************/
-static void check_misc(const char *file, int line)
+static void check_misc(const char *file, const char *function, int line)
 {
-  int nbarbs = 0;
-  players_iterate(pplayer) {
-    if (is_barbarian(pplayer)) {
-      nbarbs++;
+  int nplayers = 0, nbarbs = 0;
+
+  /* Do not use player_slots_iterate as we want to check the index! */
+  player_slots_iterate(pslot) {
+    if (player_slot_is_used(pslot)) {
+      if (is_barbarian(player_slot_get_player(pslot))) {
+        nbarbs++;
+      }
+      nplayers++;
     }
-  } players_iterate_end;
+  } player_slots_iterate_end;
+
+  SANITY_CHECK(nplayers == player_count());
   SANITY_CHECK(nbarbs == server.nbarbarians);
 
-  SANITY_CHECK(player_count() <= MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS);
-  SANITY_CHECK(team_count() <= MAX_NUM_TEAMS);
+  SANITY_CHECK(player_count() <= player_slot_count());
+  SANITY_CHECK(team_count() <= MAX_NUM_TEAM_SLOTS);
 }
 
 /**************************************************************************
   Sanity checks on the map itself.  See also check_specials.
 **************************************************************************/
-static void check_map(const char *file, int line)
+static void check_map(const char *file, const char *function, int line)
 {
   whole_map_iterate(ptile) {
     struct city *pcity = tile_city(ptile);
@@ -183,7 +169,7 @@ static void check_map(const char *file, int line)
 
     if (NULL != pcity) {
       SANITY_TILE(ptile, same_pos(pcity->tile, ptile));
-      if (0 < game.info.borders) {
+      if (BORDERS_DISABLED != game.info.borders) {
         SANITY_TILE(ptile, tile_owner(ptile) != NULL);
       }
     }
@@ -229,7 +215,8 @@ static void check_map(const char *file, int line)
 /**************************************************************************
   Verify that the city itself has sane values.
 **************************************************************************/
-static bool check_city_good(struct city *pcity, const char *file, int line)
+static bool check_city_good(struct city *pcity, const char *file,
+                            const char *function, int line)
 {
   struct player *pplayer = city_owner(pcity);
   struct tile *pcenter = city_tile(pcity);
@@ -237,7 +224,7 @@ static bool check_city_good(struct city *pcity, const char *file, int line)
   if (NULL == pcenter) {
     /* Editor! */
     SANITY_("(----,----) city has no tile (skipping remaining tests), "
-            "at %s \"%s\"[%d]%s"),
+            "at %s \"%s\"[%d]%s",
             nation_rule_name(nation_of_player(pplayer)),
             city_name(pcity), pcity->size,
             "{city center}");
@@ -246,14 +233,14 @@ static bool check_city_good(struct city *pcity, const char *file, int line)
 
   SANITY_CITY(pcity, !terrain_has_flag(tile_terrain(pcenter), TER_NO_CITIES));
 
-  if (0 < game.info.borders) {
+  if (BORDERS_DISABLED != game.info.borders) {
     SANITY_CITY(pcity, NULL != tile_owner(pcenter));
   }
 
   if (NULL != tile_owner(pcenter)) {
     if (tile_owner(pcenter) != pplayer) {
       SANITY_("(%4d,%4d) tile owned by %s, "
-              "at %s \"%s\"[%d]%s"),
+              "at %s \"%s\"[%d]%s",
               TILE_XY(pcenter),
               nation_rule_name(nation_of_player(tile_owner(pcenter))),
               nation_rule_name(nation_of_player(pplayer)),
@@ -269,137 +256,20 @@ static bool check_city_good(struct city *pcity, const char *file, int line)
 
   city_built_iterate(pcity, pimprove) {
     if (is_small_wonder(pimprove)) {
-      SANITY_CITY(pcity, find_city_from_small_wonder(pplayer, pimprove) == pcity);
+      SANITY_CITY(pcity, city_from_small_wonder(pplayer, pimprove) == pcity);
     } else if (is_great_wonder(pimprove)) {
-      SANITY_CITY(pcity, find_city_from_great_wonder(pimprove) == pcity);
+      SANITY_CITY(pcity, city_from_great_wonder(pimprove) == pcity);
     }
   } city_built_iterate_end;
 
   return TRUE;
 }
 
-/**************************************************************************
-  Verify that the city_map has sane values.
-**************************************************************************/
-static void check_city_map(struct city *pcity, const char *file, int line)
-{
-  struct player *pplayer = city_owner(pcity);
-  struct tile *pcenter = city_tile(pcity);
-
-  /* not using city_tile_iterate_cxy() as that skips out of range values */
-  city_map_iterate(x, y) {
-    struct tile *ptile = city_map_to_tile(pcenter, x, y);
-
-    if (NULL == ptile) {
-      SANITY_CITY(pcity, city_map_status(pcity, x, y) == C_TILE_UNUSABLE);
-    } else {
-      struct player *owner = tile_owner(ptile);
-      struct city *pwork = tile_worked(ptile);
-
-      /* bypass city_map_status() check is_valid_city_coords() */
-      switch (pcity->city_map[x][y]) {
-      case C_TILE_EMPTY:
-	if (NULL != pwork) {
-	  SANITY_("(%4d,%4d) marked as empty, "
-		  "but worked by \"%s\"[%d]! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pwork), pwork->size,
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	if (unit_occupies_tile(ptile, pplayer)) {
-	  SANITY_("(%4d,%4d) marked as empty, "
-		  "but occupied by an enemy unit! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	if (game.info.borders > 0 && owner && owner != city_owner(pcity)) {
-	  SANITY_("(%4d,%4d) marked as empty, "
-		  "but in enemy territory! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	if (!city_can_work_tile(pcity, ptile)) {
-	  /* Complete check. */
-	  SANITY_("(%4d,%4d) marked as empty, "
-		  "but is unavailable! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	break;
-
-      case C_TILE_WORKER:
-	if (pwork != pcity) {
-	  SANITY_("(%4d,%4d) marked as worked, "
-		  "but main map disagrees! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	if (unit_occupies_tile(ptile, pplayer)) {
-	  SANITY_("(%4d,%4d) marked as worked, "
-		  "but occupied by an enemy unit! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-        if (game.info.borders > 0 && owner && owner != city_owner(pcity)) {
-	  SANITY_("(%4d,%4d) marked as worked, "
-		  "but in enemy territory! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	if (!city_can_work_tile(pcity, ptile)) {
-	  /* Complete check. */
-	  SANITY_("(%4d,%4d) marked as worked, "
-		  "but is unavailable! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	break;
-
-      case C_TILE_UNAVAILABLE:
-	if (city_can_work_tile(pcity, ptile)) {
-	  SANITY_("(%4d,%4d) marked as unavailable, "
-		  "but seems to be available! "
-		  "\"%s\"[%d]%s"),
-		  TILE_XY(ptile),
-		  city_name(pcity), pcity->size,
-		  is_city_center(pcity, ptile) ? "{city center}" : "");
-	}
-	break;
-
-      case C_TILE_UNUSABLE:
-      default:
-	SANITY_("(%4d,%4d) marked as unusable (%d)? "
-		"\"%s\"[%d]%s"),
-		pcity->city_map[x][y],
-		TILE_XY(ptile),
-		city_name(pcity), pcity->size,
-		is_city_center(pcity, ptile) ? "{city center}" : "");
-        break;
-      };
-    }
-  } city_map_iterate_end;
-}
-
 /**************************************************************************
   Sanity check city size versus worker and specialist counts.
 **************************************************************************/
-static void check_city_size(struct city *pcity, const char *file, int line)
+static void check_city_size(struct city *pcity, const char *file,
+                            const char *function, int line)
 {
   int delta;
   int citizens = 0;
@@ -407,23 +277,28 @@ static void check_city_size(struct city *pcity, const char *file, int line)
 
   SANITY_CITY(pcity, pcity->size >= 1);
 
-  city_tile_iterate_skip_free_cxy(pcenter, ptile, cx, cy) {
+  city_tile_iterate_skip_free_worked(city_map_radius_sq_get(pcity), pcenter,
+                                  ptile, _index, _x, _y) {
     if (tile_worked(ptile) == pcity) {
       citizens++;
     }
-  } city_tile_iterate_skip_free_cxy_end;
+  } city_tile_iterate_skip_free_worked_end;
 
   citizens += city_specialists(pcity);
   delta = pcity->size - citizens;
   if (0 != delta) {
     SANITY_("(%4d,%4d) %d citizens not equal [size], "
-            "repairing \"%s\"[%d]"),
+            "repairing \"%s\"[%d]",
             TILE_XY(pcity->tile),
             citizens,
             city_name(pcity), pcity->size);
 
+    citylog_map_workers(LOG_DEBUG, pcity);
+    log_debug("[%s (%d)] specialists: %d", city_name(pcity), pcity->id,
+              city_specialists(pcity));
+
     city_repair_size(pcity, delta);
-    city_refresh_from_main_map(pcity, TRUE);
+    city_refresh_from_main_map(pcity, NULL);
   }
 }
 
@@ -432,7 +307,8 @@ static void check_city_size(struct city *pcity, const char *file, int line)
   city size.
 **************************************************************************/
 void real_sanity_check_feelings(const struct city *pcity,
-                                const char *file, int line)
+                                const char *file,
+                                const char *function, int line)
 {
   int ccategory;
   int spe = city_specialists(pcity);
@@ -452,11 +328,11 @@ void real_sanity_check_feelings(const struct city *pcity,
 /**************************************************************************
   Verify that the city has sane values.
 **************************************************************************/
-void real_sanity_check_city_all(struct city *pcity, const char *file, int line)
+void real_sanity_check_city_all(struct city *pcity, const char *file,
+                                const char *function, int line)
 {
-  if (check_city_good(pcity, file, line)) {
-    check_city_map(pcity, file, line);
-    check_city_size(pcity, file, line);
+  if (check_city_good(pcity, file, function, line)) {
+    check_city_size(pcity, file, function, line);
     /* TODO: check_feelings. Currently fails far too often to be included. */
   }
 }
@@ -464,55 +340,33 @@ void real_sanity_check_city_all(struct city *pcity, const char *file, int line)
 /**************************************************************************
   Verify that the city has sane values.
 **************************************************************************/
-void real_sanity_check_city(struct city *pcity, const char *file, int line)
+void real_sanity_check_city(struct city *pcity, const char *file,
+                            const char *function, int line)
 {
-  if (check_city_good(pcity, file, line)) {
-    check_city_size(pcity, file, line);
+  if (check_city_good(pcity, file, function, line)) {
+    check_city_size(pcity, file, function, line);
   }
 }
 
 /**************************************************************************
   Sanity checks on all cities in the world.
 **************************************************************************/
-static void check_cities(const char *file, int line)
+static void check_cities(const char *file, const char *function, int line)
 {
   players_iterate(pplayer) {
     city_list_iterate(pplayer->cities, pcity) {
       SANITY_CITY(pcity, city_owner(pcity) == pplayer);
 
-      real_sanity_check_city(pcity, file, line);
+      real_sanity_check_city(pcity, file, function, line);
     } city_list_iterate_end;
   } players_iterate_end;
-
-#ifdef DOUBLE_CHECK_TILE_WORKER
-  whole_map_iterate(ptile) {
-    struct city *pwork = tile_worked(ptile);
-
-    if (NULL != pwork) {
-      int city_x, city_y;
-      bool is_valid = city_base_to_city_map(&city_x, &city_y, pwork, ptile);
-
-      SANITY_TILE(ptile, is_valid);
-
-      if (pwork->city_map[city_x][city_y] != C_TILE_WORKER) {
-	SANITY_("(%4d,%4d) marked as worked, "
-		"but its {%d,%d} has status %d in "
-		"\"%s\"[%d]%s"),
-		TILE_XY(ptile),
-		city_x, city_y,
-		pwork->city_map[city_x][city_y],
-		city_name(pwork), pwork->size,
-		is_city_center(pwork, ptile) ? "{city center}" : "");
-      }
-    }
-  } whole_map_iterate_end;
-#endif
 }
 
 /**************************************************************************
   Sanity checks on all units in the world.
 **************************************************************************/
-static void check_units(const char *file, int line) {
+static void check_units(const char *file, const char *function, int line)
+{
   players_iterate(pplayer) {
     unit_list_iterate(pplayer->units, punit) {
       struct tile *ptile = punit->tile;
@@ -523,7 +377,8 @@ static void check_units(const char *file, int line) {
       SANITY_CHECK(unit_owner(punit) == pplayer);
 
       if (IDENTITY_NUMBER_ZERO != punit->homecity) {
-	SANITY_CHECK(phome = player_find_city_by_id(pplayer, punit->homecity));
+        SANITY_CHECK(phome = player_city_by_number(pplayer,
+                                                   punit->homecity));
 	if (phome) {
 	  SANITY_CHECK(city_owner(phome) == pplayer);
 	}
@@ -531,7 +386,7 @@ static void check_units(const char *file, int line) {
 
       if (!can_unit_continue_current_activity(punit)) {
 	SANITY_("(%4d,%4d) %s has activity %s, "
-		"but it can't continue at %s"),
+		"but it can't continue at %s",
 		TILE_XY(ptile),
 		unit_rule_name(punit),
 		get_activity_text(punit->activity),
@@ -547,7 +402,7 @@ static void check_units(const char *file, int line) {
       SANITY_CHECK(punit->hp > 0);
 
       if (punit->transported_by != -1) {
-        transporter = game_find_unit_by_number(punit->transported_by);
+        transporter = game_unit_by_number(punit->transported_by);
         SANITY_CHECK(transporter != NULL);
 
 	/* Make sure the transporter is on the tile. */
@@ -559,9 +414,11 @@ static void check_units(const char *file, int line) {
 	SANITY_CHECK(transporter2 != NULL);
 
         /* Also in the list of owner? */
-        SANITY_CHECK(player_find_unit_by_id(unit_owner(transporter),
-				      punit->transported_by) != NULL);
-        SANITY_CHECK(same_pos(ptile, transporter->tile));
+        if (NULL != transporter) {
+          SANITY_CHECK(player_unit_by_number(unit_owner(transporter),
+                                             punit->transported_by) != NULL);
+          SANITY_CHECK(same_pos(ptile, transporter->tile));
+        }
 
         /* Transporter capacity will be checked when transporter itself
 	 * is checked */
@@ -584,21 +441,22 @@ static void check_units(const char *file, int line) {
 /**************************************************************************
   Sanity checks on all players.
 **************************************************************************/
-static void check_players(const char *file, int line)
+static void check_players(const char *file, const char *function, int line)
 {
-  int player_no;
-
   players_iterate(pplayer) {
-    int one = player_index(pplayer);
     int found_palace = 0;
 
     if (!pplayer->is_alive) {
-      /* Don't do these checks.  Note there are some dead-players
-       * sanity checks below. */
+      /* Dead players' units and cities are disbanded in kill_player(). */
+      SANITY_CHECK(unit_list_size(pplayer->units) == 0);
+      SANITY_CHECK(city_list_size(pplayer->cities) == 0);
+
       continue;
     }
 
+    SANITY_CHECK(pplayer->server.aidata != NULL);
     SANITY_CHECK(!pplayer->nation || pplayer->nation->player == pplayer);
+    SANITY_CHECK(player_list_search(team_members(pplayer->team), pplayer));
 
     city_list_iterate(pplayer->cities, pcity) {
       if (is_capital(pcity)) {
@@ -608,24 +466,27 @@ static void check_players(const char *file, int line)
     } city_list_iterate_end;
 
     players_iterate(pplayer2) {
-      int two = player_index(pplayer2);
-      SANITY_CHECK(pplayer->diplstates[two].type
-	     == pplayer2->diplstates[one].type);
-      if (pplayer->diplstates[two].type == DS_CEASEFIRE) {
-	SANITY_CHECK(pplayer->diplstates[two].turns_left
-	       == pplayer2->diplstates[one].turns_left);
+      SANITY_CHECK(player_diplstate_get(pplayer, pplayer2)->type
+                   == player_diplstate_get(pplayer2, pplayer)->type);
+      if (player_diplstate_get(pplayer, pplayer2)->type == DS_CEASEFIRE) {
+        SANITY_CHECK(player_diplstate_get(pplayer, pplayer2)->turns_left
+                     == player_diplstate_get(pplayer2, pplayer)->turns_left);
       }
       if (pplayer->is_alive
           && pplayer2->is_alive
           && pplayers_allied(pplayer, pplayer2)) {
-        SANITY_CHECK(pplayer_can_make_treaty(pplayer, pplayer2, DS_ALLIANCE)
-                     != DIPL_ALLIANCE_PROBLEM);
+        enum dipl_reason allied_players_can_be_allied =
+          pplayer_can_make_treaty(pplayer, pplayer2, DS_ALLIANCE);
+        SANITY_CHECK(allied_players_can_be_allied
+                     != DIPL_ALLIANCE_PROBLEM_US);
+        SANITY_CHECK(allied_players_can_be_allied
+                     != DIPL_ALLIANCE_PROBLEM_THEM);
       }
     } players_iterate_end;
 
     if (pplayer->revolution_finishes == -1) {
       if (government_of_player(pplayer) == game.government_during_revolution) {
-        SANITY_("%s government is anarchy, but does not finish!"),
+        SANITY_("%s government is anarchy, but does not finish!",
                 nation_rule_name(nation_of_player(pplayer)));
       }
       SANITY_CHECK(government_of_player(pplayer) != game.government_during_revolution);
@@ -635,33 +496,28 @@ static void check_players(const char *file, int line)
       /* Things may vary in this case depending on when the sanity_check
        * call is made.  No better check is possible. */
     }
-  } players_iterate_end;
-
-  /* Sanity checks on living and dead players. */
-  for (player_no = 0; player_no < ARRAY_SIZE(game.players); player_no++) {
-    struct player *pplayer = &game.players[player_no];
-
-    if (!pplayer->is_alive) {
-      /* Dead players' units and cities are disbanded in kill_player(). */
-      SANITY_CHECK(unit_list_size(pplayer->units) == 0);
-      SANITY_CHECK(city_list_size(pplayer->cities) == 0);
-    }
 
     /* Dying players shouldn't be left around.  But they are. */
-    SANITY_CHECK(!pplayer->is_dying);
-  }
+    SANITY_CHECK(!BV_ISSET(pplayer->server.status, PSTATUS_DYING));
+  } players_iterate_end;
 
   nations_iterate(pnation) {
     SANITY_CHECK(!pnation->player || pnation->player->nation == pnation);
   } nations_iterate_end;
+
+  teams_iterate(pteam) {
+    player_list_iterate(team_members(pteam), pplayer) {
+      SANITY_CHECK(pplayer->team == pteam);
+    } player_list_iterate_end;
+  } teams_iterate_end;
 }
 
 /****************************************************************************
   Sanity checking on teams.
 ****************************************************************************/
-static void check_teams(const char *file, int line)
+static void check_teams(const char *file, const char *function, int line)
 {
-  int count[MAX_NUM_TEAMS], i;
+  int count[MAX_NUM_TEAM_SLOTS];
 
   memset(count, 0, sizeof(count));
   players_iterate(pplayer) {
@@ -672,15 +528,21 @@ static void check_teams(const char *file, int line)
     }
   } players_iterate_end;
 
-  for (i = 0; i < MAX_NUM_TEAMS; i++) {
-    SANITY_CHECK(team_by_number(i)->players == count[i]);
-  }
+  team_slots_iterate(tslot) {
+    if (team_slot_is_used(tslot)) {
+      struct team *pteam = team_slot_get_team(tslot);
+      fc_assert_exit(pteam);
+      SANITY_CHECK(player_list_size(team_members(pteam))
+                   == count[team_slot_index(tslot)]);
+    }
+  } team_slots_iterate_end;
 }
 
 /**************************************************************************
   Sanity checking on connections.
 **************************************************************************/
-static void check_connections(const char *file, int line)
+static void check_connections(const char *file, const char *function,
+                              int line)
 {
   /* est_connections is a subset of all_connections */
   SANITY_CHECK(conn_list_size(game.all_connections)
@@ -696,21 +558,43 @@ static void check_connections(const char *file, int line)
   can't call it in the middle of an operation that is supposed to be
   atomic.
 **************************************************************************/
-void real_sanity_check(const char *file, int line)
+void real_sanity_check(const char *file, const char *function, int line)
 {
   if (!map_is_empty()) {
     /* Don't sanity-check the map if it hasn't been created yet (this
      * happens when loading scenarios). */
-    check_specials(file, line);
-    check_map(file, line);
-    check_cities(file, line);
-    check_units(file, line);
-    check_fow(file, line);
+    check_specials(file, function, line);
+    check_map(file, function, line);
+    check_cities(file, function, line);
+    check_units(file, function, line);
+    check_fow(file, function, line);
   }
-  check_misc(file, line);
-  check_players(file, line);
-  check_teams(file, line);
-  check_connections(file, line);
+  check_misc(file, function, line);
+  check_players(file, function, line);
+  check_teams(file, function, line);
+  check_connections(file, function, line);
+}
+
+/*****************************************************************************
+  Verify that the tile has sane values. This should be called after the
+  terrain is changed.
+*****************************************************************************/
+void real_sanity_check_tile(struct tile *ptile, const char *file,
+                            const char *function, int line)
+{
+  SANITY_CHECK(ptile != NULL);
+  SANITY_CHECK(ptile->terrain != NULL);
+
+  unit_list_iterate(ptile->units, punit) {
+    /* Check if the units can survive on the tile (terrain). Here only the
+     * 'easy' test if the unit is transported is done. A complete check is
+     * done by check_units() in real_sanity_check(). */
+    if (!can_unit_exist_at_tile(punit, ptile)
+        && punit->transported_by != -1) {
+      SANITY_("(%4d,%4d) %s can't survive on %s", TILE_XY(ptile),
+              unit_rule_name(punit), tile_get_info_text(ptile, 0));
+    }
+  } unit_list_iterate_end;
 }
 
 #endif /* SANITY_CHECKING */
