@@ -885,6 +885,12 @@ static void begin_phase(bool is_new_phase)
 
   dlsend_packet_start_phase(game.est_connections, game.info.phase);
 
+  /* Must be the first thing as it is needed for lots of functions below! */
+  phase_players_iterate(pplayer) {
+    /* human players also need this for building advice */
+    ai_data_phase_init(pplayer, is_new_phase);
+  } phase_players_iterate_end;
+
   if (is_new_phase) {
     /* Unit "end of turn" activities - of course these actually go at
      * the start of the turn! */
@@ -897,8 +903,6 @@ static void begin_phase(bool is_new_phase)
   phase_players_iterate(pplayer) {
     log_debug("beginning player turn for #%d (%s)",
               player_number(pplayer), player_name(pplayer));
-    /* human players also need this for building advice */
-    ai_data_phase_init(pplayer, is_new_phase);
     if (!pplayer->ai_controlled) {
       building_advisor(pplayer);
     }
@@ -1285,6 +1289,12 @@ void start_game(void)
   /* Prevent problems with commands that only make sense in pregame. */
   clear_all_votes();
 
+  /* This value defines if the player data should be saved for a scenario. It
+   * is only FALSE if the editor was used to set it to this value. For
+   * such scenarios it has to be resetted at game start so that player data
+   * is saved. */
+  game.scenario.players = TRUE;
+
   force_end_of_sniff = TRUE;
   /* There's no stateful packet set to client until srv_ready(). */
 }
@@ -2593,6 +2603,7 @@ static void fc_interface_init_server(void)
 {
   struct functions *funcs = fc_interface_funcs();
 
+  funcs->destroy_base = destroy_base;
   funcs->player_tile_vision_get = map_is_known_and_seen;
 
   /* Keep this function call at the end. It checks if all required functions
