@@ -15,8 +15,8 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <errno.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -57,7 +57,7 @@ static const char *transliteration_string;
 
   Pass an internal encoding of NULL to use the local encoding internally.
 ***************************************************************************/
-void init_character_encodings(char *my_internal_encoding,
+void init_character_encodings(const char *my_internal_encoding,
 			      bool my_use_transliteration)
 {
 #ifdef HAVE_ICONV
@@ -88,15 +88,15 @@ void init_character_encodings(char *my_internal_encoding,
     local_encoding = "";
 #endif
 #endif
-    if (mystrcasecmp(local_encoding, "ANSI_X3.4-1968") == 0
-	|| mystrcasecmp(local_encoding, "ASCII") == 0
-	|| mystrcasecmp(local_encoding, "US-ASCII") == 0) {
+    if (fc_strcasecmp(local_encoding, "ANSI_X3.4-1968") == 0
+        || fc_strcasecmp(local_encoding, "ASCII") == 0
+        || fc_strcasecmp(local_encoding, "US-ASCII") == 0) {
       /* HACK: use latin1 instead of ascii in typical cases when the
        * encoding is unconfigured. */
       local_encoding = "ISO-8859-1";
     }
 
-    if (mystrcasecmp(local_encoding, "646") == 0) {
+    if (fc_strcasecmp(local_encoding, "646") == 0) {
       /* HACK: On Solaris the encoding always comes up as "646" (ascii),
        * which iconv doesn't understand.  Work around it by using UTF-8
        * instead. */
@@ -121,13 +121,17 @@ void init_character_encodings(char *my_internal_encoding,
 #endif
 
 #ifdef DEBUG
-  /* FIXME: Remove this output when this code has stabilized. */
   fprintf(stderr, "Encodings: Data=%s, Local=%s, Internal=%s\n",
-	     data_encoding, local_encoding, internal_encoding);
+          data_encoding, local_encoding, internal_encoding);
 #endif
 
 #else
-#  error No iconv present!
+   /* log_* may not work at this point. */
+  fprintf(stderr,
+          _("You are running Freeciv without using iconv. Unless\n"
+            "you are using the latin1 character set, some characters\n"
+            "may not be displayed properly. You can download iconv\n"
+            "at http://gnu.org/.\n"));
 #endif
 
   is_init = TRUE;
@@ -138,7 +142,7 @@ void init_character_encodings(char *my_internal_encoding,
 ***************************************************************************/
 const char *get_data_encoding(void)
 {
-  assert(is_init);
+  fc_assert_ret_val(is_init, NULL);
   return data_encoding;
 }
 
@@ -148,7 +152,7 @@ const char *get_data_encoding(void)
 const char *get_local_encoding(void)
 {
 #ifdef HAVE_ICONV
-  assert(is_init);
+  fc_assert_ret_val(is_init, NULL);
   return local_encoding;
 #else
 #  ifdef HAVE_LIBCHARSET
@@ -169,7 +173,7 @@ const char *get_local_encoding(void)
 ***************************************************************************/
 const char *get_internal_encoding(void)
 {
-  assert(is_init);
+  fc_assert_ret_val(is_init, NULL);
   return internal_encoding;
 }
 
@@ -177,29 +181,33 @@ const char *get_internal_encoding(void)
   Convert the text.  Both 'from' and 'to' must be 8-bit charsets.  The
   result will be put into the buf buffer unless it is NULL, in which case it
   will be allocated on demand.
+
+  Don't use this function if you can avoid it.  Use one of the
+  xxx_to_yyy_string functions.
 ***************************************************************************/
-static char *convert_string(const char *text,
-			    const char *from,
-			    const char *to,
-			    char *buf, size_t bufsz)
+char *convert_string(const char *text,
+		     const char *from,
+		     const char *to,
+		     char *buf, size_t bufsz)
 {
 #ifdef HAVE_ICONV
   iconv_t cd = iconv_open(to, from);
   size_t from_len = strlen(text) + 1, to_len;
   bool alloc = (buf == NULL);
 
-  assert(is_init && from != NULL && to != NULL);
-  assert(text != NULL);
+  fc_assert_ret_val(is_init && NULL != from && NULL != to, NULL);
+  fc_assert_ret_val(NULL != text, NULL);
 
   if (cd == (iconv_t) (-1)) {
-    freelog(LOG_ERROR,
-	    _("Could not convert text from %s to %s: %s"),
-	    from, to, strerror(errno));
+    /* TRANS: "Could not convert text from <encoding a> to <encoding b>:" 
+     *        <externally translated error string>."*/
+    log_error(_("Could not convert text from %s to %s: %s"),
+              from, to, fc_strerror(fc_get_errno()));
     /* The best we can do? */
     if (alloc) {
-      return mystrdup(text);
+      return fc_strdup(text);
     } else {
-      my_snprintf(buf, bufsz, "%s", text);
+      fc_snprintf(buf, bufsz, "%s", text);
       return buf;
     }
   }
@@ -228,17 +236,16 @@ static char *convert_string(const char *text,
     res = iconv(cd, (ICONV_CONST char **)&mytext, &flen, &myresult, &tlen);
     if (res == (size_t) (-1)) {
       if (errno != E2BIG) {
-	/* Invalid input. */
-	freelog(LOG_ERROR, "Invalid string conversion from %s to %s.",
-		from, to);
-	iconv_close(cd);
-	if (alloc) {
-	  free(buf);
-	  return mystrdup(text); /* The best we can do? */
-	} else {
-	  my_snprintf(buf, bufsz, "%s", text);
-	  return buf;
-	}
+        /* Invalid input. */
+        log_error("Invalid string conversion from %s to %s.", from, to);
+        iconv_close(cd);
+        if (alloc) {
+          free(buf);
+          return fc_strdup(text); /* The best we can do? */
+        } else {
+          fc_snprintf(buf, bufsz, "%s", text);
+          return buf;
+        }
       }
     } else {
       /* Success. */
@@ -252,7 +259,7 @@ static char *convert_string(const char *text,
     if (alloc) {
       /* Not enough space; try again. */
       buf[to_len - 1] = 0;
-      freelog(LOG_VERBOSE, "   Result was '%s'.", buf);
+      log_verbose("   Result was '%s'.", buf);
 
       free(buf);
       to_len *= 2;
@@ -266,7 +273,7 @@ static char *convert_string(const char *text,
     buf[bufsz - 1] = '\0';
     return buf;
   } else {
-    return mystrdup(text);
+    return fc_strdup(text);
   }
 #endif /* HAVE_ICONV */
 }
@@ -277,7 +284,7 @@ char *src ## _to_ ## dst ## _string_malloc(const char *text)                \
   const char *encoding1 = (dst ## _encoding);				    \
   char encoding[strlen(encoding1) + strlen(transliteration_string) + 1];    \
 									    \
-  my_snprintf(encoding, sizeof(encoding),				    \
+  fc_snprintf(encoding, sizeof(encoding),				    \
 	      "%s%s", encoding1, transliteration_string);		    \
   return convert_string(text, (src ## _encoding),			    \
 			(encoding), NULL, 0);				    \
@@ -290,7 +297,7 @@ char *src ## _to_ ## dst ## _string_buffer(const char *text,                \
   const char *encoding1 = (dst ## _encoding);				    \
   char encoding[strlen(encoding1) + strlen(transliteration_string) + 1];    \
 									    \
-  my_snprintf(encoding, sizeof(encoding),				    \
+  fc_snprintf(encoding, sizeof(encoding),				    \
 	      "%s%s", encoding1, transliteration_string);		    \
   return convert_string(text, (src ## _encoding),			    \
                         encoding, buf, bufsz);				    \
@@ -311,8 +318,8 @@ CONV_FUNC_MALLOC(internal, local)
 CONV_FUNC_MALLOC(local, internal)
 
 CONV_FUNC_BUFFER(local, internal)
+CONV_FUNC_BUFFER(internal, local)
 
-static CONV_FUNC_BUFFER(internal, local)
 static CONV_FUNC_STATIC(internal, local)
 
 /***************************************************************************
@@ -326,7 +333,7 @@ void fc_fprintf(FILE *stream, const char *format, ...)
   static bool recursion = FALSE;
 
   /* The recursion variable is used to prevent a recursive loop.  If
-   * an iconv conversion fails, then freelog will be called and an
+   * an iconv conversion fails, then log_* will be called and an
    * fc_fprintf will be done.  But below we do another iconv conversion
    * on the error messages, which is of course likely to fail also. */
   if (recursion) {
@@ -334,7 +341,7 @@ void fc_fprintf(FILE *stream, const char *format, ...)
   }
 
   va_start(ap, format);
-  my_vsnprintf(string, sizeof(string), format, ap);
+  fc_vsnprintf(string, sizeof(string), format, ap);
   va_end(ap);
 
   recursion = TRUE;
@@ -348,3 +355,29 @@ void fc_fprintf(FILE *stream, const char *format, ...)
   fputs(output, stream);
   fflush(stream);
 }
+
+/****************************************************************************
+  Return the length, in *characters*, of the string.  This can be used in
+  place of strlen in some places because it returns the number of characters
+  not the number of bytes (with multi-byte characters in UTF-8, the two
+  may not be the same).
+
+  Use of this function outside of GUI layout code is probably a hack.  For
+  instance the demographics code uses it, but this should instead pass the
+  data directly to the GUI library for formatting.
+****************************************************************************/
+size_t get_internal_string_length(const char *text)
+{
+  int text2[(strlen(text) + 1)]; /* UCS-4 text */
+  int i = 0;
+
+  convert_string(text, internal_encoding, "UCS-4",
+                 (char *)text2, sizeof(text2));
+  /* Check BOM */
+  fc_assert_ret_val(0x0000FEFF != text2[0]&& 0xFFFE0000!= text2[0], -1);
+  for (i = 0; ; i++) {
+    if (text2[i] == 0) {
+      return i;
+    }
+  }
+}
