@@ -20,11 +20,12 @@
 #define FC__INPUTFILE_H
 
 #include "ioz.h"
-#include "shared.h"		/* bool type */
+#include "log.h"                /* enum log_level */
+#include "support.h"            /* bool type and fc__attribute */
 
 struct inputfile;		/* opaque */
 
-typedef char *(*datafilename_fn_t)(const char *filename);
+typedef const char *(*datafilename_fn_t)(const char *filename);
 
 struct inputfile *inf_from_file(const char *filename,
 				datafilename_fn_t datafn);
@@ -48,6 +49,7 @@ enum inf_token_type {
 const char *inf_token(struct inputfile *inf, enum inf_token_type type);
 int inf_discard_tokens(struct inputfile *inf, enum inf_token_type type);
 
-void inf_log(struct inputfile *inf, int loglevel, const char *message);
+char *inf_log_str(struct inputfile *inf, const char *message, ...)
+                  fc__attribute((__format__ (__printf__, 2, 3)));
 
 #endif  /* FC__INPUTFILE_H */
